@@ -3,7 +3,7 @@
 // Purpose:     Burn dialog
 // Author:      Alex Thuering
 // Created:     23.03.2003
-// RCS-ID:      $Id: BurnDlg.cpp,v 1.33 2011/07/31 13:25:02 ntalex Exp $
+// RCS-ID:      $Id: BurnDlg.cpp,v 1.34 2012/06/17 21:56:20 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -180,7 +180,7 @@ bool IsDvdWriter(HANDLE hDevice) {
 	ULONG length = offsetof(SCSI_PASS_THROUGH_WITH_BUFFERS, DataBuf) + sptwb.Spt.DataTransferLength;
 	ULONG returned = 0;
 	bool status = DeviceIoControl(hDevice, IOCTL_SCSI_PASS_THROUGH, &sptwb,
-			sizeof(SCSI_PASS_THROUGH), &sptwb, length, &returned, false);
+			sizeof(SCSI_PASS_THROUGH), &sptwb, length, &returned, NULL);
 	if (status && sptwb.Spt.ScsiStatus == 0) {
 		return (sptwb.DataBuf[7] & 0x10) || (sptwb.DataBuf[7] & 0x20);
 	}
