@@ -3,7 +3,7 @@
 // Purpose:     Configuration
 // Author:      Alex Thuering
 // Created:     27.03.2003
-// RCS-ID:      $Id: Config.h,v 1.70 2011/12/19 21:27:11 ntalex Exp $
+// RCS-ID:      $Id: Config.h,v 1.81 2012/11/05 21:13:14 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -20,15 +20,16 @@ const wxString DEF_VERSION		= wxT("");
 const int DEF_LANGUAGE			= wxLANGUAGE_UNKNOWN;
 const wxString DEF_LANGUAGE_CODE= wxT("");
 const wxString DEF_DISC_LABEL	= wxT("DVD");
-const int DEF_DISC_CAPACITY		= 0;
+const int DEF_DISC_CAPACITY		= 2;
+const int DEF_POST_COMMAND		= 1;
 const int DEF_VIDEO_FORMAT		= 2;
 const int DEF_AUDIO_FORMAT		= 3;
-const wxString DEF_AUDIO_LANG	= wxT("EN");
-const wxString DEF_SUBTITLE_LANG= wxT("EN");
+const wxString DEF_AUDIO_LANG	= wxT("");
+const wxString DEF_SUBTITLE_LANG= wxT("");
 const int DEF_SPECT_RATIO		= 1;
 const int DEF_CHAPTER_LENGTH	= 10;
-const wxString DEF_BUTTON		= wxT("frame.xml");
-const wxString DEF_TEXT_BUTTON	= wxT("text-v2.xml");
+const wxString DEF_BUTTON		= wxT("frame-v2.xml");
+const wxString DEF_TEXT_BUTTON	= wxT("text-v3.xml");
 const bool DEF_ACCEPT_INVALID_ACTIONS = false;
 const wxString DEF_FILE_BROWSER_DIR = wxT("");
 #define DEF_FILE_BROWSER_LAST_DIR wxGetHomeDir()
@@ -44,7 +45,7 @@ const int DEF_MENU_VIDEO_BITRATE = 6000;
 const bool DEF_DRAW_BUTTONS_ON_BACKGROUND = true;
 const int DEF_SLIDESHOW_VIDEO_BITRATE = 6000;
 const int DEF_THREAD_COUNT = wxThread::GetCPUCount() > 0 ? wxThread::GetCPUCount() : 1;
-const int DEF_DVD_RESERVED_SPACE = 80000; // KB
+const int DEF_DVD_RESERVED_SPACE = 0; // KB
 #define DEF_GEN_TEMP_DIR wxFileName::GetTempDir()
 #ifdef __WXMSW__
 	const bool DEF_PREVIEW_DO        = false;
@@ -59,6 +60,11 @@ const int DEF_DVD_RESERVED_SPACE = 80000; // KB
 #endif
 const bool DEF_USE_MPLEX = false;
 const bool DEF_USE_MPLEX_FOR_MENUS = true;
+#ifdef USE_FFMPEG
+const wxString DEF_AVCONV_CMD    = _T("ffmpeg");
+#else
+const wxString DEF_AVCONV_CMD    = _T("avconv");
+#endif
 const wxString DEF_MPLEX_CMD     = _T("mplex -f 8 -S 0 -M -V -o \"$FILE_OUT\" \"$FILE_VIDEO\" \"$FILE_AUDIO\"");
 const wxString DEF_SPUMUX_CMD    = _T("spumux -P -s $STREAM \"$FILE_CONF\"");
 const wxString DEF_DVDAUTHOR_CMD = _T("dvdauthor -o \"$DIR\" -x \"$FILE_CONF\"");
@@ -68,8 +74,8 @@ const wxString DEF_BURN_SCAN_CMD = _T("dvd+rw-mediainfo $DEVICE");
 const wxString DEF_BURN_CMD      = _T("growisofs -V \"$VOL_ID\" -dvd-compat -Z $DEV -dvd-video \"$DIR\" $SPEEDSTR");
 const wxString DEF_BURN_SPEED_OPT= _T("-speed=$SPEED");
 #ifdef __WXMAC__
-const wxString DEF_BURN_ISO_CMD  = _T("/usr/bin/hdiutil burn -device $DEV $SPEEDSTR \"$FILE\"");
-const wxString DEF_FORMAT_CMD    = _T("/usr/bin/hdiutil burn -device $DEV -erase");
+const wxString DEF_BURN_ISO_CMD  = _T("/usr/bin/hdiutil burn -device \"$DEV\" $SPEEDSTR \"$FILE\"");
+const wxString DEF_FORMAT_CMD    = _T("/usr/bin/hdiutil burn -device \"$DEV\" -erase");
 #else
 const wxString DEF_BURN_ISO_CMD  = _T("growisofs -dvd-compat -Z $DEV=\"$FILE\" -use-the-force-luke=dao:$SIZE $SPEEDSTR");
 const wxString DEF_FORMAT_CMD    = _T("dvd+rw-format -force $DEV");
@@ -81,7 +87,6 @@ const bool DEF_FORMAT_DO         = false;
 #define DEF_ISO_SAVETO (wxGetHomeDir() + wxFILE_SEP_PATH)
 const int DEF_BURN_SPEED         = 0;
 const bool DEF_ADD_ECC_DO        = false;
-const int DEF_KEY = 0;
 
 #define CONFIG_PROP(name, cfgName, defValue)\
   CONFIG_PROP_T(name, cfgName, defValue, wxString, wxString)
@@ -112,10 +117,10 @@ public:
     wxConfigBase* GetConfigBase() { return cfg; }
 	
 	CONFIG_PROP(Version,  _T("Version"), DEF_VERSION)
-	CONFIG_PROP_INT(Language, _T("Interface/Language"), DEF_LANGUAGE)
-	CONFIG_PROP(LanguageCdoe, _T("Interface/LanguageCode"), DEF_LANGUAGE_CODE)
+	CONFIG_PROP(LanguageCode, _T("Interface/LanguageCode"), DEF_LANGUAGE_CODE)
     CONFIG_PROP(DefDiscLabel, _T("Interface/DefDiscLabel"), DEF_DISC_LABEL)
     CONFIG_PROP_INT(DefDiscCapacity, _T("Interface/DefDiscCapacity"), DEF_DISC_CAPACITY)
+    CONFIG_PROP_INT(DefPostCommand, _T("Interface/DefPostCommand"), DEF_POST_COMMAND)
 	
 	CONFIG_PROP_INT(DefVideoFormat, _T("Interface/DefVideoFormat"), DEF_VIDEO_FORMAT)
 	CONFIG_PROP_INT(DefAudioFormat, _T("Interface/DefAudioFormat"), DEF_AUDIO_FORMAT)
@@ -148,8 +153,9 @@ public:
     CONFIG_PROP_INT(DvdReservedSpace, _T("Generate/DvdReservedSpace"), DEF_DVD_RESERVED_SPACE)
     CONFIG_PROP_BOOL(UseMplex, _T("Generate/UseMplex"), DEF_USE_MPLEX)
     CONFIG_PROP_BOOL(UseMplexForMenus, _T("Generate/UseMplexForMenus"), DEF_USE_MPLEX_FOR_MENUS)
-    CONFIG_PROP(MplexCmd,   _T("Generate/MplexCmd"), DEF_MPLEX_CMD)
-	CONFIG_PROP(SpumuxCmd,  _T("Generate/SpumuxCmd"), DEF_SPUMUX_CMD)
+    CONFIG_PROP(AVConvCmd, _T("Generate/AVConvCmd"), DEF_AVCONV_CMD)
+    CONFIG_PROP(MplexCmd, _T("Generate/MplexCmd"), DEF_MPLEX_CMD)
+	CONFIG_PROP(SpumuxCmd, _T("Generate/SpumuxCmd"), DEF_SPUMUX_CMD)
 	CONFIG_PROP(DvdauthorCmd, _T("Generate/DvdauthorCmd"), DEF_DVDAUTHOR_CMD)
 	
 	CONFIG_PROP_BOOL(PreviewDo, _T("Preview/Do"), DEF_PREVIEW_DO)
@@ -176,8 +182,6 @@ public:
 	CONFIG_PROP_BOOL(AddECCDo, _T("AddECC/Do"), DEF_ADD_ECC_DO)
 	CONFIG_PROP(AddECCCmd, _T("AddECC/Cmd"), DEF_ADD_ECC_CMD)
 	
-    CONFIG_PROP_INT(Key, _T("Burn/Key"), DEF_KEY)
-	
   protected:
     wxConfigBase* cfg;
 };
