@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
 // Created:     29.01.2003
-// RCS-ID:      $Id: DVD.cpp,v 1.106 2011/09/11 18:49:55 ntalex Exp $
+// RCS-ID:      $Id: DVD.cpp,v 1.117 2012/07/22 09:32:04 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -24,7 +24,7 @@
 
 DVD::DVD() {
 	m_label = wxT("DVD");
-	m_capacity = dcDVD47;
+	m_capacity = dcDVD5;
 	m_videoBitrateAuto = true;
 	m_videoBitrate = 4500;
 	m_jumppad = false;
@@ -35,6 +35,7 @@ DVD::DVD() {
 	m_videoFormat = vfPAL;
 	m_audioFormat = afMP2;
 	m_aspectRatio = ar4_3;
+	m_defPostCommand = cmdCALL_LAST_MENU;
 }
 
 DVD::~DVD() {
@@ -44,9 +45,13 @@ DVD::~DVD() {
 
 /** Returns disc capacity in KB */
 long DVD::GetCapacityValue() {
-	if (m_capacity == dcDVD47)
+	if (m_capacity == dcDVD1)
+		return 1426063;
+	else if (m_capacity == dcDVD2)
+		return 2589982;
+	else if (m_capacity == dcDVD5)
 		return 4590208;
-	else if (m_capacity == dcDVD85)
+	else if (m_capacity == dcDVD9)
 		return 8343424;
 	return 0;
 }
@@ -88,10 +93,10 @@ wxArrayInt DVD::CalculateVideoBitrate() {
 					}
 				}
 				if (videoFormat != vfCOPY) {
-					duration += (long) vob->GetDuration();
+					duration += lround(vob->GetDuration());
 					size += vob->GetSize(this);
 				} else { 
-					fixDuration += (long) vob->GetDuration();
+					fixDuration += lround(vob->GetDuration());
 					fixSize += vob->GetSize(this);
 				}
 			}
@@ -100,6 +105,7 @@ wxArrayInt DVD::CalculateVideoBitrate() {
 	long bitrate = GetVideoBitrate();
 	if (m_videoBitrateAuto) {
 		if (GetCapacity() != dcUNLIMITED) {
+			// size - size of video by bitrate 500 + size of audio streams
 			long s = GetCapacityValue() - size - fixSize;
 			bitrate = 500 + (s > 0 && duration > 0 ? s/duration*8 : (duration > 0 ? 0 : 7500));
 			if (bitrate > 8000)
@@ -159,6 +165,8 @@ int DVD::AddMenu(Menu* menu, int tsi) {
 	Pgc* pgc = new Pgc;
 	pgc->GetVobs().Add(vob);
 	PgcArray& pgcs = GetPgcArray(tsi, true);
+	if (pgcs.GetCount() == 0 || pgcs.GetVideo().GetFormat() <= vfCOPY)
+		pgcs.GetVideo().SetFormat(menu->GetVideoFormat());
 	pgcs.Add(pgc);
 	return pgcs.Count()-1;
 }
@@ -210,6 +218,7 @@ Vob* DVD::GetMenuVob(int tsi, int pgci) {
 	return NULL;
 }
 
+/** Returns menu from titleset tsi und pgc pgci */
 Menu* DVD::GetMenu(int tsi, int pgci) {
 	Vob* vob = GetMenuVob(tsi, pgci);
 	if (vob)
@@ -217,6 +226,16 @@ Menu* DVD::GetMenu(int tsi, int pgci) {
 	return NULL;
 }
 
+/** Returns true if at least one menu exist  */
+bool DVD::HasMenus() {
+	if (GetVmgm().size())
+		return true;
+	for (unsigned int i = 0; i < GetTitlesets().size(); i++)
+		if (GetTitlesets()[i]->GetMenus().size())
+			return true;
+	return false;
+}
+
 /** Returns true if all menus are ok */
 bool DVD::CheckMenus() {
 	for (int tsi = -1; tsi < (int) GetTitlesets().Count(); tsi++) {
@@ -224,16 +243,23 @@ bool DVD::CheckMenus() {
 		for (unsigned int pgci = 0; pgci < pgcs.GetCount(); pgci++) {
 			Menu* menu = pgcs[pgci]->GetMenu();
 			if (menu) {
+				wxString src = DVDAction::GetSourceStr(this, tsi, pgci, true, wxT("")) + wxString(wxT(": "));
+				// check aspect ration and number of buttons
 				if (menu->GetAspectRatio() == ar16_9) {
 					if (pgcs.GetVideo().GetWidescreen() == wtAUTO && menu->GetButtonsCount() > 12) {
-						wxLogError(_("Wide screen DVD menu can contain maximal 12 buttons"));
+						wxLogError(src + _("Wide screen DVD menu can contain maximal 12 buttons"));
 						return false;
 					} else if (menu->GetButtonsCount() > 18) {
-						wxLogError(_("Wide screen DVD menu (nopanscan/noletterbox) can contain maximal 18 buttons"));
+						wxLogError(src + _("Wide screen DVD menu (nopanscan/noletterbox) can contain maximal 18 buttons"));
 						return false;
 					}
 				} else if (menu->GetButtonsCount() > 34) {
-					wxLogError(_("DVD menu can contain maximal 34 buttons"));
+					wxLogError(src + _("DVD menu can contain maximal 34 buttons"));
+					return false;
+				}
+				// check if buttons overlapping
+				if (menu->isButtonsOverlapping()) {
+					wxLogError(src + _("Some buttons are overlapping"));
 					return false;
 				}
 			}
@@ -359,6 +385,26 @@ void DVD::SetAspectRatio(AspectRatio aspectRatio, bool updateMenus) {
 	}
 }
 
+/** Returns default title post command */
+wxString DVD::GetDefPostCommandStr() {
+	if (m_defPostCommand == cmdCALL_LAST_MENU)
+		return wxT("call last menu;");
+	else if (m_defPostCommand == cmdCALL_ROOT_MENU)
+		return GetVmgm().size() > 0 ? wxT("call vmgm menu 1;") : wxT("call menu entry root;");
+	else if (m_defPostCommand == cmdJUMP_NEXT_TITLE)
+		return wxT("jump next title;");
+	return HasMenus() ? wxT("call menu entry root;") : wxT("");
+}
+
+/** Sets default title post command */
+void DVD::SetDefPostCommand(DefaultPostCommand defPostCommand, bool updateTitles) {
+	m_defPostCommand = defPostCommand;
+	if (updateTitles) {
+		for (int tsi = 0; tsi< (int) GetTitlesets().GetCount(); tsi++)
+			GetPgcArray(tsi, false).SetPostCommands(GetDefPostCommandStr());
+	}
+}
+
 /** Loads a project file */
 bool DVD::Open(wxString fname) {
 	wxSvgXmlDocument xml;
@@ -415,7 +461,7 @@ bool DVD::PutXML(const wxSvgXmlDocument& xml) {
 	if (root->GetPropVal(wxT("isoFile"), &val))
 		m_isoFile = val;
 	if (root->GetPropVal(wxT("capacity"), &val) && val.ToLong(&lval))
-		m_capacity = DiscCapacity(lval);
+		m_capacity = DiscCapacity(lval + 2);
 	if (root->GetPropVal(wxT("jumppad"), &val) && val.ToLong(&lval))
 		m_jumppad = int(lval);
 	if (root->GetPropVal(wxT("emptyMenu"), &val) && val.ToLong(&lval))
@@ -430,6 +476,8 @@ bool DVD::PutXML(const wxSvgXmlDocument& xml) {
 		m_videoBitrateAuto = lval == -1;
 		m_videoBitrate = lval == -1 ? 4500 : lval;
 	}
+	if (root->GetPropVal(wxT("defPostCommand"), &val) && val.ToLong(&lval))
+		m_defPostCommand = DefaultPostCommand(lval);
 
 	wxSvgXmlNode* child = root->GetChildren();
 	while (child) {
@@ -486,7 +534,9 @@ wxSvgXmlDocument* DVD::GetXml() {
 	if (m_isoFile.length() > 0)
 		root->AddProperty(wxT("isoFile"), m_isoFile);
 	root->AddProperty(wxT("name"), m_label);
-	root->AddProperty(wxT("capacity"), wxString::Format(wxT("%d"), m_capacity));
+	root->AddProperty(wxT("capacity"), wxString::Format(wxT("%d"), m_capacity - 2));
+	if (m_defPostCommand != cmdCALL_LAST_MENU)
+		root->AddProperty(wxT("defPostCommand"), wxString::Format(wxT("%d"), m_defPostCommand));
 	root->AddProperty(wxT("jumppad"), m_jumppad ? wxT("1") : wxT("0"));
 	root->AddProperty(wxT("emptyMenu"), m_emptyMenu ? wxT("1") : wxT("0"));
 	root->AddProperty(wxT("videoFormat"), wxString::Format(wxT("%d"), m_videoFormat));
@@ -505,10 +555,15 @@ wxSvgXmlDocument* DVD::GetXml() {
 		coloursNode->AddProperty(wxString::Format(wxT("colour%d"), i),
 				wxString::Format(_T("#%02x%02x%02x"), colour.Red(), colour.Green(), colour.Blue()));
 	}
+	root->AddChild(new wxSvgXmlNode(wxSVGXML_COMMENT_NODE, wxT(""), wxT("************** VMGM **************")));
 	root->AddChild(GetVmgm().GetXML(DVDSTYLER_XML, this));
-	for (int i=0; i<(int)m_titlesets.GetCount(); i++)
-		if (m_titlesets[i]->GetMenus().Count() || m_titlesets[i]->GetTitles().Count())
+	for (int i=0; i<(int)m_titlesets.GetCount(); i++) {
+		if (m_titlesets[i]->GetMenus().Count() || m_titlesets[i]->GetTitles().Count()) {
+			root->AddChild(new wxSvgXmlNode(wxSVGXML_COMMENT_NODE, wxT(""),
+					wxString::Format(wxT("************** TITLESET %d **************"), i + 1)));
 			root->AddChild(m_titlesets[i]->GetXML(DVDSTYLER_XML, this));
+		}
+	}
 	xml->SetRoot(root);
 	return xml;
 }
@@ -557,11 +612,11 @@ bool DVD::SaveDVDAuthor(wxString fname) {
 	wxSvgXmlDocument xml;
 	wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("dvdauthor"));
 	root->AddProperty(wxT("jumppad"), m_jumppad ? wxT("1") : wxT("0"));
-	root->AddChild(GetVmgm().GetXML(DVDAUTHOR_XML, this));
+	root->AddChild(GetVmgm().GetXML(DVDAUTHOR_XML, this)); // vmgm
 	for (unsigned int i = 0; i < m_titlesets.GetCount(); i++) {
 		int nextTitleset = i < m_titlesets.GetCount() - 1 && !m_titlesets[i + 1]->IsEmpty() ? i + 2 : -1;
 		if (!m_titlesets[i]->IsEmpty())
-			root->AddChild(m_titlesets[i]->GetXML(DVDAUTHOR_XML, this, nextTitleset));
+			root->AddChild(m_titlesets[i]->GetXML(DVDAUTHOR_XML, this, nextTitleset)); // titleset
 	}
 	xml.SetRoot(root);
 	return xml.Save(fname);
@@ -592,7 +647,7 @@ wxString DVD::GetPath(bool withSep) {
 	return withSep && path.length() > 0 ? path + wxFILE_SEP_PATH : path;
 }
 
-long DVD::GetSize(bool generated) {
+long DVD::GetSize(bool afterTranscoding) {
 	int size = s_config.GetDvdReservedSpace();
 	for (int tsi = -1; tsi<(int)GetTitlesets().Count(); tsi++) {
 		for (int m=0; m<=1; m++) {
@@ -602,8 +657,10 @@ long DVD::GetSize(bool generated) {
 			PgcArray& pgcs = GetPgcArray(tsi, menu);
 			for (int pgci = 0; pgci<(int)pgcs.Count(); pgci++) {
 				Pgc* pgc = pgcs[pgci];
-				for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++)
-					size += pgc->GetVobs()[vobi]->GetSize(this, generated);
+				for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++) {
+					Vob* vob = pgc->GetVobs()[vobi];
+					size += afterTranscoding ? vob->GetTranscodedSize(this) : vob->GetSize(this);
+				}
 			}
 		}
 	}
@@ -747,7 +804,7 @@ wxArrayString DVD::GetVideoFormatLabels(bool copy, bool none, bool menu) {
 	return formats;
 }
 
-wxArrayString DVD::GetAudioFormatLabels(bool copy, bool none) {
+wxArrayString DVD::GetAudioFormatLabels(bool copy, bool none, bool pcm) {
 	wxArrayString formats;
 	if (none)
 		formats.Add(_("None"));
@@ -755,6 +812,8 @@ wxArrayString DVD::GetAudioFormatLabels(bool copy, bool none) {
 		formats.Add(_("Copy"));
 	formats.Add(_("MP2 48 kHz"));
 	formats.Add(_("AC3 48 kHz"));
+	if (pcm)
+		formats.Add(_("PCM 48 kHz"));
 	return formats;
 }
 
@@ -778,12 +837,23 @@ wxArrayString DVD::GetAspectRatioLabels(bool autom) {
 
 wxArrayString DVD::GetCapacityLabels() {
 	wxArrayString labels;
-	labels.Add(_("4.7 GB"));
-	labels.Add(_("8.5 GB"));
+	labels.Add(_("DVD-1 (1.4 GB)"));
+	labels.Add(_("DVD-2 (2.6 GB)"));
+	labels.Add(_("DVD-5 (4.7 GB)"));
+	labels.Add(_("DVD-9 (8.5 GB)"));
 	labels.Add(_("Unlimited"));
 	return labels;
 }
 
+/** Default title post command */
+wxArrayString DVD::GetDefPostCommandLabels() {
+	wxArrayString labels;
+	labels.push_back(_("Call last menu"));
+	labels.push_back(_("Call root menu"));
+	labels.push_back(_("Play next title"));
+	return labels;
+}
+
 wxArrayString DVD::GetVideoBitrateLabels() {
 	wxArrayString labels;
 	labels.Add(_("Auto"));
@@ -808,6 +878,7 @@ wxArrayString DVD::GetAudioFormatNames() {
 	formats.Add(wxT("copy"));
 	formats.Add(wxT("mp2"));
 	formats.Add(wxT("ac3"));
+	formats.Add(wxT("pcm"));
 	return formats;
 }
 
