@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
 // Created:     29.01.2003
-// RCS-ID:      $Id: DVD.h,v 1.72 2011/08/28 10:03:23 ntalex Exp $
+// RCS-ID:      $Id: DVD.h,v 1.77 2012/07/02 21:38:37 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -14,11 +14,19 @@
 #include "Titleset.h"
 
 enum DiscCapacity {
-	dcDVD47 = 0,
-	dcDVD85,
+	dcDVD1 = 0,
+	dcDVD2,
+	dcDVD5,
+	dcDVD9,
 	dcUNLIMITED
 };
 
+enum DefaultPostCommand {
+	cmdCALL_LAST_MENU = 0,
+	cmdCALL_ROOT_MENU,
+	cmdJUMP_NEXT_TITLE
+};
+
 class DVD {
 public:
     DVD();
@@ -95,8 +103,10 @@ public:
 	Vob* GetVob(int tsi, bool isMenu, int pgci, int vobi);
 	/** Returns vob with menu from given titleset and pgc */
 	Vob* GetMenuVob(int tsi, int pgci);
-	/** Returns menu from titleset tsi und pgc pgci */
+	/** Returns menu from titleset tsi und pgc pgci */	
 	Menu* GetMenu(int tsi, int pgci);
+	/** Returns true if at least one menu exist  */
+	bool HasMenus();
 	/** Returns true if all menus are ok */
 	bool CheckMenus();
 	/** Returns true if all actions are valid */
@@ -115,7 +125,7 @@ public:
 	static inline int GetVobi(int id) { return id & 0xFF; }
 	
 	/** Returns estimated size of DVD in KB */
-	long GetSize(bool generated = false);
+	long GetSize(bool afterTranscoding = false);
 	/** Returns size required for generation of DVD in KB */
 	long GetRequiredSize(Cache* cache);
 	
@@ -140,6 +150,12 @@ public:
 	inline AspectRatio GetAspectRatio() { return m_aspectRatio; }
 	/** Sets aspect ratio of DVD */
 	void SetAspectRatio(AspectRatio format, bool updateMenus = false);
+    /** Returns default title post command */
+    DefaultPostCommand GetDefPostCommand() { return m_defPostCommand; }
+    /** Returns default title post command as string */
+    wxString GetDefPostCommandStr();
+    /** Sets default title post command */
+    void SetDefPostCommand(DefaultPostCommand defPostCommand, bool updateTitles = false);
 	
     /** Loads a project file */
 	bool Open(wxString fname);
@@ -168,10 +184,11 @@ public:
 	void RenderThumbnail(wxString fname);
 	
 	static wxArrayString GetVideoFormatLabels(bool copy = false, bool none = false, bool menu = false);
-	static wxArrayString GetAudioFormatLabels(bool copy = false, bool none = false);
+	static wxArrayString GetAudioFormatLabels(bool copy = false, bool none = false, bool pcm = true);
 	static wxArrayString GetSubtitleFormatLabels(bool copy = false, bool none = false);
 	static wxArrayString GetAspectRatioLabels(bool autom = false);
 	static wxArrayString GetCapacityLabels();
+	static wxArrayString GetDefPostCommandLabels();
 	static wxArrayString GetVideoBitrateLabels();
 	static wxArrayString GetVideoFormatNames();
 	static wxArrayString GetAudioFormatNames();
@@ -199,6 +216,7 @@ private:
     wxString m_label;
     /** dics capacity */
     DiscCapacity m_capacity;
+    DefaultPostCommand m_defPostCommand;
     bool m_videoBitrateAuto;
     int m_videoBitrate;
 	bool m_jumppad;
