@@ -3,7 +3,7 @@
 // Purpose:     Stores a DVD Action
 // Author:      Alex Thuering
 // Created:     03.09.2009
-// RCS-ID:      $Id: DVDAction.cpp,v 1.23 2011/04/28 21:39:38 ntalex Exp $
+// RCS-ID:      $Id: DVDAction.cpp,v 1.27 2012/10/14 16:23:29 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -18,6 +18,10 @@ extern "C" {
 #include "dvdvm.h"
 }
 
+/* entry menu types */
+const wxString s_entriesArr[6] = {wxT("title"), wxT("root"), wxT("subtitle"), wxT("audio"), wxT("angle"), wxT("ptt")};
+const wxArrayString s_entries(6, s_entriesArr);
+
 DVDAction::DVDAction(bool vmg, wxString id) {
 	m_vmg = vmg;
 	m_id = id;
@@ -30,10 +34,6 @@ DVDAction::DVDAction(bool vmg, wxString id) {
 	m_subtitle = -1;
 }
 
-/* entry menu types */
-const wxString s_entriesArr[6] = {wxT("title"), wxT("root"), wxT("subtitle"), wxT("audio"), wxT("angle"), wxT("ptt")};
-const wxArrayString s_entries(6, s_entriesArr);
-
 wxString DVDAction::AsString(DVD* dvd, bool translate) {
 	wxString action;
 	wxString actionPost;
@@ -193,7 +193,7 @@ bool validateJumpTarget(wxString msgPrefix, DVD* dvd, int tsi, int pgci, bool me
 	}
 	if (menu) {
 		Menus& menus = tsi < 0 ? dvd->GetVmgm() : dvd->GetTitlesets()[tsi]->GetMenus();
-		int menuCount = menus.Count() > 0 ? menus.Count() : (dvd->IsEmptyMenuEnabled() ? 1 : 0);
+		int menuCount = menus.Count() > 0 ? menus.Count() : (dvd->IsEmptyMenuEnabled() && dvd->HasMenus() ? 1 : 0);
 		if (menuCount == 0) {
 			if (tsi == -1)
 				wxLogError(msgPrefix + _("cannot jump to vmMenu; none exist"));
@@ -431,8 +431,13 @@ bool validateCall(wxString msgPrefix, DVD* dvd, const struct vm_statement *cs, i
 	int i4 = cs->i4; // resume cell if specified, else zero
 	int i5 = cs->i5; // last menu if specified, else zero
 	
-	if (i5)
+	if (i5) {
+		if (!dvd->HasMenus()) {
+			wxLogError(msgPrefix + _("cannot jump to menu; none exist"));
+			return false;
+		}
 		return true;
+	}
 	
 	if (!validateTargetNumbers(msgPrefix, i1, i2, i3)) {
 		return false;
@@ -566,7 +571,7 @@ bool DVDAction::IsCustomValid(wxString msgPrefix, wxString source, DVD* dvd, int
 		wxString msg = source + _("invalid action '%s'");
 		if (s_vmlErrorMsg.length())
 			msg += wxT(": ") + s_vmlErrorMsg;
-		else if (s_parseErrorMsg)
+		else if (s_parseErrorMsg.length())
 			msg += wxT(": ") + s_parseErrorMsg;
 		wxLogError(msg, m_custom.c_str());
 		return false;
@@ -600,6 +605,7 @@ int findFirstJump(const struct vm_statement *cs, int tsi) {
 				i1 = tsi + 2;
 			if (i1 > 1 && i2 > 128)
 				return DVD::MakeId(i1 - 2, i2 - 129, i3 < 256 ? i3 : 0, false);
+			break;
 		}
 		default:
 			break;
@@ -626,7 +632,7 @@ int DVDAction::FindFirstJump(int tsi) {
 }
 
 wxSvgXmlNode* DVDAction::GetXML(DVDFileType type, DVD* dvd) {
-	wxSvgXmlNode* rootNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("action"));
+	wxSvgXmlNode* rootNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, type == DVDAUTHOR_XML ? wxT("button") : wxT("action"));
 	switch (type) {
 	case DVDSTYLER_XML:
 		if (m_id.length())
