@@ -3,7 +3,7 @@
 // Purpose:     DVD properties dialog
 // Author:      Alex Thuering
 // Created:     7.03.2005
-// RCS-ID:      $Id: DVDPropDlg.cpp,v 1.33 2011/04/28 21:45:53 ntalex Exp $
+// RCS-ID:      $Id: DVDPropDlg.cpp,v 1.34 2012/06/10 20:47:11 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -63,7 +63,7 @@ wxArrayString DVDPropDlg::GetFPCommands() {
 bool DVDPropDlg::SetValues() {
 	if (!Validate())
 		return false;
-	int n = 7;
+	int n = 8;
 	if (GetString(n).length() > 0) {
 		DVDAction action;
 		action.SetCustom(GetString(n));
@@ -79,6 +79,8 @@ bool DVDPropDlg::SetValues() {
 		m_dvd->SetVideoFormat(GetVideoFormat(), true);
 	if (m_dvd->GetAspectRatio() != GetAspectRatio())
 		m_dvd->SetAspectRatio(GetAspectRatio(), true);
+	if (m_dvd->GetDefPostCommand() != GetDefPostCommand())
+		m_dvd->SetDefPostCommand(GetDefPostCommand(), true);
 	m_dvd->SetAudioFormat(GetAudioFormat());
 	// set FPC
 	m_dvd->GetVmgm().SetFpc(GetString(n++));
