@@ -3,7 +3,7 @@
 // Purpose:     Main window
 // Author:      Alex Thuering
 // Created:     10.10.2003
-// RCS-ID:      $Id: MainWin.cpp,v 1.152 2011/12/20 21:28:19 ntalex Exp $
+// RCS-ID:      $Id: MainWin.cpp,v 1.164 2012/09/18 20:20:02 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -53,10 +53,6 @@
 #define BUTTONS_DIR wxFindDataDirectory(_T("buttons"))
 #define HELP_FILE(lang) wxFindDataFile(wxT("docs/help_") + lang + wxT(".zip"))
 
-#if !wxCHECK_VERSION(2,9,0)
-#define wxBITMAP_TYPE_ANY -1
-#endif
-
 //////////////////////////// help functions //////////////////////////////////
 
 wxBitmap SetDefaultMask(wxBitmap bitmap)
@@ -91,6 +87,7 @@ BEGIN_EVENT_TABLE(MainWin, wxFrame)
     EVT_MENU(wxID_REDO, MainWin::OnRedo)
     EVT_MENU(MENU_DVD_ADD_FILE_ID, MainWin::OnAddFile)
     EVT_MENU(MENU_DVD_ADD_FILE_AS_CHAPTER_ID, MainWin::OnAddFileAsChapter)
+    EVT_MENU(MENU_DVD_ADD_TITLES_FROM_DVD_ID, MainWin::OnAddTitlesFromDVD)
     EVT_MENU(MENU_DVD_ADD_MENU_ID, MainWin::OnAddMenu)
     EVT_MENU(MENU_DVD_ADD_VMMENU_ID, MainWin::OnAddVmMenu)
     EVT_MENU(MENU_DVD_ADD_TITLESET_ID, MainWin::OnAddTitleset)
@@ -171,6 +168,7 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
     wxMenu* wxglade_tmp_menu_3_sub = new wxMenu();
     wxglade_tmp_menu_3_sub->Append(MENU_DVD_ADD_FILE_ID, _("&File..."), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_3_sub->Append(MENU_DVD_ADD_FILE_AS_CHAPTER_ID, _("File as &chapter..."), wxEmptyString, wxITEM_NORMAL);
+    wxglade_tmp_menu_3_sub->Append(MENU_DVD_ADD_TITLES_FROM_DVD_ID, _("Titles from &DVD..."), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_3_sub->Append(MENU_DVD_ADD_MENU_ID, _("&Menu"), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_3_sub->Append(MENU_DVD_ADD_VMMENU_ID, _("&vmMenu"), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_3_sub->Append(MENU_DVD_ADD_TITLESET_ID, _("&Titleset"), wxEmptyString, wxITEM_NORMAL);
@@ -203,9 +201,7 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
     m_statusbar->OnSize(sizeEvent);
     
     m_helpController.SetParentWindow(this);
-    wxString lang = s_config.GetLanguage() != wxLANGUAGE_UNKNOWN
-		&& wxLocale::GetLanguageInfo(s_config.GetLanguage()) ?
-    		wxLocale::GetLanguageInfo(s_config.GetLanguage())->CanonicalName : wxT("en");
+    wxString lang = s_config.GetLanguageCode().length() > 0 ? s_config.GetLanguageCode().Lower() : wxT("en");
     if (!wxFileExists(HELP_FILE(lang))) {
     	if (lang.length() > 2 && wxFileExists(HELP_FILE(lang.substr(0, 2))))
     		lang = lang.substr(0, 2);
@@ -237,7 +233,7 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
 	AspectRatio ar = (AspectRatio) s_config.GetDefAspectRatio();
 	DiscCapacity dc = (DiscCapacity) s_config.GetDefDiscCapacity();
 	NewDVD(wxT(""), wxT(""), s_config.GetDefDiscLabel(), dc, s_config.GetVideoBitrate(),
-			vf <= vfCOPY ? vfPAL : vf, af <= afCOPY ? afMP2 : af, ar <= arAUTO ? ar4_3 : ar);
+			vf <= vfCOPY ? vfPAL : vf, af <= afCOPY ? afMP2 : af, ar <= arAUTO ? ar4_3 : ar, cmdCALL_LAST_MENU);
 
 	// Restore frame size/position
 	if (s_config.IsMainWinMaximized()) {
@@ -297,8 +293,18 @@ void MainWin::set_properties() {
 	m_toolbar->AddTool(MENU_BURN_ID,  wxT(""), wxBITMAP_FROM_MEMORY(run), wxNullBitmap, wxITEM_NORMAL,
 			_("Burn..."), wxT(""));
 	m_toolbar->AddSeparator();
+#if wxCHECK_VERSION(2,9,0)
+	m_toolbar->AddTool(MENU_DVD_ADD_FILE_ID,  wxT(""), wxBITMAP_FROM_MEMORY(add), wxNullBitmap, wxITEM_DROPDOWN,
+			_("Add File..."), wxT(""));
+	wxMenu* addMenu = new wxMenu;
+	addMenu->Append(MENU_DVD_ADD_FILE_ID, _("&File..."));
+	addMenu->Append(MENU_DVD_ADD_FILE_AS_CHAPTER_ID, _("File as &chapter..."));
+	addMenu->Append(MENU_DVD_ADD_TITLES_FROM_DVD_ID, _("Titles from &DVD..."));
+	m_toolbar->SetDropdownMenu(MENU_DVD_ADD_FILE_ID, addMenu);
+#else
 	m_toolbar->AddTool(MENU_DVD_ADD_FILE_ID,  wxT(""), wxBITMAP_FROM_MEMORY(add), wxNullBitmap, wxITEM_NORMAL,
 			_("Add File..."), wxT(""));
+#endif
 	m_toolbar->Realize();
 	
 	// dir tree
@@ -315,9 +321,9 @@ void MainWin::set_properties() {
 	m_dirBookImages->Add(wxArtProvider::GetBitmap(wxART_FOLDER, wxART_TOOLBAR, imageSize)
 						 .ConvertToImage().Rotate90().Rotate90());
 	m_dirBook->SetImageList(m_dirBookImages);
-	m_dirBook->AddPage(new wxPanel(this), _("File browser"), false, 0);
-	m_dirBook->AddPage(new wxPanel(this), _("Backgrounds"));
-	m_dirBook->AddPage(new wxPanel(this), _("Buttons"));
+	m_dirBook->AddPage(new wxPanel(m_dirBook), _("File browser"), false, 0);
+	m_dirBook->AddPage(new wxPanel(m_dirBook), _("Backgrounds"));
+	m_dirBook->AddPage(new wxPanel(m_dirBook), _("Buttons"));
 	m_dirBook->SetMinSize(wxSize(m_dirBook->GetSize().x - 10, -1));
 #else
 	m_dirBar->SetSize(wxSize(40, 40));
@@ -414,7 +420,7 @@ void MainWin::OnBackgroundsBt(wxCommandEvent& event) {
 	m_thumbnails->Clear();
 	wxString fname = wxFindFirstFile(BACKGROUNDS_DIR + wxT("*"));
 	while (!fname.IsEmpty()) {
-		if (wxImage::FindHandler(fname.AfterLast('.').Lower(), wxBITMAP_TYPE_ANY))
+		if (wxImage::FindHandler(fname.AfterLast('.').Lower(), BITMAP_TYPE_ANY))
 			m_thumbnails->InsertItem(new wxThumb(fname));
 		fname = wxFindNextFile();
 	}
@@ -423,7 +429,7 @@ void MainWin::OnBackgroundsBt(wxCommandEvent& event) {
 	if (wxDirExists(bgDir)) {
 		fname = wxFindFirstFile(bgDir + wxT("*"));
 		while (!fname.IsEmpty()) {
-			if (wxImage::FindHandler(fname.AfterLast('.').Lower(), wxBITMAP_TYPE_ANY))
+			if (wxImage::FindHandler(fname.AfterLast('.').Lower(), BITMAP_TYPE_ANY))
 				m_thumbnails->InsertItem(new wxThumb(fname));
 			fname = wxFindNextFile();
 		}
@@ -540,7 +546,7 @@ void MainWin::OnTitleInfo(wxCommandEvent& event) {
 		Vob* vob = m_dvd->GetVob(DVD::GetTsi(id), DVD::IsMenu(id),
 				DVD::GetPgci(id), DVD::GetVobi(id));
 		if (vob) {
-			text = vob->GetFilename();
+			text = vob->GetFilenameDisplay();
 			for (unsigned int i=0; i<vob->GetAudioFilenames().Count(); i++)
 				text += (text.length() ? wxT(" + ") : wxT("Video + ")) + vob->GetAudioFilenames()[i];
 			for (unsigned int i=0; i<vob->GetSubtitles().Count(); i++)
@@ -574,7 +580,7 @@ void MainWin::OnThumbDoubleClick(wxCommandEvent& event) {
 	if (m_thumbnails->GetSelected() == -1)
 		return;
 	wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
-	if (wxImage::FindHandler(filename.AfterLast(wxT('.')).Lower(), wxBITMAP_TYPE_ANY) || MPEG::IsValid(filename))
+	if (wxImage::FindHandler(filename.AfterLast(wxT('.')).Lower(), BITMAP_TYPE_ANY))
 		m_menuEditor->SetBackground(filename);
 }
 
@@ -586,7 +592,7 @@ void MainWin::OnSetBackground(wxCommandEvent& event) {
 void MainWin::OnSetBackgroundUI(wxUpdateUIEvent& event) {
 	if (m_thumbnails->GetSelected() >= 0) {
 		wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
-		event.Enable(wxImage::FindHandler(filename.AfterLast(wxT('.')).Lower(), wxBITMAP_TYPE_ANY)
+		event.Enable(wxImage::FindHandler(filename.AfterLast(wxT('.')).Lower(), BITMAP_TYPE_ANY)
 				|| wxThumbnails::IsVideo(filename));
 	} else
 		event.Enable(false);
@@ -633,15 +639,18 @@ void MainWin::OnResize(wxSizeEvent &event) {
 }
 
 void MainWin::OnClose(wxCloseEvent &event) {
-	if (!TestSaved())
+	if (event.CanVeto() && !TestSaved()) {
+		event.Veto();
 		return;
+	}
 	if (!IsIconized())
 		s_config.SetMainWinLocation(GetRect(), IsMaximized());
 	Destroy();
 }
 
 void MainWin::NewDVD(wxString templateFile, wxString discTitle, wxString discLabel, DiscCapacity capacity,
-		int videoBitrate, VideoFormat videoFormat, AudioFormat audioFormat, AspectRatio aspectRatio) {
+		int videoBitrate, VideoFormat videoFormat, AudioFormat audioFormat, AspectRatio aspectRatio,
+		DefaultPostCommand defPostCommand) {
 	if (aspectRatio == ar16_9 && templateFile.Length()
 			&& wxFile::Exists(templateFile.substr(0, templateFile.length()-5) + wxT("WS.dvdt")))
 		templateFile = templateFile.substr(0, templateFile.length()-5) + wxT("WS.dvdt");
@@ -653,8 +662,10 @@ void MainWin::NewDVD(wxString templateFile, wxString discTitle, wxString discLab
 		m_dvd->SetVideoFormat(videoFormat);
 		m_dvd->SetAudioFormat(audioFormat);
 		m_dvd->SetAspectRatio(aspectRatio);
+		m_dvd->SetDefPostCommand(defPostCommand);
 		for (int tsi = -1; tsi < (int) m_dvd->GetTitlesets().GetCount(); tsi++) {
 			PgcArray& menus = m_dvd->GetPgcArray(tsi, true);
+			menus.GetVideo().SetFormat(videoFormat);
 			menus.GetVideo().SetAspect(aspectRatio);
 			for (unsigned int mi = 0; mi < menus.GetCount(); mi++) {
 				Menu* menu = menus[mi]->GetMenu();
@@ -666,12 +677,16 @@ void MainWin::NewDVD(wxString templateFile, wxString discTitle, wxString discLab
 						MenuObjectParam* initParam = obj->GetInitParam();
 						if (initParam && (initParam->type == wxT("text") || initParam->type == wxT("string"))) {
 							wxString value = obj->GetParam(initParam->name);
-							if (value == wxT("Play all"))
+							if (value == wxT("Play All"))
 								value = _("Play all");
-							else if (value == wxT("Select title"))
+							else if (value == wxT("Select Title"))
 								value = _("Select title");
 							else if (value == wxT("Back"))
 								value = _("Back");
+							else if (value == wxT("Previous"))
+								value = _("Previous");
+							else if (value == wxT("Next"))
+								value = _("Next");
 							value.Replace(wxT("$DiscTitle"), discTitle);
 							obj->SetParam(initParam->name, value);
 							obj->UpdateSize();
@@ -691,6 +706,7 @@ void MainWin::NewDVD(wxString templateFile, wxString discTitle, wxString discLab
 		m_dvd->SetVideoFormat(videoFormat);
 		m_dvd->SetAudioFormat(audioFormat);
 		m_dvd->SetAspectRatio(aspectRatio);
+		m_dvd->SetDefPostCommand(defPostCommand);
 		m_dvd->AddTitleset();
 		m_dvd->AddMenu(0);
 	}
@@ -720,7 +736,7 @@ bool MainWin::Open(wxString filename) {
 
 void MainWin::ShowOpenDlg() {
 	wxFileDialog dlg(this, _("Open a DVDStyler project file"), m_lastDir, wxT(""),
-			_("Project files") + wxString(wxT(" (*.dvds;*.xml)|*.dvds;*.xml|")), wxFD_OPEN);
+			_("Project files") + wxString(wxT(" (*.dvds;*.xml)|*.dvds;*.xml")), wxFD_OPEN);
 	dlg.Centre();
 	if (dlg.ShowModal() == wxID_OK)
 		Open(dlg.GetPath());
@@ -821,7 +837,7 @@ void MainWin::OnNew(wxCommandEvent& event) {
 		TemplateDlg templateDlg(this, dlg.GetAspectRatio());
 		wxString templateFile = templateDlg.ShowModal() == wxID_OK ? templateDlg.GetTemplate() : wxT("");
 		NewDVD(templateFile, templateDlg.GetTitle(), dlg.GetLabel(), dlg.GetCapacity(), dlg.GetVideoBitrate(),
-				dlg.GetVideoFormat(), dlg.GetAudioFormat(), dlg.GetAspectRatio());
+				dlg.GetVideoFormat(), dlg.GetAudioFormat(), dlg.GetAspectRatio(), dlg.GetDefPostCommand());
 	}
 }
 
@@ -840,18 +856,15 @@ void MainWin::OnSave(wxCommandEvent& event) {
 }
 
 void MainWin::OnSaveAs(wxCommandEvent& event) {
-	if (m_dvd->GetTitlesets().Count() || m_dvd->GetVmgm().Count()) {
-		wxFileDialog dlg(this, _("Save a DVDStyler project file"), m_lastDir, _T("dvd.dvds"),
-				_("Project files") + wxString(wxT(" (*.dvds;*.xml)|*.dvds;*.xml|"))
-				+ _("All Files") + wxString(wxT(" (*.*)|*.*")), wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
-		dlg.Centre();
-		if (dlg.ShowModal() == wxID_OK && m_dvd->Save(dlg.GetPath())) {
-			SetChanged(false);
-			m_lastDir = m_dvd->GetPath(false);
-			s_config.SetLastProjectDir(m_lastDir);
-		}
-	} else
-		wxLogError(_("Nothing to save"));
+	wxFileDialog dlg(this, _("Save a DVDStyler project file"), m_lastDir, _T("dvd.dvds"),
+			_("Project files") + wxString(wxT(" (*.dvds;*.xml)|*.dvds;*.xml|"))
+			+ _("All Files") + wxString(wxT(" (*.*)|*.*")), wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
+	dlg.Centre();
+	if (dlg.ShowModal() == wxID_OK && m_dvd->Save(dlg.GetPath())) {
+		SetChanged(false);
+		m_lastDir = m_dvd->GetPath(false);
+		s_config.SetLastProjectDir(m_lastDir);
+	}
 }
 /**
  * Starts compiling without closing
@@ -895,15 +908,6 @@ drag a video file into the bottom section of the window."), _("Burn"), wxOK|wxIC
 please drag a video file into the bottom section of the window."), emptyTsi+1), _("Burn"), wxOK|wxICON_ERROR, this);
 		return;
 	}
-	// check if the last title doesn't contain only video
-	PgcArray& pgcs = m_dvd->GetTitlesets().Last()->GetTitles();
-	Pgc* pgc = pgcs.Count()>0 ? pgcs.Last() : NULL;
-	Vob* vob = pgc && pgc->GetVobs().Count()>0 ? pgc->GetVobs().Last() : NULL;
-	if (vob && vob->GetFilename().length() && !vob->HasAudio()) {
-		wxMessageBox(wxString(_("The last title isn't complete.\n")) + _("Please add an audio track to it."),
-		_("Burn"), wxOK|wxICON_ERROR, this);
-		return;
-	}
 	// check if all menus are ok
 	if (!m_dvd->CheckMenus())
 		return;
@@ -918,7 +922,7 @@ please drag a video file into the bottom section of the window."), emptyTsi+1),
 		return;
 	// Progress dialog
 	ProgressDlg progressDlg(this, &m_cache, autoStart);
-	bool ok = progressDlg.Start(&burnDlg, m_dvd);
+	progressDlg.Start(&burnDlg, m_dvd);
 	if (autoStart)
 		Close(true);
 }
@@ -1003,11 +1007,11 @@ void MainWin::AddFileDialog(bool asChapter) {
 			+ _("Video Files") + wxString::Format(wxT(" (%s)|%s|"), video.c_str(), video.c_str())
 			+ _("Subtitle Files") + wxString::Format(wxT(" (%s)|%s|"), subtitle.c_str(), subtitle.c_str())
 			+ _("Image Files") + wxString::Format(wxT(" %s"), wxImage::GetImageExtWildcard().c_str())
-			+ wxString(wxT("|")) + _("All Files") + wxString(wxT(" (*.*)|*.*|"));
+			+ wxString(wxT("|")) + _("All Files") + wxString(wxT(" (*.*)|*.*"));
 	if (asChapter)
 		wildcard = _("Video Files") + wxString::Format(wxT(" (%s)|%s|"), video.c_str(), video.c_str())
 				+ _("Image Files") + wxString::Format(wxT(" %s"), wxImage::GetImageExtWildcard().c_str())
-				+ wxString(wxT("|")) + _("All Files") + wxString(wxT(" (*.*)|*.*|"));
+				+ wxString(wxT("|")) + _("All Files") + wxString(wxT(" (*.*)|*.*"));
 	wxFileDialog dialog(this, _("Choose a file"), wxT(""), wxT(""), wildcard, wxFD_OPEN | wxFD_MULTIPLE);
 	// set file browser dir
 	dialog.SetDirectory(s_config.GetLastAddDir() + wxFILE_SEP_PATH);
@@ -1017,27 +1021,48 @@ void MainWin::AddFileDialog(bool asChapter) {
 		for (unsigned int i = 0; i < paths.GetCount(); i++) {
 			wxString filename = paths[i];
 			if (wxThumbnails::IsVideo(filename)) {
-				// check file
-				wxFfmpegMediaDecoder decoder;
-				if (decoder.Load(filename))
-					m_titlesetManager->AddVideo(filename, !asChapter);
+				m_titlesetManager->AddVideo(filename, !asChapter);
 			} else if (wxThumbnails::IsAudio(filename)) {
 				m_titlesetManager->AddAudio(filename);
 			} else if (wxThumbnails::IsSubtitle(filename)) {
 				m_titlesetManager->AddSubtitles(filename);
-			} else if (wxImage::FindHandler(filename.AfterLast('.').Lower(), wxBITMAP_TYPE_ANY)) {
+			} else if (wxImage::FindHandler(filename.AfterLast('.').Lower(), BITMAP_TYPE_ANY)) {
 				m_titlesetManager->AddImage(filename, !asChapter && i == 0);
 			} else {
 				// try to load it as video
-				wxFfmpegMediaDecoder decoder;
-				if (decoder.Load(filename))
-					m_titlesetManager->AddVideo(filename, !asChapter);
+				m_titlesetManager->AddVideo(filename, !asChapter);
 			}
 		}
 		s_config.SetLastAddDir(dialog.GetDirectory());
 	}
 }
 
+void MainWin::OnAddTitlesFromDVD(wxCommandEvent& event) {
+	wxDirDialog dialog(this, _("Choose DVD device or directory"));
+	if (dialog.ShowModal() != wxID_OK)
+		return;
+	wxString dir = dialog.GetPath();
+	if (wxDirExists(dir + wxFILE_SEP_PATH + wxString(wxT("VIDEO_TS"))))
+		dir += wxFILE_SEP_PATH + wxString(wxT("VIDEO_TS"));
+	wxString vobs;
+	for (int i = 0; i < 100; i++) {
+		wxString fileName = dir + wxFILE_SEP_PATH + wxString::Format(wxT("VTS_01_%d.VOB"), i);
+		if (!wxFileExists(fileName)) {
+			if (i == 0)
+				continue;
+			if (i == 1) {
+				wxLogError(wxT("%s: no such file or directory"), fileName.c_str());
+				return;
+			}
+			break;
+		}
+		if (vobs.length() > 0)
+			vobs += wxT('|');
+		vobs = vobs + fileName;
+	}
+	m_titlesetManager->AddVideo(wxT("concat:") + vobs, true);
+}
+
 void MainWin::OnAddMenu(wxCommandEvent& event) {
 	m_titlesetManager->SetSelected(-1);
 	m_titlesetManager->AddMenu();
