@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu
 // Author:      Alex Thuering
 // Created:	04.11.2003
-// RCS-ID:      $Id: Menu.cpp,v 1.105 2011/11/22 20:27:02 ntalex Exp $
+// RCS-ID:      $Id: Menu.cpp,v 1.110 2012/04/01 23:25:42 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -28,10 +28,6 @@
 
 #define BUTTON_MAX_COLORS 4
 
-#if !wxCHECK_VERSION(2,9,0)
-#define wxBITMAP_TYPE_ANY -1
-#endif
-
 Menu::Menu(DVD* dvd, int tsi, int pgci, VideoFormat videoFormat, AspectRatio aspectRatio) {
 	m_dvd = dvd;
 	m_tsi = tsi;
@@ -48,6 +44,7 @@ Menu::Menu(DVD* dvd, int tsi, int pgci, VideoFormat videoFormat, AspectRatio asp
 Menu::~Menu() {
 	delete m_svg;
 	WX_CLEAR_ARRAY(m_objects)
+	VECTOR_CLEAR(m_actions, DVDAction)
 }
 
 wxSVGDocument* Menu::CreateSVG(int width, int height) {
@@ -149,7 +146,7 @@ void Menu::SetBackground(wxString fileName) {
 }
 
 void Menu::SetBackground(wxSVGSVGElement* root, wxString fileName) {
-	bool image = wxImage::FindHandler(fileName.AfterLast(wxT('.')).Lower(), wxBITMAP_TYPE_ANY) != NULL;
+	bool image = wxImage::FindHandler(fileName.AfterLast(wxT('.')).Lower(), BITMAP_TYPE_ANY) != NULL;
 	// test if element exists
 	wxSVGElement* bgElement = (wxSVGElement*) root->GetElementById(wxT("background"));
 	if (fileName.length() == 0) {
@@ -684,7 +681,7 @@ wxString Menu::AddImage(wxString fileName, int x, int y) {
 }
 
 wxString Menu::AddText(wxString text, int x, int y) {
-	return AddObject(OBJECTS_DIR + wxT("/text.xml"), text, x, y);
+	return AddObject(OBJECTS_DIR + wxT("/text-v2.xml"), text, x, y);
 }
 
 wxString Menu::AddObject(wxString fileName, wxString param, int x, int y) {
@@ -718,12 +715,16 @@ bool Menu::UpdateButtonImageFor(int actionTsi, int actionPgci, DVD* dvd) {
 				if (param->type == _T("image")) {
 					PgcArray& pgcs = dvd->GetPgcArray(actionTsi == -2 ? m_tsi : actionTsi, false);
 					wxString uri;
-					if (actionPgci < (int)pgcs.size())
+					long int pos = 0;
+					if (actionPgci < (int)pgcs.size()) {
 						uri = pgcs[actionPgci]->GetVideoFrameUri(obj->GetAction().GetChapter());
-					wxString uriFname = uri.Find(wxT('#')) >= 0 ? uri.BeforeLast(wxT('#')) : uri;
+						uri.AfterLast(wxT('#')).ToLong(&pos);
+						uri = uri.Find(wxT('#')) >= 0 ? uri.BeforeLast(wxT('#')) : uri;
+					}
 					if (!obj->IsCustomVideoFrame() || uri.length() == 0
-							|| uriFname != obj->GetParam(param->name).BeforeLast(wxT('#'))) {
-						obj->SetParam(param->name, uri);
+							|| uri != obj->GetParam(param->name).BeforeLast(wxT('#'))) {
+						double dur = obj->GetParamVideoDuration(param->name);
+						obj->SetParamImageVideo(param->name, uri, pos, (int) lround(dur));
 						obj->UpdateSize();
 						result = true;
 					}
@@ -751,6 +752,25 @@ void Menu::FixButtonCoordinates() {
 	}
 }
 
+/** Checks if buttons overlapping */
+bool Menu::isButtonsOverlapping() {
+	for (unsigned int obji = 0; obji < m_objects.GetCount(); obji++) {
+		MenuObject* obj = m_objects[obji];
+		if (!obj->IsButton())
+			continue;
+		for (unsigned int obji2 = 0; obji2 < m_objects.GetCount(); obji2++) {
+			if (obji == obji2)
+				continue;
+			MenuObject* obj2 = m_objects[obji2];
+			if (!obj2->IsButton())
+				continue;
+			if (obj->GetBBox().Intersects(obj2->GetBBox()))
+				return true;
+		}
+	}
+	return false;
+}
+
 bool Menu::SaveSpumux(wxString fileName, SubStreamMode mode, wxString btFile, wxString hlFile, wxString selFile) {
 	wxSvgXmlDocument xml;
 	wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("subpictures"));
@@ -773,7 +793,7 @@ bool Menu::SaveSpumux(wxString fileName, SubStreamMode mode, wxString btFile, wx
 			spuNode->AddChild(obj->GetXML(SPUMUX_XML, m_dvd, mode));
 	}
 	// actions
-	for (unsigned int i=0; i<m_actions.Count(); i++)
+	for (unsigned int i=0; i<m_actions.size(); i++)
 		spuNode->AddChild(m_actions[i]->GetXML(SPUMUX_XML, NULL));
 	streamNode->AddChild(spuNode);
 	root->AddChild(streamNode);
@@ -850,7 +870,7 @@ wxSvgXmlNode* Menu::GetXML(DVDFileType type, wxSvgXmlNode* node) {
 	}
 	
 	// actions
-	for (unsigned int i=0; i<m_actions.Count(); i++)
+	for (unsigned int i=0; i<m_actions.size(); i++)
 		node->AddChild(m_actions[i]->GetXML(type, m_dvd));
 	
 	return node;
@@ -1024,7 +1044,7 @@ wxString Menu::AddObject(wxSvgXmlNode* node, bool fixPosition) {
             delete action;
             return wxT("");
         }
-        m_actions.Add(action);
+        m_actions.push_back(action);
         return action->GetId();
     } else if (node->GetName() == wxT("button") || node->GetName() == wxT("object")) {
     	if (node->GetName() == wxT("button")) {
