@@ -3,7 +3,7 @@
 // Purpose:     The widget to edit a DVD Menu
 // Author:      Alex Thuering
 // Created:     11.10.2003
-// RCS-ID:      $Id: MenuEditor.cpp,v 1.60 2011/11/23 22:01:30 ntalex Exp $
+// RCS-ID:      $Id: MenuEditor.cpp,v 1.67 2012/07/22 09:33:20 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -32,10 +32,6 @@
 #define BUTTONS_DIR wxFindDataDirectory(_T("buttons"))
 #define CURSOR_FILE(fname) wxFindDataFile(_T("rc") + wxString(wxFILE_SEP_PATH) + fname)
 
-#if !wxCHECK_VERSION(2,9,0)
-#define wxBITMAP_TYPE_ANY -1
-#endif
-
 //////////////////////////////////////////////////////////////////////////////
 ////////////////////////////// MenuDnDFile ///////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
@@ -63,7 +59,7 @@ public:
 			wxString actionStr((const char*) m_actionDataObject->GetData(), wxConvISO8859_1, m_actionDataObject->GetSize());
 			DVDAction action;
 			action.Deserialize(actionStr);
-			AddButton(x, y, action.GetTsi(), action.GetPgci(), action.IsMenu());
+			AddButton(x, y, action.GetTsi(), action.GetPgci(), action.IsMenu(), action.GetChapter());
 			return def;
 		}
 		
@@ -89,7 +85,7 @@ public:
 		wxString ext = filename.AfterLast('.').Lower();
 		if (ext == _T("xml"))
 			m_menuEditor->AddButton(filename, (int) (x/m_menuEditor->GetScaleX()), (int) (y/m_menuEditor->GetScaleY()));
-		else if (wxImage::FindHandler(ext, wxBITMAP_TYPE_ANY))
+		else if (wxImage::FindHandler(ext, BITMAP_TYPE_ANY))
 			m_menuEditor->AddImage(filename, (int) (x/m_menuEditor->GetScaleX()), (int) (y/m_menuEditor->GetScaleY()));
 		else if (wxThumbnails::IsVideo(filename)) {
 			wxFfmpegMediaDecoder decoder;
@@ -113,12 +109,15 @@ public:
 			for (unsigned int tsi = 0; tsi < m_menuEditor->GetDVD()->GetTitlesets().GetCount(); tsi++) {
 				Titleset& ts = *m_menuEditor->GetDVD()->GetTitlesets()[tsi];
 				for (unsigned int pgci = 0; pgci < ts.GetTitles().GetCount(); pgci++) {
-					Pgc& pgc = *ts.GetTitles()[pgci];
-					for (unsigned int vobi = 0; vobi < pgc.GetVobs().GetCount(); vobi++) {
-						Vob& vob = *pgc.GetVobs()[vobi];
-						if (vob.GetFilename() == filename) {
+					Pgc* pgc = ts.GetTitles()[pgci];
+					for (unsigned int vobi = 0; vobi < pgc->GetVobs().GetCount(); vobi++) {
+						Vob* vob = pgc->GetVobs()[vobi];
+						if (vob->GetFilename() == filename) {
 							found = true;
-							AddButton(x, y, tsi, pgci, false, !first);
+							int chapter = 0;
+							if (vobi > 0)
+								chapter = pgc->GetChapterCount(vobi) + (vob->GetChapters().length() ? 0 : -1);
+							AddButton(x, y, tsi, pgci, false, chapter, !first);
 							break;
 						}
 					}
@@ -133,7 +132,7 @@ public:
 					tsi = 0;
 				if (m_menuEditor->GetTitlesetManager()->AddVideo(filename, true, tsi)) {
 					int pgci = m_menuEditor->GetDVD()->GetPgcArray(tsi, false).GetCount()-1;
-					AddButton(x, y, tsi, pgci, false, !first);
+					AddButton(x, y, tsi, pgci, false, 0, !first);
 				}
 			}
 		}
@@ -141,7 +140,7 @@ public:
 		return true;
 	}
 
-	void AddButton(wxCoord x, wxCoord y, int tsi, int pgci, bool isMenu, bool force = false) {
+	void AddButton(wxCoord x, wxCoord y, int tsi, int pgci, bool isMenu, int chapter, bool force = false) {
 		wxString objId = m_menuEditor->HitTest(x, y);
 		if (!force && objId.length() && m_menuEditor->GetMenu()->GetObject(objId)->IsButton()) {
 			MenuObject* obj = m_menuEditor->GetMenu()->GetObject(objId);
@@ -156,13 +155,14 @@ public:
 			obj->GetAction().SetTsi(tsi == m_menuEditor->GetTsi() ? -2 : tsi);
 			obj->GetAction().SetPgci(pgci);
 			obj->GetAction().SetMenu(isMenu);
+			obj->GetAction().SetChapter(chapter);
 		} else {
 			x = (int) (x/m_menuEditor->GetScaleX());
 			y = (int) (y/m_menuEditor->GetScaleY());
 			if (isMenu)
-				m_menuEditor->AddButton(BUTTONS_DIR + s_config.GetDefTextButton(), x, y, pgci, tsi, isMenu);
+				m_menuEditor->AddButton(BUTTONS_DIR + s_config.GetDefTextButton(), x, y, pgci, tsi, isMenu, chapter);
 			else
-				m_menuEditor->AddButton(BUTTONS_DIR + s_config.GetDefButton(), x, y, pgci, tsi, isMenu);
+				m_menuEditor->AddButton(BUTTONS_DIR + s_config.GetDefButton(), x, y, pgci, tsi, isMenu, chapter);
 		}
 	}
 
@@ -268,7 +268,8 @@ END_EVENT_TABLE()
 DEFINE_EVENT_TYPE(EVT_COMMAND_MENUEDITOR_MENU_CHANGED)
 DEFINE_EVENT_TYPE(EVT_COMMAND_MENUEDITOR_OBJECT_POINTED)
 
-MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id), m_gridColour(255, 255, 255) {
+MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id, wxDefaultPosition, wxDefaultSize,
+		wxWANTS_CHARS), m_gridColour(255, 255, 255) {
 	m_dvd = NULL;
 	m_menu = NULL;
 	objectIsMoving = false;
@@ -283,32 +284,37 @@ MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id),
 	SetFitToFrame(true);
 	
 	// sub menu "Arrange"
-	m_arrangeMenu = new wxMenu;
-	m_arrangeMenu->Append(MENUEDITOR_ARRANGEMENU_TOFRONT_ID, _("&Bring to Front"));
-	m_arrangeMenu->Append(MENUEDITOR_ARRANGEMENU_FORWARD_ID, _("Bring &Forward"));
-	m_arrangeMenu->Append(MENUEDITOR_ARRANGEMENU_BACKWARD_ID, _("Send Back&ward"));
-	m_arrangeMenu->Append(MENUEDITOR_ARRANGEMENU_TOBACK_ID, _("&Send to Back"));
+	wxMenu* arrangeMenu = new wxMenu;
+	arrangeMenu->Append
+	
+	(MENUEDITOR_ARRANGEMENU_TOFRONT_ID, _("&Bring to Front"));
+	arrangeMenu->Append(MENUEDITOR_ARRANGEMENU_FORWARD_ID, _("Bring &Forward"));
+	arrangeMenu->Append(MENUEDITOR_ARRANGEMENU_BACKWARD_ID, _("Send Back&ward"));
+	arrangeMenu->Append(MENUEDITOR_ARRANGEMENU_TOBACK_ID, _("&Send to Back"));
 
 	// sub menu "Align"
-	m_alignMenu = new wxMenu;
-	m_alignMenu->Append(MENUEDITOR_ALIGNMENU_TOCENTERH_ID, _("Center &Horizontally"));
-	m_alignMenu->Append(MENUEDITOR_ALIGNMENU_TOCENTERV_ID, _("Center &Vertically"));
+	wxMenu* alignMenu = new wxMenu;
+	alignMenu->Append(MENUEDITOR_ALIGNMENU_TOCENTERH_ID, _("Center &Horizontally"));
+	alignMenu->Append(MENUEDITOR_ALIGNMENU_TOCENTERV_ID, _("Center &Vertically"));
 
 	// sub menu "Add"
-	m_addMenu = new wxMenu;
+	wxMenu* addMenu = new wxMenu;
+	wxMenu* addMenu2 = new wxMenu;
 	wxString fname = wxFindFirstFile(OBJECTS_DIR + _T("*.xml"));
 	int addId = MENUEDITOR_ADDMENU_OBJECT_ID;
 	while (!fname.IsEmpty()) {
 		MenuObject obj(NULL, false, fname);
-		m_addMenu->Append(addId++, wxGetTranslation(obj.GetTitle().c_str()));
+		addMenu->Append(addId, wxGetTranslation(obj.GetTitle().c_str()));
+		addMenu2->Append(addId, wxGetTranslation(obj.GetTitle().c_str()));
 		m_addMenuObjects.Add(obj.GetFileName());
+		addId++;
 		fname = wxFindNextFile();
 	}
 	
 	// sub menu "View"
-	m_viewMenu = new wxMenu;
-	m_viewMenu->Append(MENUEDITOR_VIEWMENU_SAFETV_ID, _("&Show Safe TV area"), wxEmptyString, wxITEM_CHECK);
-	m_viewMenu->Check(MENUEDITOR_VIEWMENU_SAFETV_ID, m_safeTV);
+	wxMenu* viewMenu = new wxMenu;
+	viewMenu->Append(MENUEDITOR_VIEWMENU_SAFETV_ID, _("&Show Safe TV area"), wxEmptyString, wxITEM_CHECK);
+	viewMenu->Check(MENUEDITOR_VIEWMENU_SAFETV_ID, m_safeTV);
 	
 	// sub menu "Grid"
 	m_gridMenu = new wxMenu;
@@ -325,11 +331,11 @@ MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id),
 	
 	// object context menu
 	m_objMenu = new wxMenu;
-	m_objMenu->Append(MENUEDITOR_ARRANGEMENU_ID, _("A&rrange"), m_arrangeMenu);
-	m_objMenu->Append(MENUEDITOR_ALIGNMENU_ID, _("A&lign"), m_alignMenu);
-	m_objMenu->Append(MENUEDITOR_ADDMENU_ID, _("&Add"), m_addMenu);
-	m_objMenu->Append(MENUEDITOR_VIEWMENU_ID, _("&View"), m_viewMenu);
-	m_objMenu->Append(MENUEDITOR_GRIDMENU_ID, _("&Grid"), m_gridMenu);
+	m_objMenu->Append(MENUEDITOR_ARRANGEMENU_ID, _("A&rrange"), arrangeMenu);
+	m_objMenu->Append(MENUEDITOR_ALIGNMENU_ID, _("A&lign"), alignMenu);
+	m_objMenu->Append(MENUEDITOR_ADDMENU_ID, _("&Add"), addMenu);
+//	m_objMenu->Append(MENUEDITOR_VIEWMENU_ID, _("&View"), viewMenu);
+//	m_objMenu->Append(MENUEDITOR_GRIDMENU_ID, _("&Grid"), m_gridMenu);
 	m_objMenu->AppendCheckItem(MENUEDITOR_OBJMENU_FIRSTBUTTON_ID, _("&First highlighted button"));
 	m_objMenu->AppendSeparator();
 	wxMenuItem* item = new wxMenuItem(m_objMenu, MENUEDITOR_OBJMENU_CUT_ID, _("Cu&t"));
@@ -349,8 +355,8 @@ MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id),
 
 	// context menu
 	m_pmenu = new wxMenu;
-	m_pmenu->Append(MENUEDITOR_ADDMENU_ID, _("&Add"), m_addMenu);
-	m_pmenu->Append(MENUEDITOR_VIEWMENU_ID, _("&View"), m_viewMenu);
+	m_pmenu->Append(MENUEDITOR_ADDMENU_ID, _("&Add"), addMenu2);
+	m_pmenu->Append(MENUEDITOR_VIEWMENU_ID, _("&View"), viewMenu);
 	m_pmenu->Append(MENUEDITOR_GRIDMENU_ID, _("&Grid"), m_gridMenu);
 	item = new wxMenuItem(m_pmenu, MENUEDITOR_OBJMENU_PASTE_ID, _("&Paste"));
 	item->SetBitmap(wxArtProvider::GetBitmap(wxART_PASTE, wxART_MENU));
@@ -385,7 +391,7 @@ bool MenuEditor::SetBackground(wxString fname) {
 	if (!m_menu)
 		return false;
 
-	if (wxImage::FindHandler(fname.AfterLast(wxT('.')).Lower(), wxBITMAP_TYPE_ANY)) {
+	if (wxImage::FindHandler(fname.AfterLast(wxT('.')).Lower(), BITMAP_TYPE_ANY)) {
 		wxImage img;
 		if (!img.LoadFile(fname))
 			return false;
@@ -418,10 +424,6 @@ void MenuEditor::SetBackgroundColour(wxColour colour) {
 bool MenuEditor::AddImage(wxString fname, int x, int y) {
 	if (!m_menu)
 		return false;
-	if (m_menu->HasVideoBackground()) {
-		wxLogError(_("You can add only buttons to menu with VIDEO background"));
-		return false;
-	}
 	wxImage img;
 	if (!img.LoadFile(fname))
 		return false;
@@ -433,15 +435,11 @@ bool MenuEditor::AddImage(wxString fname, int x, int y) {
 bool MenuEditor::AddText(wxString text, int x, int y) {
 	if (!m_menu)
 		return false;
-	if (m_menu->HasVideoBackground()) {
-		wxLogError(_("You can add only buttons to menu with VIDEO background"));
-		return false;
-	}
 	DoSelect(m_menu->AddText(text, x, y), true, true);
 	return true;
 }
 
-bool MenuEditor::AddButton(wxString fname, int x, int y, int pgci, int tsi, bool isMenu) {
+bool MenuEditor::AddButton(wxString fname, int x, int y, int pgci, int tsi, bool isMenu, int chapter) {
 	if (!m_menu || !wxFileExists(fname))
 		return false;
 	
@@ -475,10 +473,10 @@ bool MenuEditor::AddButton(wxString fname, int x, int y, int pgci, int tsi, bool
 		Pgc* pgc = m_dvd->GetPgc(tsi, false, pgci >= 0 ? pgci : 0);
 		if (initParam->type == wxT("image")) {
 			if (pgc)
-				param = pgc->GetVideoFrameUri(0);
+				param = pgc->GetVideoFrameUri(chapter);
 		} else if (pgci >= 0 && !isMenu) {
 			if (pgc)
-				param = wxFileName(pgc->GetVideoFrameUri(0)).GetName();
+				param = wxFileName(pgc->GetVideoFrameUri(chapter)).GetName();
 		} else {
 			if (isMenu) {
 				param = wxString::Format(tsi == -1 ? _("vmMenu %d") : _("Menu %d"), pgci+1);
@@ -492,6 +490,7 @@ bool MenuEditor::AddButton(wxString fname, int x, int y, int pgci, int tsi, bool
 	obj->GetAction().SetTsi(tsi == m_tsi ? -2 : tsi);
 	obj->GetAction().SetPgci(pgci >= 0 ? pgci : 0);
 	obj->GetAction().SetMenu(isMenu);
+	obj->GetAction().SetChapter(chapter);
 	
 	// fix position
 	wxRect rect = obj->GetBBox();
@@ -645,27 +644,30 @@ void MenuEditor::RefreshSelected() {
 }
 
 void MenuEditor::OnMouseLeftButton(wxMouseEvent &event) {
-	SetFocus();
-	if (!m_menu)
+	if (!m_menu) {
+		event.Skip();
 		return;
+	}
 	if (objectIsMoving) {
 		objectIsMoving = false;
 		SendChangedEvent();
 	}
-	if (event.LeftDown()) {
-		m_action = GetEditorAction(event.GetX(), event.GetY());
-		m_oldx = event.GetX();
-		m_oldy = event.GetY();
-		m_doCopy = event.ControlDown();
-		m_copied = false;
-	} else {
-		if (m_action == eaNONE || (event.ControlDown() && !m_copied)) {
-			// select object
-			DoSelect(HitTest(event.GetX(), event.GetY()), true, false, event.ControlDown());
-			// change mouse cursor
-			OnMouseMove(event);
-		}
+	int x = event.GetX();
+	int y = event.GetY();
+	m_action = GetEditorAction(event.GetX(), event.GetY());
+	m_oldx = x;
+	m_oldy = y;
+	m_doCopy = event.ControlDown();
+	if ((event.LeftDown() && !event.ControlDown() && m_action == eaNONE)
+			|| (!event.LeftDown() && event.ControlDown() && !m_copied)) {
+		// select object
+		DoSelect(HitTest(x, y), true, false, event.ControlDown());
+		// change mouse cursor
+		event.m_leftDown = false;
+		OnMouseMove(event);
 	}
+	m_copied = false;
+	event.Skip();
 }
 
 void MenuEditor::OnMouseMove(wxMouseEvent &event) {
@@ -1038,7 +1040,7 @@ void MenuEditor::OnKeyDown(wxKeyEvent& event) {
 			break;
 		}
 		case WXK_DELETE:
-			if (m_selectedIds.size() == 1) {
+			if (m_selectedIds.size() > 0) {
 				wxCommandEvent evt;
 				OnObjectDelete(evt);
 			}
@@ -1063,6 +1065,20 @@ void MenuEditor::OnKeyDown(wxKeyEvent& event) {
 				OnObjectPaste(evt);
 			}
 			break;
+		case WXK_LEFT:
+		case WXK_RIGHT:
+		case WXK_UP:
+		case WXK_DOWN:
+			if (m_selectedIds.size() > 0) {
+				if (m_actionObject.length() == 0)
+					m_actionObject = m_selectedIds[0];
+				int key = event.GetKeyCode();
+				int x = key == WXK_LEFT ? -(1 << m_grid) : key == WXK_RIGHT ? 1 << m_grid : 0;
+				int y = key == WXK_UP ? -(1 << m_grid) : key == WXK_DOWN ? 1 << m_grid : 0;
+				MoveObjectInt(x, y);
+				RefreshSelected();
+			}
+			break;
 		default:
 			event.Skip();
 			break;
@@ -1141,6 +1157,7 @@ void MenuEditor::OnObjectDelete(wxCommandEvent& event) {
 
 void MenuEditor::OnFirstButton(wxCommandEvent& event) {
 	m_menu->SetFirstButton(GetSelected());
+	SendChangedEvent();
 }
 
 void MenuEditor::OnUpdateUIFirstButton(wxUpdateUIEvent& event) {
@@ -1176,6 +1193,10 @@ void MenuEditor::OnUpdateUIObjectBackward(wxUpdateUIEvent& event) {
 	event.Enable(m_selectedIds.size() == 1 && !m_menu->IsFirstObject(GetSelected()));
 }
 
+int MenuEditor::SnapToGrid(int pos) {
+	return m_grid > 0 ? pos >> m_grid << m_grid : pos;
+}
+
 void MenuEditor::OnAlignToCenterH(wxCommandEvent& event) {
 	int x1 = 9999;
 	int x2 = -9999;
@@ -1186,10 +1207,10 @@ void MenuEditor::OnAlignToCenterH(wxCommandEvent& event) {
 		if (obj->GetX() + obj->GetWidth() > x2)
 			x2 = obj->GetX() + obj->GetWidth();
 	}
-	int x = (m_menu->GetResolution().GetWidth() - (x2 - x1))/2;
+	int x = (m_menu->GetResolution().GetWidth() - (x2 - x1))/2 - x1;
 	for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
 		MenuObject* obj = m_menu->GetObject(*it);
-		obj->SetX(x + obj->GetX() - x1);
+		obj->SetX(SnapToGrid(x + obj->GetX()));
 	}
 	RefreshSelected();
 	SendChangedEvent();
@@ -1205,10 +1226,10 @@ void MenuEditor::OnAlignToCenterV(wxCommandEvent& event) {
 		if (obj->GetY() + obj->GetHeight() > y2)
 			y2 = obj->GetY() + obj->GetHeight();
 	}
-	int y = (m_menu->GetResolution().GetHeight() - (y2 - y1))/2;
+	int y = (m_menu->GetResolution().GetHeight() - (y2 - y1))/2 - y1;
 	for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
 		MenuObject* obj = m_menu->GetObject(*it);
-		obj->SetY(y + obj->GetY() - y1);
+		obj->SetY(SnapToGrid(y + obj->GetY()));
 	}
 	RefreshSelected();
 	SendChangedEvent();
