@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu Object
 // Author:      Alex Thuering
 // Created:	04.11.2006
-// RCS-ID:      $Id: MenuObject.cpp,v 1.32 2011/11/23 22:02:01 ntalex Exp $
+// RCS-ID:      $Id: MenuObject.cpp,v 1.42 2012/10/14 14:58:19 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -183,10 +183,10 @@ bool MenuObject::Init(wxString fileName, int x, int y, wxString param) {
 }
 
 void MenuObject::InitSize(wxString value, MenuObjectSize& size) {
-	long lval;
+	long lval = 0;
 	if (value.Index(wxT('|'))) {
-		value.BeforeFirst(wxT('|')).ToLong(&lval);
-		size.value = lval;
+		if (value.BeforeFirst(wxT('|')).ToLong(&lval))
+			size.value = lval;
 		value = value.AfterFirst(wxT('|'));
 	}
 	while (value.Length() > 0) {
@@ -315,7 +315,7 @@ wxString MenuObject::GetId(bool translate) {
 		return m_id;
 	long l = 0;
 	m_id.Mid(IsButton() ? 6 : 3).ToLong(&l);
-	return (IsButton() ? _("button") : _("object")) + wxString::Format(wxT(" %d"), l);
+	return (IsButton() ? _("button") : _("object")) + wxString::Format(wxT(" %d"), (int) l);
 }
 
 int MenuObject::GetX() {
@@ -357,13 +357,15 @@ wxRect MenuObject::GetFrameBBox(SubStreamMode mode, bool ignorePadding) {
 		fy *= (double) m_menu->GetFrameResolution().GetHeight() / m_menu->GetResolution().GetHeight();
 	int width = round(GetWidth()*fx);
 	int height = round(GetHeight()*fy);
-	if (width % 2 == 1)
-		width++;
-	if (height % 2 == 1)
-		height++;
 	int padY = mode == ssmLETTERBOX && !ignorePadding ? m_menu->GetFrameResolution().GetHeight()*0.125: 0;
 	int cropX = mode == ssmPANSCAN ? m_menu->GetFrameResolution().GetWidth()*0.125 : 0;
-	return wxRect(round((GetX() - cropX)*fx), round(GetY()*fy) + padY, width, height);
+	int x = round((GetX() - cropX)*fx);
+	int y = round(GetY()*fy) + padY;
+	if (height % 2 == 1)
+		height += y % 2 == 0 ? 1 : -1;
+	if (y % 2 == 1)
+		y++;
+	return wxRect(x, y, width, height);
 }
 
 unsigned int MenuObject::CalcSize(MenuObjectSize& size, bool width) {
@@ -420,12 +422,11 @@ void MenuObject::UpdateSize() {
 	SetHeight(height);
 }
 
-wxString MenuObject::GetParam(wxString name, wxString attrSuffix) {
+wxString MenuObject::GetParam(wxString name, wxString attribute) {
 	MenuObjectParam* param = GetObjectParam(name);
 	if (!param)
 		return wxT("");
-	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(
-			param->element);
+	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
 	if (!elem)
 		return wxT("");
 	if (param->attribute.length()) {
@@ -443,7 +444,8 @@ wxString MenuObject::GetParam(wxString name, wxString attrSuffix) {
 				return value.Trim().Trim(false);
 			}
 		}
-		return elem->GetAttribute(param->attribute + attrSuffix);
+		return elem->GetAttribute(attribute.length() && attribute[0] != wxT('-') ?
+				attribute : param->attribute + attribute);
 	} else if (elem->GetDtd() == wxSVG_TEXT_ELEMENT) {
 		wxString value;
 		wxSvgXmlNode* child = elem->GetChildren();
@@ -460,7 +462,7 @@ wxString MenuObject::GetParam(wxString name, wxString attrSuffix) {
 	return wxT("");
 }
 
-void MenuObject::SetParam(wxString name, wxString value, wxString attrSuffix) {
+void MenuObject::SetParam(wxString name, wxString value, wxString attribute) {
 	MenuObjectParam* param = GetObjectParam(name);
 	if (!param)
 		return;
@@ -485,8 +487,10 @@ void MenuObject::SetParam(wxString name, wxString value, wxString attrSuffix) {
 				value = newValue;
 			}
 			elem->SetAttribute(param->attribute.BeforeFirst(wxT('#')), value);
-		} else
-			elem->SetAttribute(param->attribute + attrSuffix, value);
+		} else {
+			elem->SetAttribute(attribute.length() && attribute[0] != wxT('-') ?
+					attribute : param->attribute + attribute, value);
+		}
 		if (elem->GetDtd() == wxSVG_IMAGE_ELEMENT && param->attribute == wxT("xlink:href"))
 			((wxSVGImageElement*) elem)->SetCanvasItem(NULL);
 	} else if (elem->GetDtd() == wxSVG_TEXT_ELEMENT) {
@@ -519,14 +523,14 @@ void MenuObject::SetParam(wxString name, wxString value, wxString attrSuffix) {
 	}
 }
 
-int MenuObject::GetParamInt(wxString name) {
+int MenuObject::GetParamInt(wxString name, wxString attribute) {
 	long lval = 0;
-	GetParam(name).ToLong(&lval);
+	GetParam(name, attribute).ToLong(&lval);
 	return lval;
 }
 
-void MenuObject::SetParamInt(wxString name, int value) {
-	SetParam(name, wxString::Format(wxT("%d"), value));
+void MenuObject::SetParamInt(wxString name, int value, wxString attribute) {
+	SetParam(name, wxString::Format(wxT("%d"), value), attribute);
 }
 
 double MenuObject::GetParamDouble(wxString name) {
@@ -666,6 +670,82 @@ void MenuObject::SetParamColour(wxString name, wxColour value, MenuButtonState s
 	}
 }
 
+double MenuObject::GetParamVideoClipBegin(const wxString& name) {
+	MenuObjectParam* param = GetObjectParam(name);
+	if (!param)
+		return 0;
+	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
+	if (!elem || elem->GetDtd() != wxSVG_VIDEO_ELEMENT)
+		return 0;
+	return ((wxSVGVideoElement*) elem)->GetClipBegin();
+}
+
+double MenuObject::GetParamVideoDuration(const wxString& name) {
+	MenuObjectParam* param = GetObjectParam(name);
+	if (!param)
+		return 0;
+	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
+	if (!elem || elem->GetDtd() != wxSVG_VIDEO_ELEMENT)
+		return 0;
+	return ((wxSVGVideoElement*) elem)->GetDur();
+}
+
+void MenuObject::SetParamImageVideo(const wxString& name, const wxString& filename, long pos, int duration) {
+	MenuObjectParam* param = GetObjectParam(name);
+	if (!param)
+		return;
+	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
+	if (!elem || (elem->GetDtd() != wxSVG_IMAGE_ELEMENT && elem->GetDtd() != wxSVG_VIDEO_ELEMENT))
+		return;
+	if (duration <= 0) {
+		if (elem->GetDtd() != wxSVG_IMAGE_ELEMENT) {
+			wxSVGVideoElement* oldElem = (wxSVGVideoElement*) elem;
+			wxSVGImageElement* newElem = new wxSVGImageElement;
+			newElem->SetId(oldElem->GetId());
+			newElem->SetX(oldElem->GetX().GetBaseVal());
+			newElem->SetY(oldElem->GetY().GetBaseVal());
+			newElem->SetWidth(oldElem->GetWidth().GetBaseVal());
+			newElem->SetHeight(oldElem->GetHeight().GetBaseVal());
+			newElem->SetStyle(oldElem->GetStyle());
+			newElem->SetPreserveAspectRatio(oldElem->GetPreserveAspectRatio());
+			oldElem->GetParent()->InsertBefore(newElem, oldElem);
+			oldElem->GetParent()->RemoveChild(oldElem);
+			elem = newElem;
+		}
+		wxString href = filename + (filename.length() && pos >= 0 ? wxString::Format(wxT("#%ld"), pos) : wxT(""));
+		if (href != ((wxSVGImageElement*) elem)->GetHref()) {
+			((wxSVGImageElement*) elem)->SetCanvasItem(NULL);
+			((wxSVGImageElement*) elem)->SetHref(href);
+		}
+	} else {
+		if (elem->GetDtd() != wxSVG_VIDEO_ELEMENT) {
+			wxSVGImageElement* oldElem = (wxSVGImageElement*) elem;
+			wxSVGVideoElement* newElem = new wxSVGVideoElement;
+			newElem->SetId(oldElem->GetId());
+			newElem->SetX(oldElem->GetX().GetBaseVal());
+			newElem->SetY(oldElem->GetY().GetBaseVal());
+			newElem->SetWidth(oldElem->GetWidth().GetBaseVal());
+			newElem->SetHeight(oldElem->GetHeight().GetBaseVal());
+			newElem->SetStyle(oldElem->GetStyle());
+			newElem->SetPreserveAspectRatio(oldElem->GetPreserveAspectRatio());
+			oldElem->GetParent()->InsertBefore(newElem, oldElem);
+			oldElem->GetParent()->RemoveChild(oldElem);
+			elem = newElem;
+		}
+		if (filename != ((wxSVGVideoElement*) elem)->GetHref()) {
+			((wxSVGVideoElement*) elem)->SetCanvasItem(NULL);
+			((wxSVGVideoElement*) elem)->SetHref(filename);
+		}
+		((wxSVGVideoElement*) elem)->SetClipBegin(((double)pos)/1000);
+		((wxSVGVideoElement*) elem)->SetDur(duration);
+	}
+}
+
+/** Returns object svg element by element id */
+wxSVGElement* MenuObject::GetElementById(wxString id) {
+	return (wxSVGElement*) m_symbol->GetElementById(id);
+}
+
 void MenuObject::ToFront() {
 	wxSVGElement* parent = (wxSVGElement*) m_use->GetParent();
 	parent->RemoveChild(m_use);
@@ -712,6 +792,70 @@ bool MenuObject::IsLast() {
 	return !next || next->GetType() != wxSVGXML_ELEMENT_NODE;
 }
 
+wxString MenuObject::GetDefaultFocusDest(NavigationButton navButton) {
+	map<int, wxArrayString> btMap; // dist -> buttons
+	map<int, wxArrayString> btMapAll; // dist -> buttons
+	int minDist = 99999;
+	int minDistAll = 99999;
+	wxRect rect = GetBBox();
+	for (unsigned int i = 0; i < m_menu->GetObjectsCount(); i++) {
+		MenuObject* obj = m_menu->GetObject(i);
+		if (!obj->IsButton())
+			continue;
+		wxRect objRect = obj->GetBBox();
+		int dist = -1;
+		if (navButton == nbLEFT)
+			dist = rect.GetX() - objRect.GetRight() - 1;
+		else if (navButton == nbRIGHT)
+			dist = objRect.GetX() - rect.GetRight() - 1;
+		else if (navButton == nbUP)
+			dist = rect.GetY() - objRect.GetBottom() - 1;
+		else // nbDOWN
+			dist = objRect.GetY() - rect.GetBottom() - 1;
+		if (dist < 0)
+			continue;
+		if (navButton == nbLEFT || navButton == nbRIGHT)
+			objRect.SetX(rect.GetX());
+		else // nbUP/nbDOWN
+			objRect.SetY(rect.GetY());
+		if (objRect.Intersects(rect)) {
+			btMap[dist].Add(obj->GetId());
+			if (dist < minDist)
+				minDist = dist;
+		}
+		btMapAll[dist].Add(obj->GetId());
+		if (dist < minDistAll)
+			minDistAll = dist;
+	}
+	// filter by minDist
+	wxArrayString buttons;
+	for (map<int, wxArrayString>::iterator it = btMap.begin(); it != btMap.end(); it++)
+		if (it->first <= minDist + 4)
+			buttons.insert(buttons.begin(), it->second.begin(), it->second.end());
+	if (buttons.IsEmpty()) {
+		if (btMapAll.empty())
+			return wxT("");
+		buttons = btMapAll[minDistAll];
+	}
+	// filter by distance to left/up
+	wxString result;
+	int resultDist = 0;
+	for (unsigned int i = 0; i < buttons.size(); i++) {
+		MenuObject* obj = m_menu->GetObject(buttons[i]);
+		wxRect objRect = obj->GetBBox();
+		int dist = navButton == nbLEFT || navButton == nbRIGHT
+				? objRect.GetY() - rect.GetY() : objRect.GetX() - rect.GetX();
+		if (result.length() == 0) {
+			result = obj->GetId();
+			resultDist = dist;
+		} else if (dist < resultDist) {
+			result = obj->GetId();
+			resultDist = dist;
+		}
+	}
+	return result;
+}
+
 wxImage MenuObject::GetImage(int maxWidth, int maxHeight) {
 	if (!m_use)
 		return wxImage();
@@ -741,17 +885,17 @@ wxSvgXmlNode* MenuObject::GetXML(DVDFileType type, DVD* dvd, SubStreamMode mode,
 	case DVDSTYLER_XML:
 		if (IsButton())
 			rootNode->AddChild(m_action.GetXML(type, dvd));
-		if (GetDirection(0).length() || GetDirection(1).length()
-				|| GetDirection(2).length() || GetDirection(3).length()) {
+		if (GetFocusDest(nbLEFT).length() || GetFocusDest(nbRIGHT).length()
+				|| GetFocusDest(nbUP).length() || GetFocusDest(nbDOWN).length()) {
 			wxSvgXmlNode* directionNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("direction"));
-			if (GetDirection(0).length())
-				directionNode->AddProperty(_T("left"), GetDirection(0));
-			if (GetDirection(1).length())
-				directionNode->AddProperty(_T("right"), GetDirection(1));
-			if (GetDirection(2).length())
-				directionNode->AddProperty(_T("up"), GetDirection(2));
-			if (GetDirection(3).length())
-				directionNode->AddProperty(_T("down"), GetDirection(3));
+			if (GetFocusDest(nbLEFT).length())
+				directionNode->AddProperty(_T("left"), GetFocusDest(nbLEFT));
+			if (GetFocusDest(nbRIGHT).length())
+				directionNode->AddProperty(_T("right"), GetFocusDest(nbRIGHT));
+			if (GetFocusDest(nbUP).length())
+				directionNode->AddProperty(_T("up"), GetFocusDest(nbUP));
+			if (GetFocusDest(nbDOWN).length())
+				directionNode->AddProperty(_T("down"), GetFocusDest(nbDOWN));
 			rootNode->AddChild(directionNode);
 		}
 		XmlWriteValue(rootNode, _T("filename"), GetFileName().AfterLast(wxFILE_SEP_PATH));
@@ -787,14 +931,18 @@ wxSvgXmlNode* MenuObject::GetXML(DVDFileType type, DVD* dvd, SubStreamMode mode,
 				rect.GetX() + rect.GetWidth() < mwidth ? rect.GetX() + rect.GetWidth() : mwidth));
 		rootNode->AddProperty(_T("y1"), wxString::Format(_T("%d"),
 				rect.GetY() + rect.GetHeight() < mheight ? rect.GetY() + rect.GetHeight() : mheight));
-		if (GetDirection(0).length())
-			rootNode->AddProperty(_T("left"), GetDirection(0));
-		if (GetDirection(1).length())
-			rootNode->AddProperty(_T("right"), GetDirection(1));
-		if (GetDirection(2).length())
-			rootNode->AddProperty(_T("up"), GetDirection(2));
-		if (GetDirection(3).length())
-			rootNode->AddProperty(_T("down"), GetDirection(3));
+		wxString focusDest = GetFocusDest(nbLEFT).length() ? GetFocusDest(nbLEFT) : GetDefaultFocusDest(nbLEFT);
+		if (focusDest.length())
+			rootNode->AddProperty(_T("left"), focusDest);
+		focusDest = GetFocusDest(nbRIGHT).length() ? GetFocusDest(nbRIGHT) : GetDefaultFocusDest(nbRIGHT);
+		if (focusDest.length())
+			rootNode->AddProperty(_T("right"), focusDest);
+		focusDest = GetFocusDest(nbUP).length() ? GetFocusDest(nbUP) : GetDefaultFocusDest(nbUP);
+		if (focusDest.length())
+			rootNode->AddProperty(_T("up"), focusDest);
+		focusDest = GetFocusDest(nbDOWN).length() ? GetFocusDest(nbDOWN) : GetDefaultFocusDest(nbDOWN);
+		if (focusDest.length())
+			rootNode->AddProperty(_T("down"), focusDest);
 		break;
 	}
 	case DVDAUTHOR_XML: {
@@ -850,13 +998,13 @@ bool MenuObject::PutXML(wxSvgXmlNode* node) {
 		wxSvgXmlNode* dirNode = XmlFindNodeSimple(node, _T("direction"));
 		if (dirNode) {
 			if (dirNode->GetPropVal(_T("left"), &val))
-				SetDirection(0, val);
+				SetFocusDest(nbLEFT, val);
 			if (dirNode->GetPropVal(_T("right"), &val))
-				SetDirection(1, val);
+				SetFocusDest(nbRIGHT, val);
 			if (dirNode->GetPropVal(_T("up"), &val))
-				SetDirection(2, val);
+				SetFocusDest(nbUP, val);
 			if (dirNode->GetPropVal(_T("down"), &val))
-				SetDirection(3, val);
+				SetFocusDest(nbDOWN, val);
 		}
 	}
 
