@@ -3,13 +3,15 @@
 // Purpose:     DVD menu button properties dialog
 // Author:      Alex Thuering
 // Created:	20.11.2003
-// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.49 2011/09/11 18:53:21 ntalex Exp $
+// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.58 2012/07/01 19:27:08 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "MenuObjectPropDlg.h"
+#include "VideoFrameDlg.h"
 #include "Config.h"
+#include <wxSVG/svg.h>
 #include <wx/statline.h>
 #include <stdint.h>
 
@@ -17,19 +19,23 @@ enum {
 	JUMP_ACTION_RADIO_ID = 7900,
 	CUSTOM_ACTION_RADIO_ID,
 	TARGET_CHOICE_ID,
+	CHAPTER_CHOICE_ID,
 	AUTO_SIZE_RADIO_ID,
 	KEEP_ASPECT_CB_ID,
 	IMAGE_RADIO_ID,
-	VIDEOFRAME_RADIO_ID
+	VIDEOFRAME_RADIO_ID,
+	VIDEOFRAME_BT_ID
 };
 
 BEGIN_EVENT_TABLE(MenuObjectPropDlg, wxPropDlg)
 	EVT_RADIOBUTTON(JUMP_ACTION_RADIO_ID, MenuObjectPropDlg::OnJumpActionSelected)
 	EVT_RADIOBUTTON(CUSTOM_ACTION_RADIO_ID, MenuObjectPropDlg::OnCustomActionSelected)
 	EVT_CHOICE(TARGET_CHOICE_ID, MenuObjectPropDlg::OnChangeTarget)
+	EVT_CHOICE(CHAPTER_CHOICE_ID, MenuObjectPropDlg::OnChangeChapter)
 	EVT_RADIOBUTTON(KEEP_ASPECT_CB_ID, MenuObjectPropDlg::OnKeepAspectRatio)
 	EVT_RADIOBUTTON(IMAGE_RADIO_ID, MenuObjectPropDlg::OnImageRadio)
 	EVT_RADIOBUTTON(VIDEOFRAME_RADIO_ID, MenuObjectPropDlg::OnImageRadio)
+	EVT_BUTTON(VIDEOFRAME_BT_ID, MenuObjectPropDlg::OnVideoFrame)
 END_EVENT_TABLE()
 
 MenuObjectPropDlg::MenuObjectPropDlg(wxWindow* parent, wxString id, bool multObjects, Menu* menu, DVD* dvd, int tsi,
@@ -44,24 +50,27 @@ MenuObjectPropDlg::MenuObjectPropDlg(wxWindow* parent, wxString id, bool multObj
 	m_imageEdit = NULL;
 	m_imageRadio = NULL;
 	m_videoFrameRadio = NULL;
-	m_videoFrameChoice = NULL;
+	m_videoFrameBt = NULL;
 	m_customActionEdit = NULL;
 	m_widthEdit = NULL;
 	m_heightEdit = NULL;
 	m_object = menu->GetObject(id);
 	m_displayVideoFrame = m_object->IsDisplayVideoFrame();
+	m_defaultPos = 0;
+	m_videoPos = 0;
+	m_videoDuration = 0;
 	Create();
 }
 
 void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 	wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);
 	wxFlexGridSizer* grid = NULL;
-	
+
 	// action
 	if (m_object->IsButton() && !m_multObjects) {
 		DVDAction& action = m_object->GetAction();
 		AddStaticLine(mainSizer, _("Action"));
-		grid = new wxFlexGridSizer(2, 2, 4, 16);
+		grid = new wxFlexGridSizer(2, 4, 16);
 		grid->AddGrowableCol(1);
 		// check menu entry if it is set
 		wxString customAction = action.GetCustom();
@@ -83,7 +92,7 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 		AddChoiceProp(jumpSizer, wxT(""), wxT(""), wxArrayString(), -1, false, TARGET_CHOICE_ID);
 		m_targetChoice = (wxChoice*) m_controls[m_controls.Count()-1];
 		FillTargets(m_targetChoice);
-		AddChoiceProp(jumpSizer, wxT(""), wxT(""), wxArrayString());
+		AddChoiceProp(jumpSizer, wxT(""), wxT(""), wxArrayString(), -1, true, CHAPTER_CHOICE_ID);
 		m_chapterChoice = (wxChoice*) m_controls[m_controls.Count()-1];
 		UpdateChapters();
 		AddCheckProp(jumpSizer, _("Play all titles"), action.IsPlayAll());
@@ -108,11 +117,11 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 			OnCustomActionSelected(evt);
 		else
 			OnJumpActionSelected(evt);
-		
+
 		// audio & subtitle
 		wxArrayString labels;
 		labels.Add(_("last selected"));
-		for (unsigned int i = 0; i < m_dvd->GetAudioStreamCount(); i++)
+		for (int i = 0; i < (int) m_dvd->GetAudioStreamCount(); i++)
 			labels.Add(_("track") + wxString::Format(wxT(" %d"), i + 1));
 		AddText(grid, _("Audio") + wxString(wxT(":")), wxALIGN_CENTER_VERTICAL|wxALIGN_RIGHT);
 		wxSizer* s = AddChoiceProp(grid, wxT(""), labels[action.GetAudio()+1], labels, 0, false);
@@ -122,7 +131,7 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 		labels.Clear();
 		labels.Add(_("last selected"));
 		labels.Add(_("off"));
-		for (unsigned int i = 0; i < m_dvd->GetSubtitleStreamsCount(); i++)
+		for (int i = 0; i < (int) m_dvd->GetSubtitleStreamsCount(); i++)
 			labels.Add(_("track") + wxString::Format(wxT(" %d"), i + 1));
 		AddText(s, _("subtitle") + wxString(wxT(":")));
 		s->Add(16, 16);
@@ -130,30 +139,37 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 		AddChoiceProp(s, wxT(""), labels[subtitleIdx], labels, 0, true);
 		if (m_dvd->GetSubtitleStreamsCount() == 0)
 			GetLastControl()->Enable(false);
-		
+
 		grid = new wxFlexGridSizer(3, 4, 4, 16);
 		grid->AddGrowableCol(0);
 		grid->Add(10,10);
 		AddText(grid, _("Focus"));
-		AddDirectionProp(grid, bdUP);
+		AddFocusDestProp(grid, nbUP);
 		grid->Add(10,10);
 		grid->Add(10,10);
-		AddDirectionProp(grid, bdLEFT);
+		AddFocusDestProp(grid, nbLEFT);
 		grid->Add(10,10);
-		AddDirectionProp(grid, bdRIGHT);
+		AddFocusDestProp(grid, nbRIGHT);
 		grid->Add(10,10);
 		grid->Add(10,10);
-		AddDirectionProp(grid, bdDOWN);
+		AddFocusDestProp(grid, nbDOWN);
 		grid->Add(10,10);
 		mainSizer->Add(grid, 0, wxEXPAND|wxALL, 4);
 		AddStaticLine(mainSizer, _("Look"));
 	}
-	
-	// object look parameters
-	bool lastChangeable = false;
-	for (int i=0; i<m_object->GetObjectParamsCount(); i++) {
+    CreateLook(mainSizer, grid);
+    if (!m_multObjects)
+		CreateLocAndSize(mainSizer);
+
+	sizer->Add(mainSizer, 1, wxEXPAND|wxALL, 4);
+}
+
+void MenuObjectPropDlg::CreateLook(wxBoxSizer* mainSizer, wxFlexGridSizer* grid) {
+    // object look parameters
+    bool lastChangeable = false;
+    for (int i=0; i<m_object->GetObjectParamsCount(); i++) {
 		MenuObjectParam* param = m_object->GetObjectParam(i);
-		
+
 		if (i == 0 || param->changeable != lastChangeable) {
 			if (param->changeable) {
 				grid = new wxFlexGridSizer(3, 4, 4, 16);
@@ -171,9 +187,9 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 				mainSizer->Add(grid, 0, wxEXPAND|wxBOTTOM, 4);
 			lastChangeable = param->changeable;
 		}
-		
+
 		wxString title = wxGetTranslation((const wxChar*)param->title.GetData()) + wxString(wxT(":"));
-		
+
 		if (param->changeable && param->type == _T("colour")) {
 			AddColourProp(grid, title, m_object->GetParamColour(param->name));
 			AddColourProp(grid, wxT(""), m_object->GetParamColour(param->name, mbsHIGHLIGHTED));
@@ -190,7 +206,7 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 			} else if (param->type == _T("string")) {
 				AddTextProp(grid, title, m_object->GetParam(param->name));
 			} else if (param->type == _T("decimal")) {
-				AddTextProp(grid, title, m_object->GetParam(param->name), false, 100);
+				AddTextProp(grid, title, m_object->GetParam(param->name), false, 54);
 				if (m_multObjects && param->attribute.Find(wxT('#')) != wxNOT_FOUND)
 					GetLastControl()->Enable(false);
 			} else if (param->type.Mid(0,7) == _T("integer")) {
@@ -205,34 +221,41 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 			} else if (param->type == _T("image") && !m_multObjects) {
 				wxString imageFile = m_object->GetParam(param->name);
 				if (m_object->IsButton()) {
-					bool hasVideo = GetVideoFilename().length();
+					bool hasVideo = GetVideoFilename(false).length();
 					wxString fname = imageFile.Find(wxT('#')) != -1 ? imageFile.BeforeLast(wxT('#')) : imageFile;
-					bool image = !m_displayVideoFrame || !hasVideo || fname != GetVideoFilename();
+					bool image = !m_displayVideoFrame || !hasVideo || fname != GetVideoFilename(false);
 					AddRadioProp(grid, wxString(_("Image")) + wxT(":"), image, wxRB_GROUP, false, IMAGE_RADIO_ID);
 					m_imageRadio = (wxRadioButton*) GetLastControl();
 					AddFileProp(grid, wxT(""), image ? imageFile : wxT(""), wxFD_OPEN, wxT("..."),
 							_("Image Files ") + wxImage::GetImageExtWildcard()
 							+ wxT("|") + wxString(_("All Files")) + wxT(" (*.*)|*.*"));
 					m_imageEdit = (wxTextCtrl*) GetLastControl();
-					AddRadioProp(grid, wxString(_("Video Frame")) + wxT(":"), !image, 0, false, VIDEOFRAME_RADIO_ID);
+					m_imageEditIdx = GetLastControlIndex();
+					AddRadioProp(grid, wxString(_("Video")) + wxT(":"), !image, 0, false, VIDEOFRAME_RADIO_ID);
 					m_videoFrameRadio = (wxRadioButton*) GetLastControl();
-					if (!hasVideo)
-						m_videoFrameRadio->Enable(false);
-					bool vfAuto = imageFile == GetVideoFilename(true) || image;
-					const wxString frameChoices[] = {_("Auto"), _("Custom")};
-					wxSizer* frameSizer = AddChoiceCustomProp(grid, wxT(""), frameChoices[vfAuto ? 0 : 1],
-							wxArrayString(2, frameChoices), 1, 100, false);
-					m_videoFrameChoice = (wxChoice*) GetLastControl();
-					m_videoFrameChoice->Enable(!image);
-					frameSizer->AddSpacer(4);
-					long videoFrame = 0;
+					m_videoFrameRadio->Enable(hasVideo);
+					m_videoFrameBt = new wxButton(this, VIDEOFRAME_BT_ID, wxT("..."));
+					int h = m_videoFrameBt->GetSize().GetHeight() > 24 ? m_videoFrameBt->GetSize().GetHeight() : 24;
+					m_videoFrameBt->SetSizeHints(h, h, h, h);
+					grid->Add(m_videoFrameBt);
+					wxCommandEvent evt;
+					OnImageRadio(evt);
+					// default video position
+					wxString url = GetVideoFilename(true);
+					if (url.length())
+						url.AfterLast(wxT('#')).ToLong(&m_defaultPos);
+					// video position
 					if (!image) {
-						wxString vf = imageFile.Find(wxT('#')) != -1 ? imageFile.AfterLast(wxT('#')) : wxT("");
-						if (vf.length()> 0)
-							vf.ToLong(&videoFrame);
-					}
-					AddSpinProp(frameSizer, wxT(""), videoFrame, 0, 99999999, 100, _("ms"), false);
-					SetLastControlCustom(GetLastControlIndex()-1, !vfAuto);
+						m_videoPos = lround(m_object->GetParamVideoClipBegin(param->name) * 1000);
+						if (!image && imageFile.Find(wxT('#')) != -1) {
+							wxString vf = imageFile.AfterLast(wxT('#'));
+							if (vf.length() > 0)
+								vf.ToLong(&m_videoPos);
+						}
+					} else
+						m_videoPos = m_defaultPos;
+					// video duration
+					m_videoDuration = m_object->GetParamVideoDuration(param->name);
 				} else {
 					AddFileProp(grid, title, imageFile, wxFD_OPEN,
 						wxT("..."), _("Image Files ") + wxImage::GetImageExtWildcard()
@@ -243,26 +266,48 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 			} else if (param->type == _T("colour")) {
 				int opacity = 100;
 				wxString opacityStr = m_object->GetParam(param->name, wxT("-opacity"));
-				if (opacityStr.length()> 0) {
+				if (opacityStr.length() > 0) {
 					double dval;
 					opacityStr.ToDouble(&dval);
 					opacity = (int) (dval*100);
 				}
 				AddColourProp(grid, title, m_object->GetParamColour(param->name), opacity);
+			} else if (param->type == wxT("shadow")) {
+				bool shadow = m_object->GetParam(param->name, wxT("visibility")) != wxT("hidden");
+				BeginCheckGroup(grid, title, shadow, false);
+				wxSizer* shadowSizer = new wxBoxSizer(wxHORIZONTAL);
+				grid->Add(shadowSizer);
+				int opacity = 100;
+				wxString opacityStr = m_object->GetParam(param->name, wxT("-opacity"));
+				if (opacityStr.length() > 0) {
+					double dval;
+					opacityStr.ToDouble(&dval);
+					opacity = (int) (dval*100);
+				}
+				AddColourProp(shadowSizer, wxT(""), m_object->GetParamColour(param->name), opacity);
+				shadowSizer->AddSpacer(8);
+				AddText(shadowSizer, _("Offset:"));
+				shadowSizer->AddSpacer(4);
+				AddSpinProp(shadowSizer, wxT(""), m_object->GetParamInt(param->name, wxT("x")), -999, 999, 54, wxT(""), false);
+				AddSpinProp(shadowSizer, wxT(""), m_object->GetParamInt(param->name, wxT("y")), -999, 999, 54, wxT(""), false);
+				shadowSizer->AddSpacer(4);
+				wxSVGElement* filter = m_object->GetElementById(param->element + wxT("Filter"));
+				if (filter && ((wxSVGElement*)filter->GetFirstChild())->GetDtd() == wxSVG_FEGAUSSIANBLUR_ELEMENT) {
+					wxSVGFEGaussianBlurElement* blur = (wxSVGFEGaussianBlurElement*) filter->GetFirstChild();
+					AddText(shadowSizer, _("Deviation:"));
+					shadowSizer->AddSpacer(4);
+					AddTextProp(shadowSizer, wxT(""), blur->GetAttribute(wxT("stdDeviation")), false, 54);
+				}
+				EndGroup();
 			}
 		}
 	}
-	
-	if (!m_multObjects)
-		CreateLocAndSize(mainSizer);
-	
-	sizer->Add(mainSizer, 1, wxEXPAND|wxALL, 4);
 }
 
 void MenuObjectPropDlg::CreateLocAndSize(wxBoxSizer* mainSizer) {
 	wxSizer* hSizer = new wxBoxSizer(wxHORIZONTAL);
 	mainSizer->Add(hSizer, 0, wxTOP, 8);
-		
+
 	// object location
 	wxSizer* grpSizer = BeginGroup(hSizer, _("Location"));
 	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 4, 16);
@@ -271,7 +316,7 @@ void MenuObjectPropDlg::CreateLocAndSize(wxBoxSizer* mainSizer) {
 	grpSizer->Add(grid, 0, wxALL, 4);
 	EndGroup();
 	hSizer->AddSpacer(8);
-	
+
 	// object size
 	grpSizer = BeginGroup(hSizer, _("Size"), _("Custom"), !m_object->IsDefaultSize());
 	grid = new wxFlexGridSizer(2, 4, 16);
@@ -311,7 +356,7 @@ int MenuObjectPropDlg::GetSelectedChapter() {
 }
 
 void* MenuObjectPropDlg::TargetToPtr(int tsi, int pgci, bool menu) {
-	return (void*) ((tsi+1)*1000 + pgci*2 + (menu ? 0 : 1));
+	return (void*) (((long) tsi+1)*1000 + pgci*2 + (menu ? 0 : 1));
 }
 
 /**
@@ -344,9 +389,18 @@ void MenuObjectPropDlg::FillTargets(wxChoice* ctrl) {
 				if (tsi == m_tsi && pgci == m_pgci)
 					continue; // current menu
 				Pgc* pgc = ts->GetMenus()[pgci];
-				if (!m_dvd->IsJumppadsEnabled() && m_tsi != tsi && pgci > 0 && pgc->GetEntries().size() == 0)
-					break; // cannot jump to specific menu from other titleset, only to an entry
 				wxString label = _("menu") + wxString::Format(wxT(" %d"), pgci + 1);
+				if (!m_dvd->IsJumppadsEnabled() && m_tsi != tsi) {
+					int rootMenu = ts->GetMenus().GetPgciByEntry(wxT("root"));
+					if (pgci != rootMenu && pgc->GetEntries().size() == 0)
+						continue; // cannot jump to specific menu from other titleset, only to an entry
+					const wxString entries[6] = {
+							_("title vmMenu"), _("root menu"), _("subtitle menu"),
+							_("audio menu"), _("angle menu"), _("chapter menu")};
+					int entryIdx = pgc->GetEntries().size() > 0 ? s_entries.Index(*pgc->GetEntries().begin()) : 1;
+					if (entryIdx >= 0)
+						label = label + wxT(" (") + entries[entryIdx] + wxT(")");
+				}
 				if (m_dvd->GetTitlesets().Count() > 1)
 					label = _("titleset") + wxString::Format(wxT(" %d "), tsi + 1) + label;
 				ctrl->Append(label, TargetToPtr(tsi, pgci, true));
@@ -396,7 +450,7 @@ void MenuObjectPropDlg::UpdateChapters() {
 	ctrl->Append(wxString::Format(_("chapter %d"), 1));
 	PgcArray& pgcs = m_dvd->GetPgcArray(tsi, false);
 	if (tsi >= 0 && !menu && pgci < (int) pgcs.Count())
-		for (unsigned int i = 1; i < pgcs[pgci]->GetChapterCount(); i++)
+		for (int i = 1; i < (int) pgcs[pgci]->GetChapterCount(); i++)
 			ctrl->Append(wxString::Format(_("chapter %d"), i + 1));
 	ctrl->SetSelection(chapter);
 	ctrl->Enable(ctrl->GetCount() > 1 && (m_dvd->IsJumppadsEnabled() || m_tsi != -1));
@@ -405,7 +459,7 @@ void MenuObjectPropDlg::UpdateChapters() {
 void MenuObjectPropDlg::OnChangeTarget(wxCommandEvent& evt) {
 	UpdateChapters();
 	if (m_videoFrameRadio) {
-		m_videoFrameRadio->Enable(GetVideoFilename().length());
+		m_videoFrameRadio->Enable(GetVideoFilename(false).length());
 		if (!m_videoFrameRadio->IsEnabled() && !m_imageRadio->GetValue()) {
 			m_imageRadio->SetValue(true);
 			wxCommandEvent event(wxEVT_COMMAND_RADIOBUTTON_SELECTED, m_imageRadio->GetId());
@@ -415,6 +469,23 @@ void MenuObjectPropDlg::OnChangeTarget(wxCommandEvent& evt) {
 			wxCommandEvent event(wxEVT_COMMAND_RADIOBUTTON_SELECTED, m_imageRadio->GetId());
 			GetEventHandler()->ProcessEvent(event);
 		}
+		// reset video position
+		wxString url = GetVideoFilename(true);
+		if (url.length()) {
+			url.AfterLast(wxT('#')).ToLong(&m_defaultPos);
+			m_videoPos = m_defaultPos;
+		}
+	}
+}
+
+void MenuObjectPropDlg::OnChangeChapter(wxCommandEvent& evt) {
+	// reset video position
+	wxString url = GetVideoFilename(true);
+	if (url.length()) {
+		bool isDefaultPos = m_videoPos == m_defaultPos;
+		url.AfterLast(wxT('#')).ToLong(&m_defaultPos);
+		if (isDefaultPos)
+			m_videoPos = m_defaultPos;
 	}
 }
 
@@ -423,13 +494,13 @@ void MenuObjectPropDlg::OnJumpActionSelected(wxCommandEvent& evt) {
 	m_chapterChoice->Enable(m_chapterChoice->GetCount() > 1 && (m_dvd->IsJumppadsEnabled() || m_tsi != -1));
 	m_customActionEdit->Enable(false);
 	if (m_videoFrameRadio) {
-		m_videoFrameRadio->Enable(GetVideoFilename().length());
-		if (GetVideoFilename().length() == 0) {
+		m_videoFrameRadio->Enable(GetVideoFilename(false).length());
+		if (GetVideoFilename(false).length() == 0) {
 			m_imageRadio->SetValue(true);
 			wxCommandEvent event(wxEVT_COMMAND_RADIOBUTTON_SELECTED, m_imageRadio->GetId());
 			GetEventHandler()->ProcessEvent(event);
 		}
-		else if (GetVideoFilename().length() && m_imageEdit->GetValue().length() == 0)
+		else if (GetVideoFilename(false).length() && m_imageEdit->GetValue().length() == 0)
 			m_videoFrameRadio->SetValue(true);
 		wxCommandEvent event(wxEVT_COMMAND_RADIOBUTTON_SELECTED, m_videoFrameRadio->GetId());
 		GetEventHandler()->ProcessEvent(event);
@@ -466,46 +537,63 @@ wxString MenuObjectPropDlg::GetVideoFilename(bool withTimestamp, long position)
 		pgci = GetSelectedPgci();
 		chapter = GetSelectedChapter();
 	}
-	
+
 	if (tsi < 0 || tsi >= (int) m_dvd->GetTitlesets().size())
 		return wxT("");
 	Pgc* pgc = m_dvd->GetPgc(tsi, false, pgci);
 	if (!pgc)
 		return wxT("");
-	return withTimestamp ? pgc->GetVideoFrameUri(chapter, position) : pgc->GetVideoFrameUri(-1);
+	return withTimestamp ? pgc->GetVideoFrameUri(chapter, position) : pgc->GetVideoFrameUri(chapter, -1, true);
 }
 
 void MenuObjectPropDlg::OnImageRadio(wxCommandEvent& evt){
+	m_imageEdit->Enable(m_imageRadio->GetValue());
+	wxWindow* button = FindWindowByName(wxString::Format(wxT("SelectFileButton_%d"), m_imageEditIdx));
+	if (button != NULL)
+		button->Enable(m_imageRadio->GetValue());
 	if (m_videoFrameRadio->IsEnabled())
 		m_displayVideoFrame = m_videoFrameRadio->GetValue();
-	m_videoFrameChoice->Enable(m_videoFrameRadio->GetValue());
-	wxCommandEvent event(wxEVT_COMMAND_CHOICE_SELECTED, m_videoFrameChoice->GetId());
-	event.SetEventObject(m_videoFrameChoice);
-	event.SetInt(m_videoFrameChoice->GetSelection());
-	GetEventHandler()->ProcessEvent(event);
+	m_videoFrameBt->Enable(m_videoFrameRadio->GetValue());
+}
+
+void MenuObjectPropDlg::OnVideoFrame(wxCommandEvent& evt) {
+	VideoFrameDlg dlg(this, GetVideoFilename(false), false, m_defaultPos, m_videoPos, m_videoDuration);
+	if (dlg.ShowModal() == wxID_OK) {
+		m_videoPos = dlg.GetPos();
+		m_videoDuration = dlg.GetDuration();
+	}
 }
 
 void MenuObjectPropDlg::OnKeepAspectRatio(wxCommandEvent& evt) {
 	m_heightEdit->SetValue(m_widthEdit->GetValue());
 }
 
-void MenuObjectPropDlg::AddDirectionProp(wxSizer* sizer, ButtonDirCommand dir) {
+void MenuObjectPropDlg::AddFocusDestProp(wxSizer* sizer, NavigationButton navButton) {
 	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 1, 4, 16);
 	AddChoiceProp(grid, wxT(""), wxT(""), wxArrayString());
 	wxChoice* ctrl = (wxChoice*) m_controls[m_controls.Count()-1];
+	
+	// default navigation
+	wxString defBtId = m_object->GetDefaultFocusDest(navButton);
+	wxString text = _("none");
+	if (defBtId.length()) {
+		long l = -1;
+		if (defBtId.Mid(6).ToLong(&l))
+			text = _("button") + wxString::Format(wxT(" %d"), (int) l);
+	}
+	ctrl->Append(_("auto") + wxString(wxT(" (")) + text + wxT(")"), new wxString);
 
 	// add all buttons except current
-	ctrl->Append(_("auto"), new wxString);
 	int sel = 0;
 	for (unsigned int i=0; i<m_menu->GetObjectsCount(); i++) {
 		MenuObject* obj = m_menu->GetObject(i);
 		if (obj->IsButton() && obj != m_object) {
 			wxString id = obj->GetId();
-			if (m_object->GetDirection(dir-bdLEFT) == id)
+			if (m_object->GetFocusDest(navButton) == id)
 			sel = ctrl->GetCount();
-			long l = 0;
+			long l = -1;
 			id.Mid(6).ToLong(&l);
-			wxString text = _("button") + wxString::Format(wxT(" %d"), l);
+			wxString text = _("button") + wxString::Format(wxT(" %d"), (int) l);
 			if (obj->GetParam(wxT("text")).length())
 				text += wxT(": ") + obj->GetParam(_T("text"));
 			ctrl->Append(text, new wxString(id));
@@ -514,12 +602,14 @@ void MenuObjectPropDlg::AddDirectionProp(wxSizer* sizer, ButtonDirCommand dir) {
 	for (unsigned int i=0; i<m_menu->GetActionsCount(); i++) {
 		DVDAction* action = m_menu->GetAction(i);
 		wxString id = action->GetId();
-		if (m_object->GetDirection(dir-bdLEFT) == id)
+		if (m_object->GetFocusDest(navButton) == id)
 			sel = ctrl->GetCount();
 		ctrl->Append(id, new wxString(id));
 	}
 	ctrl->SetSelection(sel);
+#ifdef __WXMSW__
 	ctrl->SetSizeHints(ctrl->GetSize().GetWidth()*3/2, -1, ctrl->GetSize().GetWidth()*3/2, -1);
+#endif
 	sizer->Add(grid, 0, 0, 0);
 }
 
@@ -539,9 +629,9 @@ bool MenuObjectPropDlg::SetValues() {
 				return false;
 		}
 	}
-	
+
 	int n = 0;
-	
+
 	// set action
 	if (m_object->IsButton() && !m_multObjects) {
 		DVDAction& action = m_object->GetAction();
@@ -570,21 +660,21 @@ bool MenuObjectPropDlg::SetValues() {
 		n++;
 		action.SetAudio(GetInt(n++) - 1);
 		action.SetSubtitle(GetInt(n++) - 1);
-		
+
 		wxString* id = (wxString*) GetClientData(n++);
-		m_object->SetDirection(2, *id);
+		m_object->SetFocusDest(nbUP, *id);
 		delete id;
 		id = (wxString*) GetClientData(n++);
-		m_object->SetDirection(0, *id);
+		m_object->SetFocusDest(nbLEFT, *id);
 		delete id;
 		id = (wxString*) GetClientData(n++);
-		m_object->SetDirection(1, *id);
+		m_object->SetFocusDest(nbRIGHT, *id);
 		delete id;
 		id = (wxString*) GetClientData(n++);
-		m_object->SetDirection(3, *id);
+		m_object->SetFocusDest(nbDOWN, *id);
 		delete id;
 	}
-	
+
 	// set look parameters
 	for (int i = 0; i < m_object->GetObjectParamsCount(); i++) {
 		MenuObjectParam* param = m_object->GetObjectParam(i);
@@ -607,31 +697,37 @@ bool MenuObjectPropDlg::SetValues() {
 			} else if (param->type == _T("image") && !m_multObjects) {
 				if (m_object->IsButton()) {
 					m_object->SetDisplayVideoFrame(m_displayVideoFrame);
-					bool image = GetBool(n++);
+					bool image = GetBool(n++); // image radio button
 					wxString imgFile = GetString(n++);
-					n++;
+					n++; // video radio button
 					if (!image) {
-						int videoFrame = -1;
-						if (GetInt(n++) == 1) // custom frame
-							videoFrame = GetInt(n++);
-						else
-							n++;
-						m_object->SetCustomVideoFrame(videoFrame != -1);
-						imgFile = GetVideoFilename(true, videoFrame);
+						m_object->SetCustomVideoFrame(m_videoPos != m_defaultPos);
+						m_object->SetParamImageVideo(param->name, GetVideoFilename(false), m_videoPos, m_videoDuration);
 					} else
-						n += 2;
-					m_object->SetParam(param->name, imgFile);
+						m_object->SetParamImageVideo(param->name, imgFile, -1, -1);
 				} else
 					m_object->SetParam(param->name, GetString(n++));
+			} else if (param->type == wxT("shadow")) {
+				m_object->SetParam(param->name, GetBool(n++) ? wxT("visible") : wxT("hidden"), wxT("visibility"));
+				m_object->SetParamColour(param->name, GetColour(n++));
+				wxString opacity = wxString::Format(wxT("%g"), (double) GetInt(n++) / 100);
+				m_object->SetParam(param->name, opacity, wxT("-opacity"));
+				m_object->SetParamInt(param->name, GetInt(n++), wxT("x"));
+				m_object->SetParamInt(param->name, GetInt(n++), wxT("y"));
+				wxSVGElement* filter = m_object->GetElementById(param->element + wxT("Filter"));
+				if (filter && ((wxSVGElement*)filter->GetFirstChild())->GetDtd() == wxSVG_FEGAUSSIANBLUR_ELEMENT) {
+					wxSVGFEGaussianBlurElement* blur = (wxSVGFEGaussianBlurElement*) filter->GetFirstChild();
+					blur->SetAttribute(wxT("stdDeviation"), GetString(n++));
+				}
 			}
 		}
 	}
-	
+
 	if (!m_multObjects) {
 		// set position
 		m_object->SetX(GetInt(n++));
 		m_object->SetY(GetInt(n++));
-		
+
 		// set size
 		m_object->SetDefaultSize(!GetBool(n++));
 		if (!m_object->IsDefaultSize()) {
@@ -641,6 +737,6 @@ bool MenuObjectPropDlg::SetValues() {
 		}
 	}
 	m_object->UpdateSize();
-	
+
 	return true;
 }
