@@ -3,7 +3,7 @@
 // Purpose:     DVD menu properties dialog
 // Author:      Alex Thuering
 // Created:     25.04.2004
-// RCS-ID:      $Id: MenuPropDlg.cpp,v 1.39 2011/08/14 08:43:49 ntalex Exp $
+// RCS-ID:      $Id: MenuPropDlg.cpp,v 1.41 2012/03/31 11:36:12 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -11,6 +11,8 @@
 #include "MenuPropDlg.h"
 #include "TitlesetManager.h"
 #include "Config.h"
+#include "MenuCellsDlg.h"
+#include "MenuActionsDlg.h"
 #include <wxVillaLib/utils.h>
 #include <wx/file.h>
 #include "rc/preferences.png.h"
@@ -18,13 +20,17 @@
 enum {
 	ASPECT_CHOICE_ID = 7850,
 	AUDIO_CTRL_ID,
-	LOOP_CB_ID
+	LOOP_CB_ID,
+	CELLS_BT_ID,
+	ACTIONS_BT_ID
 };
 
 BEGIN_EVENT_TABLE(MenuPropDlg, wxPropDlg)
 	EVT_CHOICE(ASPECT_CHOICE_ID, MenuPropDlg::OnChangeAspect)
 	EVT_TEXT(AUDIO_CTRL_ID, MenuPropDlg::OnChangeAudio)
 	EVT_CHECKBOX(LOOP_CB_ID, MenuPropDlg::OnLoopCheck)
+	EVT_BUTTON(CELLS_BT_ID, MenuPropDlg::OnCellsBt)
+	EVT_BUTTON(ACTIONS_BT_ID, MenuPropDlg::OnActionsBt)
 END_EVENT_TABLE()
 
 MenuPropDlg::MenuPropDlg(wxWindow *parent, DVD* dvd, int tsi, int pgci):
@@ -35,6 +41,8 @@ MenuPropDlg::MenuPropDlg(wxWindow *parent, DVD* dvd, int tsi, int pgci):
 	m_pgc = pgcs[pgci];
 	m_vob = m_dvd->GetMenuVob(tsi, pgci);
 	m_menu = m_vob->GetMenu();
+	VECTOR_COPY(m_vob->GetCells(), m_cells, Cell)
+	VECTOR_COPY(m_menu->GetActions(), m_actions, DVDAction)
 	m_audioFormat = NULL;
 	// set title
 	wxString title = _("Properties") + wxString(wxT(" - "));
@@ -50,6 +58,12 @@ MenuPropDlg::MenuPropDlg(wxWindow *parent, DVD* dvd, int tsi, int pgci):
 	SetSize(400,-1);
 }
 
+
+MenuPropDlg::~MenuPropDlg() {
+	VECTOR_CLEAR(m_cells, Cell)
+	VECTOR_CLEAR(m_actions, DVDAction)
+}
+
 void MenuPropDlg::CreatePropPanel(wxSizer* sizer) {
 	wxSizer* formatSizer = new wxBoxSizer(wxHORIZONTAL);
 	AddText(formatSizer, _("Format:"));
@@ -79,7 +93,7 @@ void MenuPropDlg::CreatePropPanel(wxSizer* sizer) {
 
 void MenuPropDlg::CreateBackgroundGroup(wxSizer* sizer) {
 	wxSizer* grpSizer = BeginGroup(sizer, _("Background"));
-	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 2, 4, 16);
+	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 4, 16);
 	grid->AddGrowableCol(1);
 	// Image
 	AddFileProp(grid, _("Image:"), m_menu->GetBackground(), wxFD_OPEN, wxT("..."),
@@ -119,7 +133,7 @@ void MenuPropDlg::CreateBackgroundGroup(wxSizer* sizer) {
 
 void MenuPropDlg::CreateButtonsGroup(wxSizer* sizer) {
 	wxSizer* grpSizer = BeginGroup(sizer, _("Buttons"));
-	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 2, 4, 16);
+	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 4, 16);
 	grid->AddGrowableCol(1);
 	// Start/End
 	wxSizer* timeSizer = AddTextProp(grid, _("Start/End:"), m_menu->GetStartTime(), false, 100, false);
@@ -163,6 +177,9 @@ void MenuPropDlg::CreateVobGroup(wxSizer* sizer, bool loop) {
 		m_langCodes->Add(s_config.GetDefAudioLanguage());
 	formatSizer->AddSpacer(4);
 	AddChoiceProp(formatSizer, wxT(""), (*m_langCodes)[0], labels, 0, false);
+	// cells & actions
+	AddButton(grid, _("Cells:"), CELLS_BT_ID);
+	AddButton(grid, _("Actions:"), ACTIONS_BT_ID);
 	// pause and loop
 	wxSizer* pSizer = AddSpinProp(grid, _("Pause:"), m_vob->GetPause(), -1, 254, 100, _("sec"),false);
 	m_pauseCtrl = (wxSpinCtrl*) GetLastControl();
@@ -301,6 +318,8 @@ bool MenuPropDlg::SetValues() {
 	n++;
 	m_langCodes->RemoveAt(0);
 	m_langCodes->Insert(GetString(n++), 0);
+	m_vob->GetCells().swap(m_cells);
+	m_menu->GetActions().swap(m_actions);
 	m_vob->SetPause(GetInt(n++));
 	n++;
 	
@@ -366,3 +385,15 @@ void MenuPropDlg::OnLoopCheck(wxCommandEvent& event) {
 	m_postCommandsCtrl->SetValue(event.IsChecked() ? wxT("jump cell 1;"): wxT(""));
 	m_postCommandsCtrl->Enable(!event.IsChecked());
 }
+
+void MenuPropDlg::OnCellsBt(wxCommandEvent& event) {
+	MenuCellsDlg dlg(this, m_dvd, m_tsi, m_cells);
+	if (dlg.ShowModal() == wxID_OK)
+		m_cells.swap(dlg.GetCells());
+}
+
+void MenuPropDlg::OnActionsBt(wxCommandEvent& event) {
+	MenuActionsDlg dlg(this, m_dvd, m_tsi, m_actions);
+	if (dlg.ShowModal() == wxID_OK)
+		m_actions.swap(dlg.GetActions());
+}
