@@ -3,7 +3,7 @@
 // Purpose:     The class to store a PGC
 // Author:      Alex Thuering
 // Created:     29.01.2003
-// RCS-ID:      $Id: Pgc.cpp,v 1.14 2011/11/23 19:18:49 ntalex Exp $
+// RCS-ID:      $Id: Pgc.cpp,v 1.30 2012/10/30 19:23:11 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -54,12 +54,12 @@ void Cell::SetStart(const wxString& startStr) {
 
 /** Returns the end time as string */
 wxString Cell::GetEndStr() const {
-	return Time2String(m_end);
+	return m_end >= 0 ? Time2String(m_end) : wxT("");
 }
 
 /** Sets the end time */
 void Cell::SetEnd(const wxString& endStr) {
-	m_end = String2Time(endStr);
+	m_end = endStr.length() ? String2Time(endStr) : -1;
 }
 
 wxSvgXmlNode* Cell::GetXML(DVDFileType type) {
@@ -72,8 +72,10 @@ wxSvgXmlNode* Cell::GetXML(DVDFileType type) {
 		node->AddProperty(wxT("chapter"), wxT("1"));
 	if (m_program)
 		node->AddProperty(wxT("program"), wxT("1"));
-	if (m_pause != 0)
-		node->AddProperty(wxT("pause"), m_pause == -1 ? wxT("inf") : wxString::Format(wxT("%d"), m_pause));
+	if (m_pause != 0) {
+		wxString pauseStr = m_pause == -1 ? wxT("inf") : wxString::Format(wxT("%d"), m_pause);
+		node->AddProperty(wxT("pause"), pauseStr);
+	}
 	if (m_commands.length())
 		node->AddChild(new wxSvgXmlNode(wxSVGXML_TEXT_NODE, wxEmptyString, m_commands));
 	return node;
@@ -107,17 +109,22 @@ bool Cell::PutXML(wxSvgXmlNode* node) {
 Stream::Stream(StreamType type, wxString codecName) {
 	m_type = type;
 	m_sourceCodecName = codecName;
+	m_sourceBitrate = 0;
+	m_sourceSampleRate = 0;
 	m_sourceChannelNumber = -1;
 	m_sourceAspectRatio = -1;
 	m_sourceFps = -1;
 	m_destinationFormat = 1; // vfCOPY/afCOPY/sfCOPY
 	m_audioVolume = 256;
+	m_channelNumber = -1;
+	m_tsOffset = 0;
 }
 
 VideoFormat Stream::GetSourceVideoFormat() {
-	if (m_sourceVideoSize.GetWidth() == 720 && m_sourceVideoSize.GetHeight() == 576)
+	int fps = lroundf(m_sourceFps);
+	if (m_sourceVideoSize.GetWidth() == 720 && m_sourceVideoSize.GetHeight() == 576 && fps == 25)
 		return vfPAL;
-	else if (m_sourceVideoSize.GetWidth() == 720 && m_sourceVideoSize.GetHeight() == 480)
+	else if (m_sourceVideoSize.GetWidth() == 720 && m_sourceVideoSize.GetHeight() == 480 && (fps == 24 || fps == 30))
 		return vfNTSC;
 	return vfNONE;
 }
@@ -140,12 +147,16 @@ wxString Stream::GetSourceFormat() {
 	switch (m_type) {
 	case stVIDEO:
 		if (m_sourceVideoSize.IsFullySpecified()) {
-			result += wxString::Format(wxT(", %dx%d"),
-					m_sourceVideoSize.GetWidth(), m_sourceVideoSize.GetHeight());
-			if (GetSourceVideoFormat() == vfPAL)
+			result += wxString::Format(wxT(", %dx%d"), m_sourceVideoSize.GetWidth(), m_sourceVideoSize.GetHeight());
+			int fps = lroundf(m_sourceFps);
+			if (fps == 25)
 				result += wxT(" (PAL)");
-			else if (GetSourceVideoFormat() == vfNTSC)
+			else if (fps == 30)
 				result += wxT(" (NTSC)");
+			else if (fps == 24)
+				result += wxT(" (NTSC film)");
+			else if (fps > 0)
+				result += wxString::Format(wxT(" (%d fps)"), fps);
 			if (round(GetSourceAspectRatio()*100) == 133 || round(GetSourceAspectRatio()*100) == 136)
 				result += wxT(", 4:3");
 			else if (round(GetSourceAspectRatio()*100) == 178)
@@ -162,7 +173,7 @@ wxString Stream::GetSourceFormat() {
 			else if (m_sourceChannelNumber == 2)
 				result += _("stereo");
 			else if (m_sourceChannelNumber == 6)
-				result += wxT("5:1");
+				result += wxT("5.1");
 			else
 				result += wxString::Format(_("%d channels"), m_sourceChannelNumber);
 		}
@@ -195,6 +206,7 @@ Vob::Vob(Slideshow* slideshow) {
 
 void Vob::Init(Menu* menu, Slideshow* slideshow) {
 	m_pause = 0;
+	m_duration = 0;
 	m_startTime = 0;
 	m_recordingTime = -1;
 	m_menu = menu;
@@ -245,11 +257,12 @@ Vob::~Vob() {
 	VECTOR_CLEAR(m_cells, Cell)
 }
 
-bool Vob::SetFilename(const wxString& value) {
-	m_filename = value;
+bool Vob::SetFilename(const wxString& filename) {
 	wxFfmpegMediaDecoder ffmpeg;
-	bool ok = m_filename.length() && ffmpeg.Load(m_filename);
-	m_duration = ok && ffmpeg.GetDuration() > 0 ? ffmpeg.GetDuration() : 0;
+	if (filename.length() == 0 || !ffmpeg.Load(filename))
+		return false;
+	m_filename = filename;
+	m_duration = ffmpeg.GetDuration() > 0 ? ffmpeg.GetDuration() : 0;
 	if (m_streams.GetCount() - m_audioFilenames.GetCount() > 0)
 		m_streams.RemoveAt(0, m_streams.GetCount() - m_audioFilenames.GetCount());
 	for (unsigned int stIdx = 0; stIdx < ffmpeg.GetStreamCount(); stIdx++) {
@@ -266,7 +279,7 @@ bool Vob::SetFilename(const wxString& value) {
 		}
 		m_streams.Insert(stream, stIdx);
 	}
-	return ok;
+	return true;
 }
 
 bool Vob::HasAudio() {
@@ -324,6 +337,16 @@ bool Vob::AddAudioFile(wxString filename) {
 	return true;
 }
 
+/** Returns name of file to display */
+wxString Vob::GetFilenameDisplay() {
+	if (m_filename.StartsWith(wxT("concat:"))) {
+		wxString fname = m_filename.Mid(7).BeforeFirst(wxT('|'));
+		int idx = fname.Find(wxT("VIDEO_TS"));
+		return idx > 0 ? fname.Mid(0, idx - 1) : fname;
+	}
+	return m_filename;
+}
+
 /** Removes audio file with given index from the vob */
 void Vob::RemoveAudioFile(int index) {
 	m_streams.RemoveAt(m_streams.GetCount() - m_audioFilenames.GetCount() + index);
@@ -428,8 +451,7 @@ wxString Vob::GetAllVideoFilters() {
 		if (result.length())
 			result += wxT(',');
 		double fps = isNTSC(videoFormat) ? 30000/1001 : 25;
-		long startFrame = GetStartTime() * fps;
-		result += wxString::Format(wxT("fade=in:%d:%d"), startFrame, (int) (GetFadeIn() * fps));
+		result += wxString::Format(wxT("fade=in:%d:%d"), 0, (int) (GetFadeIn() * fps));
 	}
 	if (GetFadeOut() > 0) {
 		if (result.length())
@@ -457,18 +479,38 @@ Stream* Vob::GetVideoStream() {
 	return NULL;
 }
 
+/** Returns video stream index */
+int Vob::GetVideoStreamIndex() {
+	for (int i = 0; i < (int)GetStreams().GetCount(); i++)
+		if (GetStreams()[i]->GetType() == stVIDEO)
+			return i;
+	return -1;
+}
+
 void ReplaceCallLastMenu(wxString& commands, DVD* dvd) {
 	if (dvd->GetLastMenuRegister() == -1)
 		return;
 	wxString cmd = wxT("call menu;");
-	int stPos = commands.Index(cmd);
+	int stPos = commands.Find(cmd);
 	if (stPos >= 0)
-		commands = commands.Mid(0, stPos) + wxString::Format(wxT("g%d = 0;"), dvd->GetLastMenuRegister())
-			+ commands.Mid(stPos);
+		commands.replace(stPos, cmd.length(), wxString::Format(wxT("g%d = 0;"), dvd->GetLastMenuRegister()));
 	cmd = wxT("call last menu");
 	stPos = commands.Find(cmd);
 	if (stPos >= 0)
-		commands = commands.Mid(0, stPos) + wxT("call menu") + commands.Mid(stPos + cmd.length());
+		commands.replace(stPos, cmd.length(), wxT("call menu entry root"));
+}
+
+void ReplaceJumpNextTitle(wxString& commands, DVD* dvd, wxString nextTitle) {
+	wxString cmd = wxT("jump next title;");
+	int stPos = commands.Find(cmd);
+	if (stPos >= 0) {
+		wxString jmp;
+		if (nextTitle.length())
+			jmp = wxT("jump ") + nextTitle + wxT(';');
+		else
+			jmp = dvd->HasMenus() ? wxT("call menu entry root;") : wxT("exit;");
+		commands.replace(stPos, cmd.length(), jmp);
+	}
 }
 
 wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
@@ -520,6 +562,10 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 			} else if (stream->GetType() == stAUDIO) {
 				if (stream->GetAudioVolume() != 256)
 					streamNode->AddProperty(wxT("volume"), wxString::Format(wxT("%d"), stream->GetAudioVolume()));
+				if (stream->GetChannelNumber() != -1)
+					streamNode->AddProperty(wxT("channelNumber"), wxString::Format(wxT("%d"), stream->GetChannelNumber()));
+				if (stream->GetTsOffset() != 0)
+					streamNode->AddProperty(wxT("tsOffset"), wxString::Format(wxT("%ld"), stream->GetTsOffset()));
 			}
 			node->AddChild(streamNode);
 		}
@@ -530,6 +576,10 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 			audioNode->AddProperty(wxT("format"), wxString::Format(wxT("%d"), stream->GetDestinationFormat()));
 			if (stream->GetAudioVolume() != 256)
 				audioNode->AddProperty(wxT("volume"), wxString::Format(wxT("%d"), stream->GetAudioVolume()));
+			if (stream->GetChannelNumber() != -1)
+				audioNode->AddProperty(wxT("channelNumber"), wxString::Format(wxT("%d"), stream->GetChannelNumber()));
+			if (stream->GetTsOffset() != 0)
+				audioNode->AddProperty(wxT("tsOffset"), wxString::Format(wxT("%ld"), stream->GetTsOffset()));
 			node->AddChild(audioNode);
 		}
 		for (int i = 0;  i < (int) GetSubtitles().GetCount(); i++) {
@@ -606,10 +656,14 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 			} else
 				node->AddChild(cell->GetXML(type));
 		}
-	} else if (m_cells.size() > 0) {
-		node->AddProperty(wxT("chapters"), GetChapters());
-		if (GetPause() != 0)
-			node->AddProperty(wxT("pause"), GetPause() > 0 ? wxString::Format(wxT("%d"), GetPause()) : wxT("inf"));
+	} else {
+		if (m_cells.size() > 0)
+			node->AddProperty(wxT("chapters"), GetChapters());
+		
+		if (GetPause() != 0) {
+			wxString pauseStr = GetPause() > 0 ? wxString::Format(wxT("%d"), GetPause()) : wxT("inf");
+			node->AddProperty(wxT("pause"), pauseStr);
+		}
 	}
 	
 	if (GetMenu() && type == DVDSTYLER_XML)
@@ -697,6 +751,10 @@ bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci, bool menu) {
 						m_streams[stIdx]->SetDestinationFormat(lval);
 					if (child->GetPropVal(wxT("volume"), &val) && val.length() > 0 && val.ToLong(&lval))
 						m_streams[stIdx]->SetAudioVolume(lval);
+					if (child->GetPropVal(wxT("channelNumber"), &val) && val.length() > 0 && val.ToLong(&lval))
+						m_streams[stIdx]->SetChannelNumber(lval);
+					if (child->GetPropVal(wxT("tsOffset"), &val) && val.length() > 0 && val.ToLong(&lval))
+						m_streams[stIdx]->SetTsOffset(lval);
 					stIdx++;
 				}
 			} else if (child->GetPropVal(wxT("format"), &val) && val.length() > 0 && val.ToLong(&lval)) {
@@ -707,6 +765,21 @@ bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci, bool menu) {
 						m_streams[stIdx]->SetDestinationFormat(lval);
 						if (child->GetPropVal(wxT("volume"), &val) && val.length() > 0 && val.ToLong(&lval))
 							m_streams[stIdx]->SetAudioVolume(lval);
+						if (child->GetPropVal(wxT("channelNumber"), &val) && val.length() > 0 && val.ToLong(&lval))
+							m_streams[stIdx]->SetChannelNumber(lval);
+						stIdx++;
+						break;
+					}
+					stIdx++;
+				}
+			}
+		} else if (child->GetName() == wxT("subtitle")) {
+			if (child->GetPropVal(wxT("format"), &val) && val.length() > 0 && val.ToLong(&lval) && lval != 1) {
+				if ((int)m_streams.GetCount() <= stIdx || m_streams[stIdx]->GetType() != stSUBTITLE)
+					stIdx = 0;
+				while ((int)m_streams.GetCount() > stIdx) {
+					if (m_streams[stIdx]->GetType() == stSUBTITLE) {
+						m_streams[stIdx]->SetDestinationFormat(lval);
 						stIdx++;
 						break;
 					}
@@ -761,19 +834,25 @@ bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci, bool menu) {
 	return true;
 }
 
-unsigned int Vob::GetFileSize(wxString filename) {
+unsigned int Vob::GetFileSize(const wxString& filename) {
+	if (filename.StartsWith(wxT("concat:"))) {
+		unsigned int size = 0;
+		wxString files = filename.substr(7);
+		while (files.length() > 0) {
+			wxString fname = files.BeforeFirst(wxT('|'));
+			files = files.AfterFirst(wxT('|'));
+			size += wxFile(fname).Length()/1024;
+		}
+		return size;
+	}
 	return wxFile(filename).Length()/1024;
 }
 
-int Vob::GetSize(DVD* dvd, bool generated) {
-	if (generated) {
-		if (GetTmpFilename().length())
-			return GetFileSize(GetTmpFilename());
-		else if (GetFilename().length())
-			return GetFileSize(GetFilename());
-		return 0;
-	}
-
+/**
+ * Returns size in KB
+ * @return Size of VOB file in KB
+ */
+int Vob::GetSize(DVD* dvd) {
 	long size = 0;
 	int stIdx = 0;
 	if (GetMenu()) {
@@ -832,6 +911,18 @@ int Vob::GetSize(DVD* dvd, bool generated) {
 	return size;
 }
 
+/**
+ * Returns size after transcoding in KB
+ * @return Size of VOB file in KB
+ */
+int Vob::GetTranscodedSize(DVD* dvd) {
+	if (GetTmpFilename().length())
+		return GetFileSize(GetTmpFilename());
+	else if (GetFilename().length())
+		return GetFileSize(GetFilename());
+	return 0;
+}
+
 int Vob::GetRequiredSize(DVD* dvd, Cache* cache) {
 	int size = GetSize(dvd);
 	if (GetMenu() || GetSlideshow() || GetSubtitles().Count() > 0
@@ -883,9 +974,11 @@ Slideshow* Pgc::GetSlideshow() {
 	return NULL;
 }
 
-unsigned int Pgc::GetChapterCount() {
+unsigned int Pgc::GetChapterCount(int lastVobi) {
+	if (lastVobi == -1)
+		lastVobi = (int) GetVobs().Count();
 	int result = 1;
-	for (int vi = 0; vi < (int) GetVobs().Count(); vi++) {
+	for (int vi = 0; vi < lastVobi; vi++) {
 		Vob& vob = *GetVobs()[vi];
 		if (vob.GetSlideshow()) {
 			result += vob.GetSlideshow()->GetCount() / 5;
@@ -902,7 +995,7 @@ unsigned int Pgc::GetChapterCount() {
 }
 
 /** Returns URI of video frame */
-wxString Pgc::GetVideoFrameUri(int chapter, long position) {
+wxString Pgc::GetVideoFrameUri(int chapter, long position, bool fileNameOnly) {
 	if (GetVobs().size() == 0)
 		return wxT("");
 	wxString uri;
@@ -926,8 +1019,8 @@ wxString Pgc::GetVideoFrameUri(int chapter, long position) {
 			position = lround((vob->GetStartTime() + dpos) * 1000);
 		}
 	}
-	if (position >= 0 && uri.length() > 0)
-		uri += wxT('#') + wxString::Format(wxT("%d"), position);
+	if (!fileNameOnly && position >= 0 && uri.length() > 0)
+		uri += wxT('#') + wxString::Format(wxT("%ld"), position);
 	return uri;
 }
 
@@ -997,8 +1090,10 @@ wxSvgXmlNode* Pgc::GetXML(DVDFileType type, DVD* dvd, PgcArray& pgcs, wxString n
 				preCommands = wxString::Format(wxT("if (%sg%d>0) button = g%d; else button = 1024;%s"),
 						expr1.c_str(), lastBtReg, lastBtReg, preCommands.c_str());
 			}
-		} else
+		} else {
 			ReplaceCallLastMenu(preCommands, dvd);
+			ReplaceJumpNextTitle(preCommands, dvd, nextTitle);
+		}
 	}
 	if (preCommands.length())
 		XmlWriteValue(node, wxT("pre"), preCommands);
@@ -1006,6 +1101,7 @@ wxSvgXmlNode* Pgc::GetXML(DVDFileType type, DVD* dvd, PgcArray& pgcs, wxString n
 	wxString postCommands = GetPostCommands();
 	if (type == DVDAUTHOR_XML) {
 		ReplaceCallLastMenu(postCommands, dvd);
+		ReplaceJumpNextTitle(postCommands, dvd, nextTitle);
 		if (dvd->GetPlayAllRegister() != -1 && nextTitle.length()) {
 			wxString g = wxT("g") + wxString::Format(wxT("%d"), dvd->GetPlayAllRegister());
 			postCommands = wxT("if (") + g + wxT("==1) jump ") + nextTitle + wxT(";") + postCommands;
