@@ -3,7 +3,7 @@
 // Purpose:     Progress of generation/burning dialog
 // Author:      Alex Thuering
 // Created:     14.08.2004
-// RCS-ID:      $Id: ProgressDlg.cpp,v 1.122 2011/11/24 07:30:37 ntalex Exp $
+// RCS-ID:      $Id: ProgressDlg.cpp,v 1.140 2012/11/05 21:10:58 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -38,6 +38,7 @@ extern "C" {
 
 #define DATA_FILE(fname) wxFindDataFile(_T("data") + wxString(wxFILE_SEP_PATH) + fname)
 #define TMP_ISO wxT("dvd-out.iso")
+#define WARNING_MPLEX_MSG wxT("Warning: using of mplex (mjpegs-tool) for generation of menus with audio is disabled. This can produce not compliant DVD.")
 
 ////////////////////////// Process Execute ///////////////////////////////////
 
@@ -175,9 +176,39 @@ protected:
 	int     m_percent;
 };
 
+class AVConvExecute: public ProcessExecute {
+public:
+	AVConvExecute(ProgressDlg* processDlg, long totalFrames): ProcessExecute(processDlg), m_percent(0),
+			m_pattern(wxT("frame=[[:space:]]+([0-9]+).*")) {
+		m_initSubStep = m_processDlg->GetSubStep();
+		m_totalFrames = totalFrames;
+	}
+	virtual ~AVConvExecute() {}
+
+	virtual void ProcessOutput(wxString line) {
+		if (line.Find(wxT("buffer underflow i=1")) >= 0
+				|| line.Find(wxT("packet too large, ignoring buffer limits")) >= 0
+				|| line.Find(wxT("Last message repeated 1 times")) >= 0)
+			return;
+		if (m_pattern.Matches(line)) {
+			long frame = 0;
+			m_pattern.GetMatch(line, 1).ToLong(&frame);
+			m_percent = (m_totalFrames > 0) ? (frame * 100) / m_totalFrames : 0;
+			m_processDlg->SetSubStep(m_initSubStep + (int) m_percent);
+		}
+		m_processDlg->AddDetailText(line + _T("\n"));
+	}
+
+protected:
+	long m_totalFrames;
+	int m_initSubStep;
+	int m_percent;
+	wxRegEx m_pattern;
+};
+
 class BurnExecute: public ProgressExecute {
 public:
-	BurnExecute(ProgressDlg* processDlg, wxString filter): ProgressExecute(processDlg, filter) {}
+	BurnExecute(ProgressDlg* processDlg, wxString filter): ProgressExecute(processDlg, filter), m_burnOk(false) {}
 
 	virtual void ProcessOutput(wxString line) {
 		if (line.Find(wxT(": writing lead-out")) >= 0)
@@ -261,6 +292,7 @@ protected:
 	int m_warnStep;
 	int m_dvdauthorStep;
 };
+
 /////////////////////////// Process Dialog ///////////////////////////////////
 class ProgressDlgLog: public wxLog {
 public:
@@ -418,6 +450,8 @@ void ProgressDlg::OnCancel(wxCommandEvent& WXUNUSED(event)) {
 }
 
 void ProgressDlg::End() {
+	if (!m_cancel)
+		wxBell();
 	m_cancelBt->SetLabel(_("Close"));
 	m_cancel = true;
 }
@@ -444,8 +478,10 @@ bool ProgressDlg::Start(BurnDlg* burnDlg, DVD* dvd) {
 	if (m_autoStart) {
 		Close(true);
 	} else {
-    	while (!m_end)
+    	while (!m_end) {
+    		wxMilliSleep(100);
     		wxYield();
+    	}
     }
 	MakeModal(false);
 	return isOk;
@@ -466,7 +502,8 @@ int ProgressDlg::GetMenuSubSteps(wxArrayPtrVoid& menuVobs, wxArrayInt& menuWSTyp
 			// calculate sub steps
 			Menu* menu = vob->GetMenu();
 			WidescreenType widescreenType = pgcs.GetVideo().GetWidescreen();
-			if (menu->HasVideoBackground()) {
+			bool videoMenu = menu->GetSVG()->GetDuration() > 0;
+			if (videoMenu) {
 				menuSubSteps += 400; // generate mpeg(200) + transcode(200)
 			} else if (vob->GetAudioFilenames().size() > 0)
 				menuSubSteps += 300; // generate mpeg(25+75) + transcode(200)
@@ -639,10 +676,7 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd) {
 		AddSummaryMsg(_("Burning was successful."), wxEmptyString, wxColour(0, 128, 0));
 	else
 		AddSummaryMsg(_("Generating was successful."), wxEmptyString, wxColour(0, 128, 0));
-	IncStep();
 	wxLog::FlushActive();
-
-	s_config.SetKey(s_config.GetKey() + 1);
 }
 
 bool ProgressDlg::CleanTemp(const wxString& tmpDir, const wxString& dvdTmpDir, const wxString& dvdOutDir) {
@@ -674,12 +708,12 @@ bool ProgressDlg::GenerateMenus(wxArrayPtrVoid& menuVobs, wxArrayInt& menuWSType
 		return false;
 	AddSummaryMsg(_("Generating menus"));
 	SetSubSteps(menuSubSteps);
-	for (int i = 0; i < (int) menuVobs.Count(); i++) {
-		AddDetailMsg(wxString::Format(_("Generating menu %d of %d"), i + 1, menuVobs.Count()));
+	for (unsigned int i = 0; i < menuVobs.Count(); i++) {
+		AddDetailMsg(wxString::Format(_("Generating menu %u of %lu"), i + 1, menuVobs.Count()));
 		Vob* vob = (Vob*) menuVobs[i];
 		WidescreenType wsType = (WidescreenType) menuWSTypes[i];
-		if (!GenerateMenu(vob->GetMenu(), wsType, vob->GetTmpFilename(), dvd->GetAudioFormat(),
-				vob->GetAudioFilenames().size() > 0 ? vob->GetAudioFilenames()[0] : wxT(""),
+		if (!GenerateMenu(vob->GetMenu(), wsType, vob->GetTmpFilename(), (AudioFormat) dvd->GetAudioFormat(),
+				vob->GetAudioFilenames().size() > 0 ? vob->GetAudioFilenames()[0] : wxString(),
 				dvd->GetVideoBitrate()))
 			return false;
 	}
@@ -694,14 +728,18 @@ bool ProgressDlg::Transcode(wxArrayPtrVoid& titleVobs, wxArrayInt& titleAspects,
 		return false;
 	AddSummaryMsg(_("Transcode/remultiplex"));
 	SetSubSteps(titleSubSteps);
+	int videoBitrate = dvd->GetVideoBitrate();
+	if (dvd->GetCapacity() != dcUNLIMITED && dvd->GetSize() > dvd->GetCapacityValue()*0.965)
+		videoBitrate *= 0.965;
 	for (int i = 0; i < (int) titleVobs.Count(); i++) {
 		Vob* vob = (Vob*) titleVobs[i];
 		AspectRatio aspect = (AspectRatio) titleAspects[i];
 		wxString cacheFile = m_cache->Add(vob, dvd);
 		AddDetailMsg(wxT("Add file to cache:") + cacheFile);
 		vob->SetTmpFilename(cacheFile);
-		if (!Transcode(vob, aspect, dvd->GetVideoBitrate(), s_config.GetUseMplex())) {
-			wxRemoveFile(vob->GetTmpFilename());
+		if (!Transcode(vob, aspect, videoBitrate, s_config.GetUseMplex(), dvd->GetAudioFormat())) {
+			if (wxFileExists(vob->GetTmpFilename()))
+				wxRemoveFile(vob->GetTmpFilename());
 			return false;
 		}
 	}
@@ -716,11 +754,11 @@ bool ProgressDlg::GenerateSlideshow(wxArrayPtrVoid& slideshowVobs, int slideshow
 		return false;
 	AddSummaryMsg(_("Generate slideshow"));
 	SetSubSteps(slideshowSubSteps);
-	for (int i = 0; i < (int) slideshowVobs.Count(); i++) {
-		AddDetailMsg(wxString::Format(_("Generating slideshow %d of %d"), i + 1, slideshowVobs.Count()));
+	for (unsigned int i = 0; i < slideshowVobs.Count(); i++) {
+		AddDetailMsg(wxString::Format(_("Generating slideshow %u of %lu"), i + 1, slideshowVobs.Count()));
 		Vob* vob = (Vob*) slideshowVobs[i];
 		if (!GenerateSlideshow(vob->GetSlideshow(), vob->GetTmpFilename(), dvd->GetAudioFormat(),
-				vob->GetAudioFilenames().size() > 0 ? vob->GetAudioFilenames()[0] : wxT("")))
+				vob->GetAudioFilenames().size() > 0 ? vob->GetAudioFilenames()[0] : wxString()))
 			return false;
 	}
 	IncStep();
@@ -735,8 +773,8 @@ bool ProgressDlg::MultiplexSubtitles(wxArrayPtrVoid& subtitleVobs, wxArrayInt& s
 		return false;
 	AddSummaryMsg(_("Multiplexing subtitles"));
 	SetSubSteps(subtitleSubSteps);
-	for (int i = 0; i < (int) subtitleVobs.Count(); i++) {
-		AddDetailMsg(wxString::Format(_("Multiplexing subtitles %d of %d"), i + 1, subtitleVobs.Count()));
+	for (unsigned int i = 0; i < subtitleVobs.Count(); i++) {
+		AddDetailMsg(wxString::Format(_("Multiplexing subtitles %u of %lu"), i + 1, subtitleVobs.Count()));
 		Vob* vob = (Vob*) subtitleVobs[i];
 		AspectRatio aspectRatio = dvd->GetTitlesets()[subtitleTsi[i]]->GetTitles().GetVideo().GetAspect();
 		
@@ -789,7 +827,7 @@ bool ProgressDlg::MultiplexSubtitles(const wxString& vobFile, TextSub* textSub,
 	wxString spuFile = resultFile + wxT("_spumux.xml");
 	textSub->SaveSpumux(spuFile, videoFormat);
 	cmd.Replace(wxT("$FILE_CONF"), spuFile);
-	cmd.Replace(wxT("$STREAM"), wxString::Format(wxT("%d"), streamIdx));
+	cmd.Replace(wxT("$STREAM"), wxString::Format(wxT("%u"), streamIdx));
 	wxULongLong fileSize = wxFileName::GetSize(vobFile);
 	SpumuxExecute exec(this, fileSize != wxInvalidSize ? fileSize.ToDouble() : -1);
 	if (!exec.Execute(cmd, vobFile, resultFile)) {
@@ -858,7 +896,7 @@ or open the following directory with your DVD player: ");
 				if (cmd.Find(wxT("wmplayer")) >= 0)
 					cmd += wxT(" \"$DIR\\VIDEO_TS\\VIDEO_TS.VOB\"");
 				else
-					cmd += wxT(" \"dvd://$DIR\"");
+					cmd += wxT(" \"dvd:///$DIR\"");
 			}
 			cmd.Replace(wxT("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
 			if (!Exec(cmd))
@@ -1012,7 +1050,7 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 		}
 	}
 	long discSize = GetMediaSpaceFree(device); // size in 2048 blocks
-	AddDetailMsg(wxString::Format(wxT("Disc size: %d MB"), discSize / 512));
+	AddDetailMsg(wxString::Format(wxT("Disc size: %ld MB"), discSize / 512));
 #else
 	cmd = s_config.GetBurnScanCmd();
 	cmd.Replace(wxT("$DEVICE"), device);
@@ -1035,7 +1073,7 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 		if (output[i].length() > 12 && output[i].SubString(1, 12) == wxT("Free Blocks:")) {
 			wxString discSizeStr = output[i].AfterFirst(wxT(':')).Trim(false).BeforeFirst(wxT('*'));
 			discSizeStr.ToLong(&discSize);
-			AddDetailMsg(wxString::Format(wxT("Disc size: %d MB"), discSize / 512));
+			AddDetailMsg(wxString::Format(wxT("Disc size: %ld MB"), discSize / 512));
 			break;
 		}
 	}
@@ -1064,7 +1102,7 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 			size = size + 254;
 		}
 	}
-	AddDetailMsg(wxString::Format(wxT("ISO Size: %d MB"), size / 512));
+	AddDetailMsg(wxString::Format(wxT("ISO Size: %ld MB"), size / 512));
 	if (size > discSize && wxMessageBox(wxString::Format(_("Size of Disc Image > %.2f GB. Do you want to continue?"), (double) discSize / 512
 			/ 1024), _("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxNO) {
 		AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
@@ -1076,7 +1114,7 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 		cmd.Replace(_T("$FILE"), tmpDir + TMP_ISO);
 		// get iso size in sectors
 		long size = wxFile(tmpDir + TMP_ISO).Length() / 2048;
-		cmd.Replace(_T("$SIZE"), wxString::Format(wxT("%d"),size));
+		cmd.Replace(_T("$SIZE"), wxString::Format(wxT("%ld"), size));
 	} else {
 		cmd = s_config.GetBurnCmd();
 		cmd.Replace(_T("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
@@ -1098,7 +1136,7 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 	return true;
 }
 
-bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool useMplex) {
+bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool useMplex, AudioFormat defAudioFormat) {
 	if (IsCanceled())
 		return false;
 	int subStep = GetSubStep();
@@ -1111,6 +1149,7 @@ bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool
 	bool needEncode = vob->GetAudioFilenames().GetCount() != vob->GetAudioStreamCount()
 			|| startTime != 0 || recordingTime != -1; // some files are not elementary streams
 	VideoFormat videoFormat = vfCOPY;
+	bool ntscFilm = false;
 	wxArrayInt audioFormats;
 	wxArrayInt subtitleFormats;
 	for (unsigned int stIdx = 0; stIdx < vob->GetStreams().GetCount(); stIdx++) {
@@ -1118,6 +1157,7 @@ bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool
 		switch (stream->GetType()) {
 		case stVIDEO:
 			videoFormat = stream->GetVideoFormat();
+			ntscFilm = lround(stream->GetSourceFps()) == 24;
 			needEncode = needEncode || stream->GetVideoFormat() != vfCOPY;
 			break;
 		case stAUDIO:
@@ -1148,12 +1188,22 @@ bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool
 			Failed(wxT("Error by transcoding of ") + vob->GetFilename());
 			return false;
 		}
-		for (int i = 0; i < (int)vob->GetAudioFilenames().GetCount(); i++) {
-			if (!transcoder.AddInputFile(vob->GetAudioFilenames()[i])) {
+		for (int i = 0; i < (int)vob->GetAudioFilenames().size(); i++) {
+			int stIdx = vob->GetStreams().size() - vob->GetAudioFilenames().size() + i;
+			if (!transcoder.AddInputFile(vob->GetAudioFilenames()[i], false, false, false,
+					vob->GetStreams()[stIdx]->GetTsOffset())) {
 				Failed(wxT("Error by transcoding of ") + vob->GetAudioFilenames()[i]);
 				return false;
 			}
 		}
+		if (vob && vob->GetFilename().length() && !vob->HasAudio()) {
+			wxString zero = wxT("pipe:-1");
+			if (!transcoder.AddInputFile(zero, false, false, false, 0)) {
+				Failed(wxT("Error by transcoding of ") + zero);
+				return false;
+			}
+			audioFormats.Add(defAudioFormat);
+		}
 		
 		// set output options
 		transcoder.SetInterlaced(vob->GetInterlaced());
@@ -1164,15 +1214,22 @@ bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool
 		
 		// set video filters
 		wxString videoFilters = vob->GetAllVideoFilters();
-		if (videoFilters.length())
+		if (videoFilters.length() > 0) {
 			wxLogMessage(wxT("Video filters: ") + videoFilters);
+		}
 		transcoder.SetVideoFilters(videoFilters);
 		
 		// set audio volume
+		int audioIdx = 0;
 		for (unsigned int stIdx = 0; stIdx < vob->GetStreams().GetCount(); stIdx++) {
 			Stream* stream = vob->GetStreams()[stIdx];
-			if (stream->GetType() == stAUDIO && stream->GetAudioVolume() != 256)
-				transcoder.SetAudioVolume(stIdx, stream->GetAudioVolume());
+			if (stream->GetType() == stAUDIO) {
+				if (stream->GetAudioVolume() != 256)
+					transcoder.SetAudioVolume(audioIdx, stream->GetAudioVolume());
+				if (stream->GetChannelNumber() > 0)
+					transcoder.SetChannelNumber(audioIdx, stream->GetChannelNumber());
+				audioIdx++;
+			}
 		}
 		
 		if (startTime != 0) {
@@ -1185,10 +1242,13 @@ bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool
 		}
 		
 		if (!useMplex) {
-			if (!transcoder.SetOutputFile(vob->GetTmpFilename(), videoFormat, audioFormats, subtitleFormats,
+			double fps = GetFps(videoFormat, ntscFilm);
+			AVConvExecute exec(this, lround(vob->GetDuration() * fps));
+			if (!transcoder.SetOutputFile(vob->GetTmpFilename(), videoFormat, ntscFilm, audioFormats, subtitleFormats,
 					videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())
-					|| !transcoder.Run(m_cancel)) {
-				wxRemoveFile(vob->GetTmpFilename());
+					|| !exec.Execute(transcoder.GetCmd())) {
+				if (wxFileExists(vob->GetTmpFilename()))
+					wxRemoveFile(vob->GetTmpFilename());
 				Failed(_("Error transcoding of ") + vob->GetFilename());
 				return false;
 			}
@@ -1197,8 +1257,8 @@ bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool
 			if (wxFileExists(videoFile) && !wxRemoveFile(videoFile)) {
 				wxLogError(wxString::Format(_("Can't remove file '%s'"), videoFile.c_str()));
 			}
-			if (!transcoder.SetOutputFile(videoFile, videoFormat, afNONE, sfNONE,
-					videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())) {
+			if (!transcoder.SetOutputFile(videoFile, videoFormat, ntscFilm, afNONE, sfNONE,
+					videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate(), vob->GetVideoStreamIndex())) {
 				Failed(_("Error transcoding of ") + vob->GetFilename());
 				return false;
 			}
@@ -1206,29 +1266,35 @@ bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool
 			for (unsigned int audioIdx = 0; audioIdx < audioFormats.GetCount(); audioIdx++) {
 				if (audioFormats[audioIdx] == afNONE)
 					continue;
-				wxString audioFile = vob->GetTmpFilename() + wxString::Format(wxT(".audio%d"), audioIdx);
+				wxString audioFile = vob->GetTmpFilename() + wxString::Format(wxT(".audio%u"), audioIdx);
 				audioFiles.Add(audioFile);
 				if (wxFileExists(audioFile) && !wxRemoveFile(audioFile)) {
 					wxLogError(wxString::Format(_("Can't remove file '%s'"), audioFile.c_str()));
 				}
-				if (!transcoder.SetOutputFile(audioFile, vfNONE, (AudioFormat) audioFormats[audioIdx], sfNONE,
-						videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())) {
+				if (!transcoder.SetOutputFile(audioFile, vfNONE, false, (AudioFormat) audioFormats[audioIdx], sfNONE,
+						videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate(), audioIdx + 1)) {
 					Failed(_("Error transcoding of ") + vob->GetFilename());
 					return false;
 				}
 			}
-			if (!transcoder.Run(m_cancel)) {
-				wxRemoveFile(videoFile);
+			double fps = GetFps(videoFormat, ntscFilm);
+			AVConvExecute exec(this, lround(vob->GetDuration() * fps));
+			if (!exec.Execute(transcoder.GetCmd())) {
+				if (wxFileExists(videoFile))
+					wxRemoveFile(videoFile);
 				for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++)
-					wxRemoveFile(audioFiles[audioIdx]);
+					if (wxFileExists(audioFiles[audioIdx]))
+						wxRemoveFile(audioFiles[audioIdx]);
 				Failed(_("Error transcoding of ") + vob->GetFilename());
 				return false;
 			}
 			SetSubStep(subStep+150);
 			if (!Multiplex(videoFile, audioFiles, vob->GetTmpFilename())) {
-				wxRemoveFile(videoFile);
+				if (wxFileExists(videoFile))
+					wxRemoveFile(videoFile);
 				for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++)
-					wxRemoveFile(audioFiles[audioIdx]);
+					if (wxFileExists(audioFiles[audioIdx]))
+						wxRemoveFile(audioFiles[audioIdx]);
 				return false;
 			}
 			// remove temp files
@@ -1268,6 +1334,7 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 		return false;
 	wxString mpegFile = menuFile + _T("_bg.mpg");
 	wxString m2vFile = menuFile + _T("_bg.m2v");
+	wxString audioFileTmp = menuFile + _T("_bg.audio");
 	wxString btFile = menuFile + _T("_buttons.png");
 	wxString hlFile = menuFile + _T("_highlight.png");
 	wxString selFile = menuFile + _T("_select.png");
@@ -1281,7 +1348,29 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 		s_config.SetMenuVideoBitrate(1000);
 	
 	if (videoMenu) {
-		bool hasAudio = audioFile.length() || menu->HasVideoBackground();
+		// get background audio format
+		AudioFormat bgAudioFormat = afNONE;
+		wxString bgAudioOutputFormat;
+		if (menu->HasVideoBackground()) {
+			wxFfmpegMediaDecoder ffmpeg;
+			if (!ffmpeg.Load(menu->GetBackground()))
+				return false;
+			for (unsigned int i = 0; i < ffmpeg.GetStreamCount(); i++) {
+				if (ffmpeg.GetStreamType(i) == stAUDIO) {
+					if (ffmpeg.GetCodecName(i) != wxT("mp2") && ffmpeg.GetCodecName(i) != wxT("ac3")) {
+						bgAudioFormat = audioFormat;
+					} else if (ffmpeg.GetSampleRate(i) != 48000) {
+						bgAudioFormat = ffmpeg.GetCodecName(i) == wxT("mp2") ? afMP2 : afAC3;
+					} else {
+						bgAudioFormat = afCOPY;
+						bgAudioOutputFormat = ffmpeg.GetCodecName(i);
+					}
+					break;
+				}
+			}
+		}
+		// encode video
+		bool hasAudio = audioFile.length() || bgAudioFormat != afNONE;
 		wxFfmpegMediaEncoder ffmpeg(s_config.GetThreadCount());
 		if (!ffmpeg.BeginEncode(hasAudio ? m2vFile : mpegFile, menu->GetVideoFormat(), hasAudio ? afNONE : audioFormat,
 				menu->GetAspectRatio(), s_config.GetMenuVideoBitrate())) {
@@ -1310,12 +1399,35 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 		}
 		ffmpeg.EndEncode();
 		if (hasAudio) {
-			AddDetailMsg(_("Multiplexing audio and video"));
 			Vob menuVob;
 			menuVob.SetFilename(m2vFile);
-			menuVob.GetVideoStream()->SetDestinationFormat(menu->GetVideoFormat());
-			if (hasAudio)
-				menuVob.AddAudioFile(audioFile.length() ? audioFile : menu->GetBackground());
+			menuVob.GetVideoStream()->SetDestinationFormat(vfCOPY);
+			if (audioFile.length()) {
+				menuVob.AddAudioFile(audioFile);
+			} else {
+				AddDetailMsg(_("Transcode audio from") + wxString(wxT(" ")) + menu->GetBackground());
+				// transcode audio
+				wxFfmpegMediaTranscoder transcoder(s_config.GetThreadCount());
+				if (!transcoder.AddInputFile(menu->GetBackground(), true)) {
+					Failed(wxT("Error by transcoding of ") + menu->GetBackground());
+					return false;
+				}
+				transcoder.SetOutputFormat(bgAudioOutputFormat);
+				if (!transcoder.SetOutputFile(audioFileTmp, vfNONE, false, bgAudioFormat, sfNONE, 0, false,
+						s_config.GetAudioBitrate())) {
+					Failed(_("Error transcoding of ") + menu->GetBackground());
+					return false;
+				}
+				AVConvExecute exec(this, -1);
+				if (!exec.Execute(transcoder.GetCmd())) {
+					if (wxFileExists(audioFileTmp))
+						wxRemoveFile(audioFileTmp);
+					Failed(_("Error transcoding of ") + menu->GetBackground());
+					return false;
+				}
+				menuVob.AddAudioFile(audioFileTmp);
+			}
+			AddDetailMsg(_("Multiplexing audio and video"));
 			for (unsigned int i=0; i<menuVob.GetStreams().GetCount(); i++) {
 				Stream* stream = menuVob.GetStreams()[i];
 				if (stream->GetType() == stAUDIO && stream->GetAudioFormat() == afCOPY) {
@@ -1329,10 +1441,16 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 			}
 			menuVob.SetKeepAspectRatio(false);
 			menuVob.SetTmpFilename(mpegFile);
-			if (!Transcode(&menuVob, menu->GetAspectRatio(), videoBitrate, s_config.GetUseMplexForMenus()))
+			if (!s_config.GetUseMplexForMenus())
+				AddDetailMsg(WARNING_MPLEX_MSG, *wxRED);
+			if (!Transcode(&menuVob, menu->GetAspectRatio(), s_config.GetMenuVideoBitrate(),
+					s_config.GetUseMplexForMenus()))
 				return false;
-			if (s_config.GetRemoveTempFiles())
+			if (s_config.GetRemoveTempFiles()) {
 				DeleteFile(m2vFile);
+				if (wxFileExists(audioFileTmp))
+					wxRemoveFile(audioFileTmp);
+			}
 		}
 	} else {
 		wxImage bgImage = menu->GetBackgroundImage();
@@ -1385,7 +1503,10 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 				}
 			}
 			menuVob.SetTmpFilename(mpegFile);
-			if (!Transcode(&menuVob, menu->GetAspectRatio(), videoBitrate, s_config.GetUseMplexForMenus()))
+			if (!s_config.GetUseMplexForMenus())
+				AddDetailMsg(WARNING_MPLEX_MSG, *wxRED);
+			if (!Transcode(&menuVob, menu->GetAspectRatio(), s_config.GetMenuVideoBitrate(),
+					s_config.GetUseMplexForMenus()))
 				return false;
 			if (s_config.GetRemoveTempFiles())
 				DeleteFile(m2vFile);
@@ -1458,8 +1579,8 @@ bool ProgressDlg::GenerateSlideshow(Slideshow* slideshow, const wxString& vobFil
 		return false;
 	}
 
-	for (unsigned i=0; i<slideshow->GetCount(); i++) {
-		AddDetailMsg(wxString::Format(_("Converting slide %d image to video"),i+1));
+	for (unsigned i = 0; i < slideshow->GetCount(); i++) {
+		AddDetailMsg(wxString::Format(_("Converting slide %u image to video"), i + 1));
 		wxYield();
 		wxImage img = slideshow->GetImage(i);
 		if (!ffmpeg.EncodeImage(img, (int)(slideshow->GetDuration()*slideshow->GetFPS()))) {
