@@ -3,7 +3,7 @@
 // Purpose:     Progress of generation/burning dialog
 // Author:      Alex Thuering
 // Created:     14.08.2004
-// RCS-ID:      $Id: ProgressDlg.h,v 1.33 2011/04/14 07:29:18 ntalex Exp $
+// RCS-ID:      $Id: ProgressDlg.h,v 1.34 2012/01/22 20:40:34 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -138,7 +138,7 @@ private:
 	bool GenerateMenu(Menu* menu, WidescreenType widescreenType, const wxString& menuFile, AudioFormat audioFormat,
 			wxString audioFile, int videoBitrate);
     bool GenerateSlideshow(Slideshow* slideshow, const wxString& vobFile, AudioFormat audioFormat, wxString audioFile);
-	bool Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool useMplex);
+	bool Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool useMplex, AudioFormat defAudioFormat = afAC3);
 	bool MultiplexSubtitles(const wxString& vobFile, TextSub* textSub, unsigned int streamIdx, VideoFormat videoFormat,
 			const wxString& resultFile);
 	/** removes given file and print error message if it failed */
