@@ -3,7 +3,7 @@
 // Purpose:     Settings dialog
 // Author:      Alex Thuering
 // Created:     27.03.2004
-// RCS-ID:      $Id: SettingsDlg.cpp,v 1.59 2011/04/14 07:19:17 ntalex Exp $
+// RCS-ID:      $Id: SettingsDlg.cpp,v 1.64 2012/08/29 18:49:06 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -28,8 +28,7 @@ BEGIN_EVENT_TABLE(SettingsDlg, wxPropDlg)
 	EVT_BUTTON(RESET_DONT_SHOW_FLAGS_BT_ID, SettingsDlg::OnResetDontShowFlags)
 END_EVENT_TABLE()
 
-SettingsDlg::SettingsDlg(wxWindow* parent) :
-	wxPropDlg(parent) {
+SettingsDlg::SettingsDlg(wxWindow* parent): wxPropDlg(parent), m_directoryEdit(NULL), m_directorySelectButton(NULL) {
 	Create(true);
 	SetTitle(_("Settings"));
 	SetSize(600, -1);
@@ -56,13 +55,13 @@ void SettingsDlg::CreatePropPanel(wxSizer* sizer) {
 		interfacePanel->SetAutoLayout(true);
 		interfacePanel->SetSizer(interfaceSizer);
 
-		interfaceGrid = new wxFlexGridSizer(2, 2, 4, 16);
+		interfaceGrid = new wxFlexGridSizer(2, 4, 16);
 		interfaceGrid->AddGrowableCol(1);
 		interfaceSizer->Add(interfaceGrid, 1, wxEXPAND | wxALL, 10);
 	}
 
 	// -------------- Interface Tab ----------------- //
-	wxString lang = GetLangName(s_config.GetLanguage());
+	wxString lang = GetLangName(s_config.GetLanguageCode());
 	AddBitmapComboProp(interfaceGrid, _("Language:"), lang, GetLangNames(), GetLangBitmaps(), wxCB_READONLY);
 	AddTextProp(interfaceGrid, _("Default disc label:"), s_config.GetDefDiscLabel(def));
 	wxArrayString labels = DVD::GetCapacityLabels();
@@ -105,7 +104,11 @@ void SettingsDlg::CreatePropPanel(wxSizer* sizer) {
 	GetLastControl()->SetMinSize(aspectRatioChoice->GetBestSize());
 	
 	// default chapter length
-	AddSpinProp(interfaceGrid, _("Default chapter length:"), s_config.GetDefChapterLength(def), 1, 100, 60, _("min"));
+	AddSpinProp(interfaceGrid, _("Default chapter length:"), s_config.GetDefChapterLength(def), 1, 999, 60, _("min"));
+	
+	// default title post command
+	labels = DVD::GetDefPostCommandLabels();
+	AddChoiceProp(interfaceGrid, _("Default title post command:"), labels[s_config.GetDefPostCommand(def)], labels, 0);
 	
 	// default button
 	int sel = 0;
@@ -179,7 +182,7 @@ void SettingsDlg::CreatePropPanel(wxSizer* sizer) {
 		interfacePanel->SetAutoLayout(true);
 		interfacePanel->SetSizer(coreSizer);
 
-		coreGrid = new wxFlexGridSizer(2, 2, 4, 16);
+		coreGrid = new wxFlexGridSizer(2, 4, 16);
 		coreGrid->AddGrowableCol(1);
 		coreSizer->Add(coreGrid, 1, wxEXPAND | wxTOP | wxLEFT | wxRIGHT, 10);
 
@@ -220,8 +223,11 @@ void SettingsDlg::CreatePropPanel(wxSizer* sizer) {
 bool SettingsDlg::SetValues() {
 	// interface settings
 	int i = 0;
-	if (GetLangCode(GetInt(i++)) != s_config.GetLanguage()) {
-		s_config.SetLanguage(GetLangCode(GetInt(i - 1)));
+	if (GetLangCode(GetInt(i++)) != s_config.GetLanguageCode()) {
+		s_config.SetLanguageCode(GetLangCode(GetInt(i - 1)));
+		wxString langCode = s_config.GetLanguageCode().Upper().substr(0, 2);
+		if (DVD::GetAudioLanguageCodes().Index(langCode) != wxNOT_FOUND)
+			s_config.SetDefSubtitleLanguage(langCode);
 		wxMessageBox(_("Language change will not take effect until DVDStyler is restarted"),
 				GetTitle(), wxOK|wxICON_INFORMATION, this);
 	}
@@ -232,6 +238,7 @@ bool SettingsDlg::SetValues() {
 	s_config.SetDefAudioFormat(GetInt(i++) + 2);
 	s_config.SetDefAudioLanguage(GetString(i++));
 	s_config.SetDefChapterLength(GetInt(i++));
+	s_config.SetDefPostCommand(GetInt(i++));
 	s_config.SetDefButton(m_buttons[GetInt(i++)]);
 	s_config.SetAcceptInvalidActions(GetBool(i++));
 	i++;
@@ -262,7 +269,8 @@ bool SettingsDlg::SetValues() {
 }
 
 void SettingsDlg::OnChangeFileBrowserDir(wxCommandEvent& evt) {
-	m_directoryEdit->SetValue(evt.GetSelection() == 0 ? wxT("") : wxGetHomeDir());
+	wxString dir = evt.GetSelection() == 0 ? wxT("") : wxGetHomeDir();
+	m_directoryEdit->SetValue(dir);
 	m_directoryEdit->Enable(evt.GetSelection() == 1);
 	m_directorySelectButton->Enable(evt.GetSelection() == 1);
 }
