@@ -3,7 +3,7 @@
 // Purpose:     DVD title properties dialog
 // Author:      Alex Thuering
 // Created:     31.01.2004
-// RCS-ID:      $Id: TitlePropDlg.cpp,v 1.53 2011/08/01 20:30:08 ntalex Exp $
+// RCS-ID:      $Id: TitlePropDlg.cpp,v 1.57 2012/06/24 22:57:24 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -120,8 +120,7 @@ void TitlePropDlg::CreateVobGroup(wxSizer* sizer) {
 	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 6);
 	grid->AddGrowableCol(1);
 	AddText(grid, wxT(""));
-	AddCheckProp(grid, _("do not remultiplex/transcode"), m_vob->GetDoNotTranscode(), false,
-			DO_NOT_TRANSCODE_CB_ID);
+	AddCheckProp(grid, _("do not remultiplex/transcode"), m_vob->GetDoNotTranscode(), false, DO_NOT_TRANSCODE_CB_ID);
 	m_doNotTranscodeCheckBox = (wxCheckBox*) GetLastControl();
 	UpdateDoNotTranscodeCheck();
 	grid->Add(2, 2);
@@ -243,6 +242,7 @@ wxArrayString TitlePropDlg::GetCommandList(DVD* dvd, int currentTsi) {
 			for (StringSet::const_iterator entry = entrySet.begin(); entry != entrySet.end(); entry++)
 				if (*entry != wxT("root"))
 					commands.Add(wxString::Format(wxT("call menu entry %s;"), entry->c_str()));
+			commands.Add(wxT("jump next title;"));
 		}
 		if (dvd->IsJumppadsEnabled()) {
 			for (int pgci=1; pgci<(int)ts->GetMenus().Count(); pgci++)
@@ -345,6 +345,8 @@ bool TitlePropDlg::SetValues() {
 		}
 		// update video parameters
 		m_pgcs->GetVideo().SetAspect(m_vobListBox->GetAspectRatio());
+		if (m_pgci == 0 && m_vobi == 0 && m_vob->GetVideoStream())
+			m_pgcs->GetVideo().SetFormat(m_vob->GetVideoStream()->GetVideoFormat());
 	}
 	m_pgc->SetPalette(GetString(n++));
 	m_pgc->SetPreCommands(GetString(n++));
@@ -362,7 +364,6 @@ void TitlePropDlg::OnSelectItem(wxCommandEvent& event) {
 		m_delBt->Enable(m_slideShowBox->GetSelectedCount() > 0);
 	} else {
 		m_delBt->Enable(m_vobListBox->GetSelection() > 0);
-		m_propBt->Enable(m_vobListBox->GetSelection() > 0);
 	}
 }
 
@@ -385,7 +386,7 @@ void TitlePropDlg::OnAddBt(wxCommandEvent& event) {
 	wxFileDialog dialog(this, _("Choose a file"), wxT(""), wxT(""),
 			_("Audio & Subtitle Files") + wxString::Format(wxT("|%s%s|"), audio.c_str(), subtitle.c_str())
 			+ _("Audio Files") + wxString::Format(wxT(" (%s)|%s|"), audio.c_str(), audio.c_str())
-			+ _("Subtitle Files") + wxString::Format(wxT(" (%s)|%s|"), subtitle.c_str(), subtitle.c_str()),
+			+ _("Subtitle Files") + wxString::Format(wxT(" (%s)|%s"), subtitle.c_str(), subtitle.c_str()),
 			wxFD_OPEN);
 	// set file browser dir
 	dialog.SetDirectory(s_config.GetLastAddDir() + wxFILE_SEP_PATH);
