@@ -3,7 +3,7 @@
 // Purpose:     Titleset Manager
 // Author:      Alex Thuering
 // Created:	27.01.2003
-// RCS-ID:      $Id: TitlesetManager.cpp,v 1.58 2011/11/22 20:27:20 ntalex Exp $
+// RCS-ID:      $Id: TitlesetManager.cpp,v 1.67 2012/10/14 16:23:02 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -60,7 +60,9 @@ public:
 		if (dobj->GetReceivedFormat() == wxDF_FILENAME) {
 			return OnDropFiles(x, y, m_fileDataObject->GetFilenames(), def) ? def : wxDragError;
 		} else if (dobj->GetReceivedFormat().GetId() == DATAFORMAT_ACTION) {
-			m_pOwner->MoveSelectedTo(m_pOwner->GetItemIndex(x, y));
+			int x1, y1;
+			m_pOwner->CalcUnscrolledPosition(x, y, &x1, &y1);
+			m_pOwner->MoveSelectedTo(m_pOwner->GetItemIndex(x1, y1));
 			return def;
 		}
 		
@@ -75,7 +77,7 @@ public:
 	}
 	
 	bool AddFile(wxString filename, bool first, wxDragResult def) {
-		if (wxImage::FindHandler(filename.AfterLast('.').Lower(), -1)) {
+		if (wxImage::FindHandler(filename.AfterLast('.').Lower(), BITMAP_TYPE_ANY)) {
 			m_pOwner->AddImage(filename, def == wxDragMove && first);
 			return true;
 		} else if (wxThumbnails::IsVideo(filename)) {
@@ -167,18 +169,27 @@ TitlesetManager::TitlesetManager(wxWindow* parent, int id): wxThumbnails(parent,
 	addMenu->Append(MainWin::MENU_DVD_ADD_VMMENU_ID, _("&vmMenu"));
 	addMenu->Append(MainWin::MENU_DVD_ADD_TITLESET_ID, _("&Titleset"));
 
-	wxMenu* moveMenu = new wxMenu;
-	moveMenu->Append(MENU_MOVE_TITLE_LEFT_ID, _("&Left"));
-	moveMenu->Append(MENU_MOVE_TITLE_RIGHT_ID, _("&Right"));
-
 	m_mainMenu = new wxMenu;
 	m_mainMenu->Append(MENU_MAIN_ADD_ID, _("&Add"), addMenu);
 	wxMenuItem* item = new wxMenuItem(m_mainMenu, MENU_TITLE_PASTE_ID, _("&Paste"));
 	item->SetBitmap(wxArtProvider::GetBitmap(wxART_PASTE, wxART_MENU));
 	m_mainMenu->Append(item);
+	SetGlobalPopupMenu(m_mainMenu);
+
+
+	wxMenu* moveMenu = new wxMenu;
+	moveMenu->Append(MENU_MOVE_TITLE_LEFT_ID, _("&Left"));
+	moveMenu->Append(MENU_MOVE_TITLE_RIGHT_ID, _("&Right"));
+
+	wxMenu* addMenu2 = new wxMenu;
+	addMenu2->Append(MainWin::MENU_DVD_ADD_FILE_ID, _("&File..."));
+	addMenu2->Append(MainWin::MENU_DVD_ADD_FILE_AS_CHAPTER_ID, _("File as &chapter..."));
+	addMenu2->Append(MainWin::MENU_DVD_ADD_MENU_ID, _("&Menu"));
+	addMenu2->Append(MainWin::MENU_DVD_ADD_VMMENU_ID, _("&vmMenu"));
+	addMenu2->Append(MainWin::MENU_DVD_ADD_TITLESET_ID, _("&Titleset"));
 
 	m_titleMenu = new wxMenu;
-	m_titleMenu->Append(MENU_TITLE_ADD_ID, _("&Add"), addMenu);
+	m_titleMenu->Append(MENU_TITLE_ADD_ID, _("&Add"), addMenu2);
 	m_titleMenu->Append(MENU_TITLE_MOVE_ID, _("&Move"), moveMenu);
 	item = new wxMenuItem(m_titleMenu, MENU_TITLE_COPY_ID, _("&Copy"));
 	item->SetBitmap(wxArtProvider::GetBitmap(wxART_COPY, wxART_MENU));
@@ -190,9 +201,7 @@ TitlesetManager::TitlesetManager(wxWindow* parent, int id): wxThumbnails(parent,
 	item->SetBitmap(wxArtProvider::GetBitmap(wxART_DELETE, wxART_MENU));
 	m_titleMenu->Append(item);
 	m_titleMenu->Append(MENU_TITLE_PROP_ID, _("&Properties..."));
-
 	SetPopupMenu(m_titleMenu);
-	SetGlobalPopupMenu(m_mainMenu);
 }
 
 TitlesetManager::~TitlesetManager() {
@@ -234,7 +243,8 @@ void TitlesetManager::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index) {
 	int pgci = DVD::GetPgci(id);
 	int vobi = DVD::GetVobi(id);
 	bool isMenu = DVD::IsMenu(id);
-	Vob* vob = m_dvd->GetVob(tsi, isMenu, pgci, vobi);
+	Pgc* pgc = m_dvd->GetPgc(tsi, isMenu, pgci);
+	Vob* vob = pgc->GetVobs()[vobi];
 
 	wxMemoryDC dc;
 	dc.SelectObject(bmp);
@@ -248,6 +258,16 @@ void TitlesetManager::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index) {
 	dc.SetPen(wxPen(*wxBLACK, 0, wxTRANSPARENT));
 	dc.SetBrush(wxBrush(GetBackgroundColour(), wxSOLID));
 	dc.DrawRectangle(0, 0, bmp.GetWidth(), bmp.GetHeight());
+	if (pgc->GetVobs().Count() > 1) {
+		dc.SetBrush(wxBrush(wxColour(64, 64, 64), wxSOLID));
+		if (vobi == 0) {
+			dc.DrawRectangle(bmp.GetWidth()/2, y, bmp.GetWidth()/2, th);
+		} else if (vobi == (int) pgc->GetVobs().Count() - 1) {
+			dc.DrawRectangle(0, y, bmp.GetWidth()/2, th);
+		} else {
+			dc.DrawRectangle(0, y, bmp.GetWidth(), th);
+		}
+	}
 	// rounded rectangle
 	wxColour rectColour = isMenu ? wxColour(64, 64, 64) : wxColour(0, 0, 0);
 	dc.SetBrush(wxBrush(rectColour, wxSOLID));
@@ -394,7 +414,9 @@ void TitlesetManager::UpdateItems() {
 						caption = _("Slideshow (1 image)");
 				} else {
 					fname = vob->GetFilename();
-					caption = fname.AfterLast(wxFILE_SEP_PATH).BeforeLast('.');
+					caption = vob->GetFilenameDisplay().AfterLast(wxFILE_SEP_PATH);
+					if (caption.Find(wxT('.')) > 0)
+						caption = caption.BeforeLast(wxT('.'));
 				}
 				wxThumb* thumb = new wxThumb(fname, caption);
 				thumb->SetId(DVD::MakeId(tsi, pgci, vobi, false));
@@ -423,6 +445,15 @@ int TitlesetManager::GetIndex(int id) {
 }
 
 bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi) {
+	Vob* vob = new Vob(_T(""));
+	if (!vob->SetFilename(fname))
+		return false;
+	Stream* videoStream = vob->GetVideoStream();
+	if (videoStream == NULL) {
+		wxLogError(_("%s doesn't contain video stream."), fname.c_str());
+		return false;
+	}
+		
 	int videoBitrateOld = m_dvd->GetVideoBitrate();
 	
 	if (m_dvd->GetTitlesets().Count() == 0)
@@ -430,21 +461,7 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi)
 	if (tsi < 0)
 		tsi = m_dvd->GetTitlesets().GetCount() - 1;
 	PgcArray& pgcs = m_dvd->GetTitlesets()[tsi]->GetTitles();
-	Pgc* pgc = pgcs.Count()>0 ? pgcs.Last() : NULL;
-	Vob* vob = pgc && pgc->GetVobs().Count()>0 ? pgc->GetVobs().Last() : NULL;
-	if (vob && vob->GetFilename().length() && !vob->HasAudio()) {
-		wxLogError(wxString(_("The last title isn't complete.\n"))
-				+ _("Please add an audio track to it."));
-		return false;
-	}
 	
-	vob = new Vob(_T(""));
-	vob->SetFilename(fname);
-	Stream* videoStream = vob->GetVideoStream();
-	if (videoStream == NULL) {
-		wxLogError(_("%s doesn't contain video stream."), fname.c_str());
-		return false;
-	}
 	if (pgcs.Count() == 0) {
 		// first title => choose aspect ratio
 		pgcs.GetVideo().SetAspect(videoStream->GetSourceAspectRatio() > 1.4 ? ar16_9 : ar4_3);
@@ -461,17 +478,22 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi)
 					m_dvd->SetVideoFormat(vfNTSC, true);
 			}
 		}
+	} else if (createTitle && pgcs.Count() >= 99) {
+		wxLogError(_("Maximal 99 titles are allowed for one titleset."));
+		return false;
 	}
+	if (pgcs.GetVideo().GetFormat() <= vfCOPY)
+		pgcs.GetVideo().SetFormat(m_dvd->GetVideoFormat());
 	// check if reencoding is needed
 	for (unsigned int i=0; i<vob->GetStreams().GetCount(); i++) {
 		Stream* stream = vob->GetStreams()[i];
 		if (stream->GetType() == stVIDEO) {
 			// set destination format (default COPY) if codec is not mpeg2
 			if (!stream->GetSourceCodecName().StartsWith(wxT("mpeg2video"))
-					|| stream->GetSourceVideoFormat() != m_dvd->GetVideoFormat()
+					|| stream->GetSourceVideoFormat() != pgcs.GetVideo().GetFormat()
 					|| fabs(stream->GetSourceAspectRatio()
 							- GetFrameAspectRatio(vfPAL, pgcs.GetVideo().GetAspect())) >= 0.15)
-				stream->SetDestinationFormat(m_dvd->GetVideoFormat());
+				stream->SetDestinationFormat(pgcs.GetVideo().GetFormat());
 			else if (vob->GetAudioFilenames().Count() == 0 && MPEG::HasNavPacks(vob->GetFilename()))
 				vob->SetDoNotTranscode(true);  // do not remultiplex/transcode if the video is already a VOB
 		} else if (stream->GetType() == stAUDIO) {
@@ -496,10 +518,11 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi)
 	if (n>100)
 		n = 100;
 	
+	Pgc* pgc = pgcs.Count()>0 ? pgcs.Last() : NULL;
 	if (!createTitle && pgc != NULL)
 		vob->GetCells().push_back(new Cell(0));
-	long t = s_config.GetDefChapterLength();
-	for (int i = 0; i < n; i++) {
+		long t = s_config.GetDefChapterLength();
+		for (int i = 0; i < n; i++) {
 		vob->GetCells().push_back(new Cell(t * 60000));
 		t += s_config.GetDefChapterLength();
 	}
@@ -507,7 +530,7 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi)
 	// add vob
 	if (createTitle || pgc == NULL) {
 		pgc = new Pgc;
-		pgc->SetPostCommands(_T("call last menu;"));
+		pgc->SetPostCommands(m_dvd->GetDefPostCommandStr());
 		pgc->GetVobs().Add(vob);
 		pgcs.Add(pgc);
 		if (!m_dvd->UpdateButtonImageFor(tsi, pgcs.GetCount() - 1) && pgcs.GetCount() > 4) {
@@ -686,6 +709,12 @@ void TitlesetManager::OnMouseMove(wxMouseEvent &event) {
 		action.SetTsi(DVD::GetTsi(GetSelectedId()));
 		action.SetPgci(DVD::GetPgci(GetSelectedId()));
 		action.SetMenu(DVD::IsMenu(GetSelectedId()));
+		int vobi = DVD::GetVobi(GetSelectedId());
+		if (vobi > 0) {
+			Pgc* pgc = m_dvd->GetPgc(action.GetTsi(), action.IsMenu(), action.GetPgci());
+			Vob* vob = pgc->GetVobs()[vobi];
+			action.SetChapter(pgc->GetChapterCount(vobi) + (vob->GetChapters().length() ? 0 : -1));
+		}
 		wxString actionStr = action.Serialize();
 		wxDataFormat format(DATAFORMAT_ACTION);
 		wxCustomDataObject dataObj(format);
@@ -751,6 +780,17 @@ bool TitlesetManager::DuplicateMenu(int tsi, int pgci) {
 	return true;
 }
 
+void TitlesetManager::FixVobChapters(Pgc* pgc, int vobi, int vobi2) {
+	wxString chapters = pgc->GetVobs()[vobi]->GetChapters();
+	wxString chapters2 = pgc->GetVobs()[vobi2]->GetChapters();
+	if (!(chapters + wxT(",")).StartsWith(wxT("0:00,")) && (chapters2 + wxT(",")).StartsWith(wxT("0:00,"))) {
+		chapters = (chapters.length() ? wxT("0:00,") : wxT("0:00")) + chapters;
+		pgc->GetVobs()[vobi]->SetChapters(chapters);
+		chapters2 = chapters2.length() > 4 ? chapters2.substr(5) : wxT("");
+		pgc->GetVobs()[vobi2]->SetChapters(chapters2);
+	}
+}
+
 void TitlesetManager::MoveSelectedTo(int idx) {
 	if (GetSelected() < 0)
 		return;
@@ -758,6 +798,7 @@ void TitlesetManager::MoveSelectedTo(int idx) {
 	int tsi = DVD::GetTsi(id);
 	int pgci = DVD::GetPgci(id);
 	bool isMenu = DVD::IsMenu(id);
+	int vobi = DVD::GetVobi(id);
 	PgcArray& pgcs = m_dvd->GetPgcArray(tsi, isMenu);
 	
 	int id2 = -1;
@@ -772,6 +813,7 @@ void TitlesetManager::MoveSelectedTo(int idx) {
 	int tsi2 = DVD::GetTsi(id2);
 	int pgci2 = DVD::GetPgci(id2);
 	bool isMenu2 = DVD::IsMenu(id2);
+	int vobi2 = DVD::GetVobi(id2);
 	
 	if (!isMenu && isMenu2) {
 		pgci2 = 0;
@@ -810,6 +852,30 @@ void TitlesetManager::MoveSelectedTo(int idx) {
 				for (int i = pgci2; i <= pgci; i++)
 					m_dvd->UpdateButtonImageFor(tsi, i);
 			}
+		} else if (vobi2 > vobi) {
+			Pgc* pgc = pgcs[pgci];
+			if (vobi == 0)
+				FixVobChapters(pgc, vobi, vobi2);
+			int vobiTmp = vobi;
+			while (vobiTmp != vobi2) {
+				Vob* vob = pgc->GetVobs()[vobiTmp];
+				pgc->GetVobs()[vobiTmp] = pgc->GetVobs()[vobiTmp + 1];
+				pgc->GetVobs()[++vobiTmp] = vob;
+			}
+			if (!isMenu)
+				m_dvd->UpdateButtonImageFor(tsi, pgci);
+		} else if (vobi2 < vobi) {
+			Pgc* pgc = pgcs[pgci];
+			if (vobi2 == 0)
+				FixVobChapters(pgc, vobi2, vobi);
+			int vobiTmp = vobi;
+			while (vobiTmp != vobi2) {
+				Vob* vob = pgc->GetVobs()[vobiTmp];
+				pgc->GetVobs()[vobiTmp] = pgc->GetVobs()[vobiTmp - 1];
+				pgc->GetVobs()[--vobiTmp] = vob;
+			}
+			if (!isMenu)
+				m_dvd->UpdateButtonImageFor(tsi, pgci);
 		} else
 			return;
 	} else {
