@@ -3,7 +3,7 @@
 // Purpose:     Miscellaneous utilities
 // Author:      Alex Thuering
 // Created:		06.04.2008
-// RCS-ID:      $Id: Utils.cpp,v 1.4 2011/04/28 21:35:50 ntalex Exp $
+// RCS-ID:      $Id: Utils.cpp,v 1.8 2012/05/13 17:26:43 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -15,6 +15,8 @@
 #include <wx/dataobj.h>
 #include <wx/datetime.h>
 
+wxRegEx s_timeRE(wxT("^(([[:digit:]]+:)?[[:digit:]][[:digit:]]?:)?[[:digit:]][[:digit:]]?(\\.[[:digit:]]+)?$"));
+
 void CopyXmlToClipboard(wxSvgXmlNode* node, wxString dataFormatId) {
 	wxMemoryOutputStream stream;
 	wxSvgXmlDocument doc;
@@ -27,10 +29,11 @@ void CopyXmlToClipboard(wxSvgXmlNode* node, wxString dataFormatId) {
 	wxCustomDataObject* dataObj = new wxCustomDataObject(format);
 	dataObj->TakeData(bufferSize, buffer);
 	wxClipboardLocker locker;
-	if (!locker)
+	if (!locker) {
 		wxLogError(wxT("Can't open clipboard"));
-	else if (!wxTheClipboard->SetData(dataObj))
+	} else if (!wxTheClipboard->SetData(dataObj)) {
 		wxLogError(wxT("Can't copy xml node to the clipboard"));
+	}
 }
 
 bool GetXmlFromClipboard(wxString dataFormatId, wxSvgXmlDocument& doc) {
@@ -93,12 +96,13 @@ int Sort(wxArrayString& keys, wxArrayString& values, const wxString& selectedKey
  */
 wxString Time2String(long value, bool full) {
 	int t = value / 1000;
-	if (full || (t >= 3600 && value % 1000 > 0))
-		return wxString::Format(wxT("%d:%02d:%02d.%03d"), t/3600, (t/60) % 60, t % 60, value % 1000);
+	int ms = value % 1000;
+	if (full || (t >= 3600 && ms > 0))
+		return wxString::Format(wxT("%d:%02d:%02d.%03d"), t/3600, (t/60) % 60, t % 60, ms);
 	else if (t >= 3600)
 		return wxString::Format(wxT("%d:%02d:%02d"), t/3600, (t/60) % 60, t % 60);
-	else if (value % 1000 > 0)
-		return wxString::Format(wxT("%d:%02d.%03d"), (t/60) % 60, t % 60, value % 1000);
+	else if (ms > 0)
+		return wxString::Format(wxT("%d:%02d.%03d"), (t/60) % 60, t % 60, ms);
 	return wxString::Format(wxT("%d:%02d"), (t/60) % 60, t % 60);
 }
 
@@ -114,7 +118,7 @@ long String2Time(const wxString& value) {
 			result *= 1000;
 			double t = 0;
 			if (val.ToDouble(&t))
-				result += (int) (t*1000);
+				result += lround(t*1000);
 			break;
 		} else {
 			long t = 0;
