@@ -3,7 +3,7 @@
 // Purpose:     Miscellaneous utilities
 // Author:      Alex Thuering
 // Created:		06.04.2008
-// RCS-ID:      $Id: Utils.h,v 1.4 2011/04/28 21:35:50 ntalex Exp $
+// RCS-ID:      $Id: Utils.h,v 1.6 2012/01/22 20:24:27 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -13,6 +13,19 @@
 
 #include <wxSVGXML/svgxml.h>
 #include <wx/arrstr.h>
+#include <wx/regex.h>
+#include <vector>
+
+using namespace std;
+
+#define VECTOR_CLEAR(obj, elem_type) \
+	for (vector<elem_type*>::iterator vectorIt = obj.begin(); vectorIt != obj.end(); vectorIt++) \
+		delete *vectorIt; \
+	obj.clear();
+
+#define VECTOR_COPY(src, dst, elem_type) \
+	for (vector<elem_type*>::const_iterator vectorIt = src.begin(); vectorIt != src.end(); vectorIt++) \
+		dst.push_back(new elem_type(**vectorIt));
 
 const wxString DATAFORMAT_ACTION =  wxT("application/dvdstyler-action");
 const wxString DATAFORMAT_MENU =  wxT("application/dvdstyler-menu");
@@ -45,4 +58,7 @@ wxString Time2String(long value, bool full = false);
  */
 long String2Time(const wxString& value);
 
+/** Regular expression for time values */
+extern wxRegEx s_timeRE;
+
 #endif // UTILS_H
