@@ -3,7 +3,7 @@
 // Purpose:     The video properties dialog
 // Author:      Alex Thuering
 // Created:     09.09.2010
-// RCS-ID:      $Id: VideoPropDlg.cpp,v 1.11 2011/08/01 20:30:08 ntalex Exp $
+// RCS-ID:      $Id: VideoPropDlg.cpp,v 1.15 2012/06/24 22:57:45 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -132,16 +132,7 @@ VideoPropDlg::VideoPropDlg(wxWindow* parent, Vob* vob, AspectRatio aspectRatio):
     
     wxCommandEvent evt;
     OnChangeFormat(evt);
-    
-    if (m_vob->GetDoNotTranscode()) {
-    	m_dstChoice->Enable(false);
-    	m_startCtrl->Enable(false);
-    	m_endCtrl->Enable(false);
-    	m_startBt->Enable(false);
-    	m_endBt->Enable(false);
-    	m_aspectChoice->SetFocus();
-    } else
-    	m_dstChoice->SetFocus();
+    m_dstChoice->SetFocus();
 }
 
 
@@ -174,7 +165,7 @@ void VideoPropDlg::set_properties() {
 	m_startCtrl->SetMinSize(wxSize(x + 10, -1));
 	m_endCtrl->SetMinSize(wxSize(x + 10, -1));
     
-    m_fileNameText->SetLabel(m_vob->GetFilename());
+    m_fileNameText->SetLabel(m_vob->GetFilenameDisplay());
     // duration
 	wxString s = wxT("N/A");
 	if (m_vob->GetDuration()>0) {
@@ -216,16 +207,16 @@ void VideoPropDlg::set_properties() {
     m_filtersCtrl->SetValue(m_vob->GetVideoFilters());
     
     // start time
-    int ms = m_vob->GetStartTime() * 1000;
+    int ms = lround(m_vob->GetStartTime() * 1000);
     ms = ms % 1000;
-	int t = (int) m_vob->GetStartTime();
+	long t = lround(m_vob->GetStartTime());
 	m_startCtrl->SetValue(wxString::Format(wxT("%d:%02d:%02d.%03d"), t/3600, (t/60)%60, t%60, ms));
 	
 	// end time
 	if (m_vob->GetRecordingTime() > 0) {
-		ms = (m_vob->GetStartTime() + m_vob->GetRecordingTime()) * 1000;
+		ms = lround((m_vob->GetStartTime() + m_vob->GetRecordingTime()) * 1000);
 		ms = ms % 1000;
-		t = (int) (m_vob->GetStartTime() + m_vob->GetRecordingTime());
+		t = lround(m_vob->GetStartTime() + m_vob->GetRecordingTime());
 		m_endCtrl->SetValue(wxString::Format(wxT("%d:%02d:%02d.%03d"), t/3600, (t/60) % 60, t%60, ms));
 	}
 }
@@ -237,7 +228,7 @@ void VideoPropDlg::do_layout() {
     wxBoxSizer* mainHSizer = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* mediaSizer = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer* timeSizer = new wxBoxSizer(wxHORIZONTAL);
-    wxFlexGridSizer* propSizer = new wxFlexGridSizer(10, 2, 4, 2);
+    wxFlexGridSizer* propSizer = new wxFlexGridSizer(2, 4, 2);
     wxBoxSizer* fadeOutSizer = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* fadeInSizer = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* propSizer2 = new wxBoxSizer(wxHORIZONTAL);
