@@ -3,7 +3,7 @@
 // Purpose:     The video properties dialog
 // Author:      Alex Thuering
 // Created:     09.09.2010
-// RCS-ID:      $Id: VideoPropDlg.h,v 1.6 2011/02/20 19:03:16 ntalex Exp $
+// RCS-ID:      $Id: VideoPropDlg.h,v 1.9 2011/08/01 20:30:08 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -11,11 +11,12 @@
 #ifndef VIDEO_PROP_DLG_H
 #define VIDEO_PROP_DLG_H
 
-#include "DVD.h"
+#include "Pgc.h"
 #include "mediactrl_ffmpeg.h"
 
 // begin wxGlade: ::dependencies
 #include <wx/spinctrl.h>
+#include <wx/spinbutt.h>
 // end wxGlade
 
 // begin wxGlade: ::extracode
@@ -28,20 +29,17 @@ public:
     // begin wxGlade: VideoPropDlg::ids
     // end wxGlade
 	
-	/**
-	 * Constructor
-	 */
+	/** Constructor */
 	VideoPropDlg(wxWindow* parent, Vob* vob, AspectRatio aspectRatio);
     
-    /**
-	 * Displays dialog
-	 */
+    /** Displays dialog */
 	virtual int ShowModal();
 	
-	/**
-	 * Returns selected aspect ratio
-	 */
-	AspectRatio GetSelectedAspectRatio() { return m_aspectRatio; }
+	/** Returns selected video format */
+    VideoFormat GetVideoFormat();
+    
+	/** Returns selected aspect ratio */
+    AspectRatio GetAspectRatio();
     
 private:
     // begin wxGlade: VideoPropDlg::methods
@@ -72,10 +70,19 @@ private:
     wxSpinCtrl* m_cropLeft;
     wxSpinCtrl* m_cropRight;
     wxSpinCtrl* m_cropBottom;
+    wxStaticText* m_fadeInLabel;
+    wxSpinCtrl* m_fadeInCtrl;
+    wxStaticText* m_fadeInLabel2;
+    wxStaticText* m_fadeOutLabel;
+    wxSpinCtrl* m_fadeOutCtrl;
+    wxStaticText* m_fadeOutLabel2;
+    wxStaticText* m_filtersLabel;
+    wxTextCtrl* m_filtersCtrl;
     MediaCtrlFF* m_mediaCtrl;
     wxSlider* m_slider;
     wxStaticText* m_timeLabel;
     wxTextCtrl* m_timeCtrl;
+    wxSpinButton* m_timeSpinBt;
     wxStaticText* m_startLabel;
     wxTextCtrl* m_startCtrl;
     wxBitmapButton* m_startBt;
@@ -92,9 +99,8 @@ private:
     VideoFormat m_lastFormat;
     vector<int> GetPad();
     vector<int> GetCrop();
-    VideoFormat GetVideoFormat();
-    AspectRatio GetAspectRatio();
     void UpdatePad();
+    void SeekVideo(long pos, bool updateTimeCtrl = true);
     virtual void OnChangeFormat(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnInterlaced(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnChangeAspect(wxCommandEvent &event); // wxGlade: <event_handler>
@@ -105,6 +111,8 @@ private:
     virtual void OnStartBt(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnEndBt(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnOkBt(wxCommandEvent &event); // wxGlade: <event_handler>
+    virtual void OnChangeTime(wxCommandEvent &event); // wxGlade: <event_handler>
+    virtual void OnTimeSpin(wxSpinEvent &event); // wxGlade: <event_handler>
     
 protected:
     DECLARE_EVENT_TABLE()
