@@ -3,7 +3,7 @@
 // Purpose:     The list box to display information about streams in given VOB
 // Author:      Alex Thuering
 // Created:     03.05.2009
-// RCS-ID:      $Id: VobListBox.cpp,v 1.19 2011/08/01 20:30:08 ntalex Exp $
+// RCS-ID:      $Id: VobListBox.cpp,v 1.23 2012/07/24 17:59:30 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -13,6 +13,7 @@
 #include "VideoPropDlg.h"
 #include "AudioPropDlg.h"
 #include "SubtitlePropDlg.h"
+#include "TitlePropDlg.h"
 #include <wx/filedlg.h>
 #include <wx/arrimpl.cpp>
 #include <wxSVG/mediadec_ffmpeg.h>
@@ -28,12 +29,13 @@ WX_DEFINE_OBJARRAY(StringListOfList);
 BEGIN_EVENT_TABLE(VobListBox, wxVListBox)
 	EVT_LEFT_DCLICK(VobListBox::OnDoubleClick)
 	EVT_BUTTON(wxID_ANY, VobListBox::OnButton)
+	EVT_CHOICE(wxID_ANY, VobListBox::OnFormatChange)
 END_EVENT_TABLE()
 
 const int ITEM_HEIGHT = 50;
 #define ITEM_FONT wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT)
 
-VobListBox::VobListBox(wxWindow* parent, wxWindowID id, Vob* vob, PgcArray* pgcs, DVD* dvd): wxVListBox(parent, id) {
+VobListBox::VobListBox(TitlePropDlg* parent, wxWindowID id, Vob* vob, PgcArray* pgcs, DVD* dvd): wxVListBox(parent, id) {
 	m_vob = vob;
 	m_pgcs = pgcs;
 	m_dvd = dvd;
@@ -59,7 +61,9 @@ void VobListBox::OnDrawItem(wxDC& dc, const wxRect& rect, size_t n) const {
 	
 	// info
 	dc.SetFont(ITEM_FONT);
-	dc.SetTextForeground((int)n == GetSelection() ? *wxWHITE : *wxBLACK);
+	dc.SetTextForeground((int)n == GetSelection() ? wxSystemSettings::GetColour(wxSYS_COLOUR_LISTBOXHIGHLIGHTTEXT)
+			: wxSystemSettings::GetColour(wxSYS_COLOUR_LISTBOXTEXT));
+	dc.SetTextForeground(*wxBLACK);
 	wxArrayString& info = m_info[n];
 	RectList& infoRect = m_infoRect[n];
 	for (unsigned int i=0; i<info.GetCount(); i++) {
@@ -89,7 +93,7 @@ void VobListBox::RefreshInfo() {
 		int y = 2;
 		
 		// filename
-		y = AddInfo(n == 0 ? m_vob->GetFilename() : m_vob->GetAudioFilenames()[n-1], n, dc, x, y);
+		y = AddInfo(n == 0 ? m_vob->GetFilenameDisplay() : m_vob->GetAudioFilenames()[n-1], n, dc, x, y);
 			
 		// duration
 		if (n == 0) {
@@ -238,7 +242,7 @@ void VobListBox::AddChoiceCtrl(wxArrayString formats, int selection, int x, int
 	} else
 		ctrl = m_choiceList[choiceIdx];
 	choiceIdx++;
-	ctrl->Enable(enabled);
+	//ctrl->Enable(enabled);
 	ctrl->SetPosition(wxPoint(x, y));
 }
 
@@ -365,13 +369,12 @@ void VobListBox::SetDoNotTranscode(bool value) {
 		case stVIDEO:
 			if (value)
 				m_choiceList[choiceIdx]->SetSelection(vfCOPY-1); // vfNONE is not in the selection list
-			m_choiceList[choiceIdx++]->Enable(!value);
+			choiceIdx++;
 			break;
 		case stAUDIO:
 			if (value)
 				m_choiceList[choiceIdx]->SetSelection(afCOPY);
-			m_choiceList[choiceIdx++]->Enable(!value);
-			choiceIdx++;
+			choiceIdx += 2;
 			break;
 		default:
 			break;
@@ -399,9 +402,13 @@ void VobListBox::SetValues() {
 		switch (stream->GetType()) {
 		case stVIDEO:
 			stream->SetDestinationFormat(m_choiceList[choiceIdx++]->GetSelection() + 1);
+			if (stream->GetVideoFormat() != vfCOPY)
+				m_vob->SetDoNotTranscode(false);
 			break;
 		case stAUDIO:
 			stream->SetDestinationFormat(m_choiceList[choiceIdx++]->GetSelection());
+			if (stream->GetAudioFormat() != afCOPY)
+				m_vob->SetDoNotTranscode(false);
 			if (stream->GetAudioFormat() != afNONE) {
 				if (audioIdx >= m_pgcs->GetAudioLangCodes().size())
 					m_pgcs->GetAudioLangCodes().push_back(m_choiceList[choiceIdx]->GetStringSelection());
@@ -413,6 +420,8 @@ void VobListBox::SetValues() {
 			break;
 		case stSUBTITLE:
 			stream->SetDestinationFormat(m_choiceList[choiceIdx++]->GetSelection());
+			if (stream->GetSubtitleFormat() != sfCOPY)
+				m_vob->SetDoNotTranscode(false);
 			if (stream->GetSubtitleFormat() != sfNONE) {
 				if (m_pgcs->GetSubPictures().size() <= subtitleIdx)
 					m_pgcs->GetSubPictures().push_back(new SubPicture(m_choiceList[choiceIdx]->GetStringSelection()));
@@ -421,6 +430,7 @@ void VobListBox::SetValues() {
 				subtitleIdx++;
 			}
 			choiceIdx++;
+			break;
 		default:
 			break;
 		}
@@ -535,10 +545,9 @@ void VobListBox::ShowPropDialog(unsigned int streamIdx) {
 		SetValues(); // update destination format
 		VideoPropDlg dialog(this, m_vob, m_aspectRatio);
 		if (dialog.ShowModal() == wxID_OK) {
-			if (dialog.GetVideoFormat() > vfCOPY)
-				SetDoNotTranscode(false);
 			SetVideoFormat(dialog.GetVideoFormat() - 1);
 			m_aspectRatio = dialog.GetAspectRatio();
+			UpdateDoNotTranscodeCheck();
 		}
 	} else if (m_vob->GetStreams()[streamIdx]->GetType() == stAUDIO) {
 		// audio
@@ -549,6 +558,7 @@ void VobListBox::ShowPropDialog(unsigned int streamIdx) {
 		if (dialog.ShowModal() == wxID_OK) {
 			SetAudioFormat(streamIdx, dialog.GetAudioFormat());
 			SetAudioLangCode(streamIdx, dialog.GetLangCode());
+			UpdateDoNotTranscodeCheck();
 		}
 	}
 }
@@ -568,3 +578,16 @@ void VobListBox::OnButton(wxCommandEvent& event) {
 	}
 	ShowPropDialog(streamIdx);
 }
+
+void VobListBox::UpdateDoNotTranscodeCheck() {
+	if (m_vob->GetDoNotTranscode()) {
+		SetValues(); // update destination format
+		if (!m_vob->GetDoNotTranscode())
+			((TitlePropDlg*) GetParent())->UpdateDoNotTranscodeCheck();
+	}
+}
+
+/** Processes a format change event */
+void VobListBox::OnFormatChange(wxCommandEvent& event) {
+	UpdateDoNotTranscodeCheck();
+}
