@@ -3,7 +3,7 @@
 // Purpose:     Displays video
 // Author:      Alex Thuering
 // Created:     21.01.2011
-// RCS-ID:      $Id: mediactrl_ffmpeg.cpp,v 1.5 2011/06/15 18:39:28 ntalex Exp $
+// RCS-ID:      $Id: mediactrl_ffmpeg.cpp,v 1.7 2012/06/24 22:30:33 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -49,6 +49,8 @@ protected:
 	void OnPaint(wxPaintEvent &event) {
 		wxPaintDC dc(m_ctrl);
 		if (!m_bitmap.Ok()) {
+			if (!m_image.Ok())
+				return;
 			wxImage img = m_image;
 			double aspect = m_frameAspectRatio;
 			if (m_aspectRatio != arAUTO)
@@ -119,7 +121,12 @@ END_EVENT_TABLE()
 class wxFFMediaBackend: public wxMediaBackendCommonBase {
 public:
 	wxFFMediaBackend(): m_loaded(false), m_evtHandler(NULL) {}
-	virtual ~wxFFMediaBackend() {}
+	virtual ~wxFFMediaBackend() {
+		if (m_evtHandler != NULL) {
+			m_ctrl->PopEventHandler(true);
+			m_evtHandler = NULL;
+		}
+	}
 	
 	virtual bool CreateControl(wxControl* ctrl, wxWindow* parent, wxWindowID id, const wxPoint& pos,
 			const wxSize& size, long style, const wxValidator& validator, const wxString& name) {
