@@ -3,7 +3,7 @@
 // Purpose:     Media Encoder Interface
 // Author:      Alex Thuering
 // Created:     2.05.2010
-// RCS-ID:      $Id: mediaenc.cpp,v 1.3 2011/03/17 20:33:14 ntalex Exp $
+// RCS-ID:      $Id: mediaenc.cpp,v 1.4 2012/06/24 22:30:05 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -45,3 +45,8 @@ double GetFrameAspectRatio(VideoFormat format, AspectRatio aspect) {
 bool isNTSC(VideoFormat format) {
 	return format == vfNTSC || format == vfNTSC_CROPPED || format == vfNTSC_HALF_D1 || format == vfNTSC_VCD; 
 }
+
+/** Returns count of frames per second */
+double GetFps(VideoFormat videoFormat, bool ntscFilm) {
+	return isNTSC(videoFormat) ? ntscFilm ? : 30000.0/1001 : 25;
+}
