@@ -3,7 +3,7 @@
 // Purpose:     Properties dialog (base class for all properties dialogs)
 // Author:      Alex Thuering
 // Created:     18.11.2003
-// RCS-ID:      $Id: PropDlg.cpp,v 1.29 2011/07/24 16:35:22 ntalex Exp $
+// RCS-ID:      $Id: PropDlg.cpp,v 1.38 2012/10/14 13:15:14 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -86,8 +86,7 @@ END_EVENT_TABLE()
 //////////////////////////////////////////////////////////////////////////////
 
 enum {
-  RESET_BT_ID = 7830,
-  CUSTOM_CHOICE_ID,
+  CUSTOM_CHOICE_ID = 7830,
   SELECT_FONT_BT_ID,
   SELECT_COLOR_BT_ID,
   SELECT_FILE_BT_ID,
@@ -99,7 +98,7 @@ enum {
 BEGIN_EVENT_TABLE(wxPropDlg, wxDialog)
   EVT_BUTTON(wxID_CANCEL, wxPropDlg::OnCancel)
   EVT_BUTTON(wxID_OK, wxPropDlg::OnOk)
-  EVT_BUTTON(RESET_BT_ID, wxPropDlg::OnReset)
+  EVT_BUTTON(wxID_APPLY, wxPropDlg::OnReset)
   EVT_BUTTON(SELECT_FONT_BT_ID, wxPropDlg::OnSelectFont)
   EVT_BUTTON(SELECT_COLOR_BT_ID, wxPropDlg::OnSelectColour)
   EVT_BUTTON(SELECT_FILE_BT_ID, wxPropDlg::OnSelectFile)
@@ -129,15 +128,16 @@ enum PropertyType {
 
 wxColourData wxPropDlg::m_colourData;
 
-wxPropDlg::wxPropDlg(wxWindow* parent, wxString title): wxDialog(parent, -1, title, wxDefaultPosition,
-		wxDefaultSize, wxDEFAULT_DIALOG_STYLE| wxRESIZE_BORDER) {
+wxPropDlg::wxPropDlg(wxWindow* parent, wxString title): wxDialog(parent, -1, title, wxDefaultPosition, wxDefaultSize,
+		wxDEFAULT_DIALOG_STYLE| wxRESIZE_BORDER), m_currGroupChecker(NULL), propWindow(NULL) {
 	m_currGroupId = 0;
 	m_updateIndex = -1;
+	m_currObject = NULL;
+	m_currObjectItem = 0;
 	SetIcon(((wxTopLevelWindow*)wxGetTopLevelParent(parent))->GetIcon());
 }
 
-void wxPropDlg::Create(bool resetButton, bool dontShowCheckbox)
-{
+void wxPropDlg::Create(bool resetButton, bool dontShowCheckbox) {
   propWindow = this;
   wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);
   wxBoxSizer* propSizer = new wxBoxSizer(wxVERTICAL);
@@ -160,8 +160,7 @@ void wxPropDlg::Create(bool resetButton, bool dontShowCheckbox)
   }
 }
 
-wxPropDlg::~wxPropDlg()
-{
+wxPropDlg::~wxPropDlg() {
   for (int i=0; i<(int)m_types.GetCount(); i++)
 	switch(m_types[i])
 	{
@@ -174,39 +173,33 @@ wxPropDlg::~wxPropDlg()
 	}
 }
 
-void wxPropDlg::OnCancel(wxCommandEvent &WXUNUSED(event))
-{
+void wxPropDlg::OnCancel(wxCommandEvent &WXUNUSED(event)) {
   EndModal(wxID_CANCEL);
 }
 
-void wxPropDlg::OnOk(wxCommandEvent &WXUNUSED(event))
-{
+void wxPropDlg::OnOk(wxCommandEvent &WXUNUSED(event)) {
   if (SetValues())
     EndModal(wxID_OK);
 }
 
-void wxPropDlg::OnReset(wxCommandEvent& event)
-{
+void wxPropDlg::OnReset(wxCommandEvent& event) {
   Reset();
 }
 
 void wxPropDlg::CreateButtonPane(wxSizer* sizer, bool resetButton, bool dontShowCheckbox) {
-	wxBoxSizer* buttonPane = new wxBoxSizer(wxHORIZONTAL);
+	wxStdDialogButtonSizer* buttonPane = new wxStdDialogButtonSizer();
 	if (dontShowCheckbox)
 		AddCheckProp(buttonPane, _("&Don't show this dialog again"), false);
 	buttonPane->Add(10, 10, dontShowCheckbox ? 0 : 1, wxEXPAND);
-	wxButton* okBt = new wxButton(this, wxID_OK, _("OK"));
-	okBt->SetDefault();
-	buttonPane->Add(okBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
-	buttonPane->Add(8, 0);
-	wxButton* cancelBt = new wxButton(this, wxID_CANCEL, _("Cancel"));
-	buttonPane->Add(cancelBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
+	
+	buttonPane->AddButton(new wxButton(this, wxID_OK, wxEmptyString));
+	buttonPane->AddButton(new wxButton(this, wxID_CANCEL, wxEmptyString));
 	if (resetButton) {
-		buttonPane->Add(8, 0);
-		wxButton* resetBt = new wxButton(this, RESET_BT_ID, _("Reset"));
-		buttonPane->Add(resetBt, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
+		buttonPane->AddButton(new wxButton(this, wxID_APPLY, _("Reset")));
 	}
 	buttonPane->Add(10, 10, dontShowCheckbox ? 0 : 1, wxEXPAND);
+	buttonPane->Realize();
+	buttonPane->GetAffirmativeButton()->SetDefault();
 	sizer->Add(buttonPane, 0, wxEXPAND|wxLEFT|wxRIGHT|wxBOTTOM, 10);
 }
 
@@ -301,7 +294,7 @@ wxSizer* wxPropDlg::AddSpinProp(wxSizer* sizer, const wxString& label, int value
 	wxBoxSizer* pane = NULL;
 	if (width > 0) {
 		pane = new wxBoxSizer(wxHORIZONTAL);
-		pane->Add(ctrl);
+		pane->Add(ctrl, 0, wxALIGN_CENTER_VERTICAL, 0);
 		if (label2.length() > 0) {
 			pane->AddSpacer(4);
 			AddText(pane, label2);
@@ -329,7 +322,7 @@ void wxPropDlg::AddCheckProp(wxSizer* sizer, const wxString& label, bool value,
   wxCheckBox* ctrl = new wxCheckBox(propWindow, id, label);
   ctrl->SetValue(value);
   ctrl->Enable(!readonly);
-  ctrl->SetName(wxString::Format(_T("%d"), m_controls.GetCount()));
+  ctrl->SetName(wxString::Format(_T("%d"), (int) m_controls.GetCount()));
   sizer->Add(ctrl, 0, wxEXPAND, 0);
   m_types.Add(propCHECK);
   m_controls.Add(ctrl);
@@ -349,7 +342,7 @@ void wxPropDlg::AddRadioProp(wxSizer* sizer, const wxString& label, bool value,
 	 wxDefaultPosition, wxDefaultSize, style);
   ctrl->SetValue(value);
   ctrl->Enable(!readonly);
-  ctrl->SetName(wxString::Format(_T("%d"), m_controls.GetCount()));
+  ctrl->SetName(wxString::Format(_T("%d"), (int) m_controls.GetCount()));
   sizer->Add(ctrl, 1, wxEXPAND, 0);
   m_types.Add(propRADIO);
   m_controls.Add(ctrl);
@@ -374,7 +367,7 @@ void wxPropDlg::AddRadioGroupProp(wxSizer* sizer, const wxArrayString& text, int
 		 wxDefaultPosition, wxDefaultSize, i == 0 ? wxRB_GROUP : 0);
 	  ctrl->SetValue(i == value);
 	  ctrl->Enable(!readonly);
-	  ctrl->SetName(wxString::Format(_T("%d_%d"), m_controls.GetCount(), i));
+	  ctrl->SetName(wxString::Format(_T("%d_%d"), (int) m_controls.GetCount(), i));
 	  sizer->Add(ctrl, 1, wxEXPAND, 0);
 	  radioControls->Add(ctrl);
   }
@@ -424,8 +417,14 @@ wxSizer* wxPropDlg::AddBitmapComboProp(wxSizer* sizer, const wxString& label, co
 		sizer->Add(new wxStaticText(propWindow, -1, label), 0, wxALIGN_CENTER_VERTICAL);
 	wxBitmapComboBox* ctrl =
 		new wxBitmapComboBox(propWindow, id, value, wxDefaultPosition, wxDefaultSize, choices, style);
+#if wxCHECK_VERSION(2,9,3)
+	wxAssertHandler_t oldAssertHandler = wxSetAssertHandler(NULL);
+#endif
 	for (unsigned int i = 0; i < bitmaps.size(); i++)
 		ctrl->SetItemBitmap(i, bitmaps[i]);
+#if wxCHECK_VERSION(2,9,3)
+	wxSetAssertHandler(oldAssertHandler);
+#endif
 	ctrl->SetStringSelection(value);
 	wxBoxSizer* pane = NULL;
 	if (width > 0) {
@@ -504,7 +503,7 @@ void wxPropDlg::OnSelectCustomChoice(wxCommandEvent& evt) {
 	long idx = 0;
 	long itemIdx = 0;
 	if (name.BeforeLast(wxT('-')).ToLong(&idx) && name.AfterFirst(wxT('-')).ToLong(&itemIdx)) {
-		wxWindow* ctrl = this->FindWindowByName(wxString::Format(wxT("customCtrlFor%d"), idx));
+		wxWindow* ctrl = this->FindWindowByName(wxString::Format(wxT("customCtrlFor%d"), (int) idx));
 		ctrl->Enable(evt.GetSelection() == itemIdx && ((wxWindow*)evt.GetEventObject())->IsEnabled());
 	}
 }
@@ -634,7 +633,7 @@ void wxPropDlg::AddFontProp(wxSizer* sizer, const wxString& label,
   wxButton* button = new wxButton(propWindow, SELECT_FONT_BT_ID, caption);
   if (caption == _T("..."))
 	button->SetSizeHints(24, 24, 24, 24);
-  button->SetName(wxString::Format(_T("%d"), m_controls.GetCount()));
+  button->SetName(wxString::Format(_T("%d"), (int) m_controls.GetCount()));
   sizer->Add(button);
   m_types.Add(propFONT);
   m_controls.Add(new wxFontData(font));
@@ -655,11 +654,8 @@ void wxPropDlg::OnSelectFont(wxCommandEvent& event)
 	*data = dialog.GetFontData();
 }
 
-void wxPropDlg::AddColourProp(wxSizer* sizer, const wxString& label,
-  wxColour colour, int opacity, wxString caption)
-{
-  if (!sizer) // only update value
-  {
+void wxPropDlg::AddColourProp(wxSizer* sizer, const wxString& label, wxColour colour, int opacity, wxString caption) {
+  if (!sizer) { // only update value
 	((ColourPanel*)m_controls[m_updateIndex])->SetColour(colour);
 	m_updateIndex++;
 	return;
@@ -669,12 +665,13 @@ void wxPropDlg::AddColourProp(wxSizer* sizer, const wxString& label,
   wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
   ColourPanel* panel = new ColourPanel(propWindow, -1);
   panel->SetSize(wxSize(20, 20));
+  panel->SetMinSize(wxSize(20, 20));
   panel->SetColour(colour);
   sizer2->Add(panel, 1, wxALL, 2);
   wxButton* button = new wxButton(propWindow, SELECT_COLOR_BT_ID, caption);
   if (caption == _T("..."))
 	button->SetSizeHints(24, 24, 24, 24);
-  button->SetName(wxString::Format(_T("%d"), m_controls.GetCount()));
+  button->SetName(wxString::Format(wxT("PropColourBt%d"), (int) m_controls.GetCount()));
   sizer2->Add(button);
   m_types.Add(propCOLOUR);
   m_controls.Add(panel);
@@ -686,7 +683,6 @@ void wxPropDlg::AddColourProp(wxSizer* sizer, const wxString& label,
 	  ctrl->SetValue(opacity);
       sizer2->Add(ctrl, 0, wxALL, 2);
       AddText(sizer2, wxT("%"));
-      sizer2->Add(10, 10, 1, wxEXPAND);
       m_types.Add(propSPIN);
 	  m_controls.Add(ctrl);
 	  m_groupIds.Add(m_currGroupId);
@@ -696,7 +692,7 @@ void wxPropDlg::AddColourProp(wxSizer* sizer, const wxString& label,
 
 void wxPropDlg::OnSelectColour(wxCommandEvent& event) {
 	long index = -1;
-	((wxButton*) event.GetEventObject())->GetName().ToLong(&index);
+	((wxButton*) event.GetEventObject())->GetName().Mid(12).ToLong(&index);
 	ColourPanel* panel = ((ColourPanel*) m_controls[index]);
 	m_colourData.SetColour(panel->GetColour());
 	wxColourDialog dialog(propWindow, &m_colourData);
@@ -706,90 +702,95 @@ void wxPropDlg::OnSelectColour(wxCommandEvent& event) {
 	}
 }
 
-void wxPropDlg::AddFileProp(wxSizer* sizer, const wxString& label,
-  const wxString& value, int dlgStyle, wxString caption, wxString wildcard, int id)
-{
-  if (!sizer) // only update value
-  {
-	wxTextCtrl* ctrl = (wxTextCtrl*) m_controls[m_updateIndex];
-	ctrl->SetValue(value);
-	m_updateIndex++;
-	return;
-  }
-  // add text control
-  if (label.length())
-	sizer->Add(new wxStaticText(propWindow, -1, label), 0, wxALIGN_CENTER_VERTICAL);
-  wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
-  wxTextCtrl* ctrl = new wxTextCtrl(propWindow, id, value);
-  sizer2->Add(ctrl, 1, wxEXPAND);
-  wxButton* button = new wxButton(propWindow, SELECT_FILE_BT_ID, caption);
-  int h = ctrl->GetSize().GetHeight() > 24 ? ctrl->GetSize().GetHeight() : 24;
-  if (caption == _T("..."))
-	button->SetSizeHints(h, h, h, h);
-  button->SetName(wxString::Format(_T("%d:%d:%s"), m_controls.GetCount(), dlgStyle, wildcard.c_str()));
-  sizer2->Add(button);
-  sizer->Add(sizer2, 1, wxEXPAND);
-  m_types.Add(propTEXT);
-  m_controls.Add(ctrl);
-  m_groupIds.Add(m_currGroupId);
+void wxPropDlg::AddFileProp(wxSizer* sizer, const wxString& label, const wxString& value, int dlgStyle,
+		wxString buttonLabel, wxString wildcard, int id) {
+	if (!sizer) // only update value
+	{
+		wxTextCtrl* ctrl = (wxTextCtrl*) m_controls[m_updateIndex];
+		ctrl->SetValue(value);
+		m_updateIndex++;
+		return;
+	}
+	// add text control
+	if (label.length())
+		sizer->Add(new wxStaticText(propWindow, -1, label), 0, wxALIGN_CENTER_VERTICAL);
+	wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
+	wxTextCtrl* ctrl = new wxTextCtrl(propWindow, id, value);
+	sizer2->Add(ctrl, 1, wxEXPAND);
+	wxButton* button = new wxButton(propWindow, SELECT_FILE_BT_ID, buttonLabel);
+	int h = ctrl->GetSize().GetHeight() > 24 ? ctrl->GetSize().GetHeight() : 24;
+	if (buttonLabel == _T("..."))
+		button->SetSizeHints(h, h, h, h);
+	button->SetName(wxString::Format(_T("SelectFileButton_%d"), (int) m_controls.GetCount()));
+	button->SetClientData(new wxStringClientData(wxString::Format(_T("%d:%s"), dlgStyle, wildcard.c_str())));
+	sizer2->Add(button);
+	sizer->Add(sizer2, 1, wxEXPAND);
+	m_types.Add(propTEXT);
+	m_controls.Add(ctrl);
+	m_groupIds.Add(m_currGroupId);
 }
 
-void wxPropDlg::OnSelectFile(wxCommandEvent& event)
-{
-  wxString name = ((wxButton*)event.GetEventObject())->GetName();
-  long index = -1;
-  name.BeforeFirst(wxT(':')).ToLong(&index);
-  long dlgStyle = wxFD_DEFAULT_STYLE;
-  name = name.AfterFirst(wxT(':'));
-  name.BeforeFirst(wxT(':')).ToLong(&dlgStyle);
-  name = name.AfterFirst(wxT(':'));
-  wxString wildcard = name.BeforeFirst(wxT(':'));
-  wxTextCtrl* ctrl = ((wxTextCtrl*)m_controls[index]);
-  wxFileDialog dialog(propWindow, _("Choose a file"), ctrl->GetValue(), wxT(""),
-    wildcard, dlgStyle);
-  dialog.SetPath(ctrl->GetValue());
-  if (dialog.ShowModal() == wxID_OK)
-	ctrl->SetValue(dialog.GetPath());
+void wxPropDlg::OnSelectFile(wxCommandEvent& event) {
+	long index = -1;
+	((wxButton*) event.GetEventObject())->GetName().Mid(17).ToLong(&index);
+	wxString clientData = ((wxStringClientData*) ((wxButton*) event.GetEventObject())->GetClientData())->GetData();
+	long dlgStyle = wxFD_DEFAULT_STYLE;
+	clientData.BeforeFirst(wxT(':')).ToLong(&dlgStyle);
+	wxString wildcard = clientData.AfterFirst(wxT(':'));
+	wxTextCtrl* ctrl = ((wxTextCtrl*) m_controls[index]);
+	wxFileDialog dialog(propWindow, _("Choose a file"), ctrl->GetValue(), wxT(""), wildcard, dlgStyle);
+	dialog.SetPath(ctrl->GetValue());
+	if (dialog.ShowModal() == wxID_OK)
+		ctrl->SetValue(dialog.GetPath());
 }
 
-void wxPropDlg::AddDirectoryProp(wxSizer* sizer, const wxString& label,
-  const wxString& value, wxString caption)
-{
-  if (!sizer) // only update value
-  {
-	wxTextCtrl* ctrl = (wxTextCtrl*) m_controls[m_updateIndex];
-	ctrl->SetValue(value);
-	m_updateIndex++;
-	return;
-  }
-  // add text control
-  if (label.length())
-	sizer->Add(new wxStaticText(propWindow, -1, label), 0, wxALIGN_CENTER_VERTICAL);
-  wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
-  wxTextCtrl* ctrl = new wxTextCtrl(propWindow, -1, value);
-  sizer2->Add(ctrl, 1, wxEXPAND, 0);
-  wxButton* button = new wxButton(propWindow, SELECT_DIR_BT_ID, caption);
-  int h = ctrl->GetSize().GetHeight() > 24 ? ctrl->GetSize().GetHeight() : 24;
-  if (caption == _T("..."))
-	button->SetSizeHints(h, h, h, h);
-  button->SetName(wxString::Format(wxT("SelectDirButton_%d"), m_controls.GetCount()));
-  sizer2->Add(button);
-  sizer->Add(sizer2, 1, wxEXPAND);
-  m_types.Add(propTEXT);
-  m_controls.Add(ctrl);
-  m_groupIds.Add(m_currGroupId);
+void wxPropDlg::AddDirectoryProp(wxSizer* sizer, const wxString& label, const wxString& value, wxString buttonLabel) {
+	if (!sizer) { // only update value
+		wxTextCtrl* ctrl = (wxTextCtrl*) m_controls[m_updateIndex];
+		ctrl->SetValue(value);
+		m_updateIndex++;
+		return;
+	}
+	// add text control
+	if (label.length())
+		sizer->Add(new wxStaticText(propWindow, -1, label), 0, wxALIGN_CENTER_VERTICAL);
+	wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
+	wxTextCtrl* ctrl = new wxTextCtrl(propWindow, -1, value);
+	sizer2->Add(ctrl, 1, wxEXPAND, 0);
+	wxButton* button = new wxButton(propWindow, SELECT_DIR_BT_ID, buttonLabel);
+	int h = ctrl->GetSize().GetHeight() > 24 ? ctrl->GetSize().GetHeight() : 24;
+	if (buttonLabel == _T("..."))
+		button->SetSizeHints(h, h, h, h);
+	button->SetName(wxString::Format(wxT("SelectDirButton_%d"), (int) m_controls.GetCount()));
+	sizer2->Add(button);
+	sizer->Add(sizer2, 1, wxEXPAND);
+	m_types.Add(propTEXT);
+	m_controls.Add(ctrl);
+	m_groupIds.Add(m_currGroupId);
 }
 
-void wxPropDlg::OnSelectDir(wxCommandEvent& event)
-{
-  long index = -1;
-  ((wxButton*)event.GetEventObject())->GetName().AfterLast(wxT('_')).ToLong(&index);
-  wxTextCtrl* ctrl = ((wxTextCtrl*)m_controls[index]);
-  wxDirDialog dialog(propWindow, _("Choose a directory"), ctrl->GetValue(),
-    wxDD_DEFAULT_STYLE | wxDD_NEW_DIR_BUTTON);
-  dialog.SetPath(ctrl->GetValue());
-  if (dialog.ShowModal() == wxID_OK)
-	ctrl->SetValue(dialog.GetPath());
+void wxPropDlg::OnSelectDir(wxCommandEvent& event) {
+	long index = -1;
+	((wxButton*) event.GetEventObject())->GetName().AfterLast(wxT('_')).ToLong(&index);
+	wxTextCtrl* ctrl = ((wxTextCtrl*) m_controls[index]);
+	wxDirDialog dialog(propWindow, _("Choose a directory"), ctrl->GetValue(), wxDD_DEFAULT_STYLE | wxDD_NEW_DIR_BUTTON);
+	dialog.SetPath(ctrl->GetValue());
+	if (dialog.ShowModal() == wxID_OK)
+		ctrl->SetValue(dialog.GetPath());
+}
+
+void wxPropDlg::AddButton(wxSizer* sizer, const wxString& label, const long id, const wxString& buttonLabel) {
+	if (!sizer) // skip
+		return;
+	// add text control
+	if (label.length())
+		sizer->Add(new wxStaticText(propWindow, -1, label), 0, wxALIGN_CENTER_VERTICAL);
+	wxButton* bt = new wxButton(this, id, buttonLabel);
+	if (buttonLabel == wxT("...")) {
+		int h = bt->GetSize().GetHeight() > 24 ? bt->GetSize().GetHeight() : 24;
+		bt->SetSizeHints(h, h, h, h);
+	}
+	sizer->Add(bt);
 }
 
 ////////////////////////////// Group /////////////////////////////////////////
@@ -845,8 +846,14 @@ void wxPropDlg::OnGroupCheck(wxCommandEvent& event) {
 	ctrl->GetName().ToLong(&index);
 	int groupId = m_groupIds[index];
 	for (int i = 0; i < (int) m_groupIds.GetCount(); i++)
-		if (i != index && m_groupIds[i] == groupId && m_types[i] != propFONT)
+		if (i != index && m_groupIds[i] == groupId && m_types[i] != propFONT) {
 			((wxWindow*) m_controls[i])->Enable(ctrl->GetValue());
+			if (m_types[i] == propCOLOUR) {
+				wxWindow* button = FindWindowByName(wxString::Format(wxT("PropColourBt%d"), i));
+				if (button != NULL)
+					button->Enable(ctrl->GetValue());
+			}
+		}
 }
 
 wxControl* wxPropDlg::GetControl(int index) {
@@ -918,24 +925,24 @@ bool wxPropDlg::GetBool(int index)
   return true;
 }
 
-void* wxPropDlg::GetClientData(int index)
-{
-  switch(m_types[index])
-  {
-    case propCOMBO:
-	{
-	  wxComboBox* ctrl = (wxComboBox*) m_controls[index];
-	  if (ctrl->GetSelection() >= 0)
-		return ctrl->GetClientData(ctrl->GetSelection());
+void* wxPropDlg::GetClientData(int index) {
+	switch (m_types[index]) {
+	case propCOMBO: {
+		wxComboBox* ctrl = (wxComboBox*) m_controls[index];
+		if (ctrl->GetSelection() >= 0)
+			return ctrl->GetClientData(ctrl->GetSelection());
+		break;
 	}
-	case propCHOICE:
-	{
-	  wxChoice* ctrl = (wxChoice*) m_controls[index];
-      if (ctrl->GetSelection() >= 0)
-		return ctrl->GetClientData(ctrl->GetSelection());
+	case propCHOICE: {
+		wxChoice* ctrl = (wxChoice*) m_controls[index];
+		if (ctrl->GetSelection() >= 0)
+			return ctrl->GetClientData(ctrl->GetSelection());
+		break;
 	}
-  }
-  return NULL;
+	default:
+		break;
+	}
+	return NULL;
 }
 
 wxArrayPtrVoid wxPropDlg::GetGrid(int index)
