@@ -3,7 +3,7 @@
 // Purpose:     Base class for property dialogs
 // Author:      Alex Thuering
 // Created:     18.11.2003
-// RCS-ID:      $Id: PropDlg.h,v 1.25 2011/07/24 16:35:22 ntalex Exp $
+// RCS-ID:      $Id: PropDlg.h,v 1.27 2012/03/29 20:43:10 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -18,6 +18,9 @@
 #if wxCHECK_VERSION(2,9,0)
 #include <wx/fontdata.h>
 #endif
+#if wxCHECK_VERSION(2,9,3)
+#include <wx/colourdata.h>
+#endif
 
 WX_DECLARE_OBJARRAY(wxBitmap, BitmapArray);
 
@@ -142,8 +145,8 @@ protected:
 	  wxColour colour, int opacity = -1, wxString caption = _T("..."));
 	void AddFileProp(wxSizer* sizer, const wxString& label, const wxString& value,
 	  int dlgStyle = wxFD_OPEN, wxString caption = _T("..."), wxString wildcard = _T("*.*"), int id = -1);
-	void AddDirectoryProp(wxSizer* sizer, const wxString& label,
-	  const wxString& value, wxString caption = _T("..."));
+	void AddDirectoryProp(wxSizer* sizer, const wxString& label, const wxString& value, wxString caption = _T("..."));
+	void AddButton(wxSizer* sizer, const wxString& label, const long id, const wxString& buttonLabel = _T("..."));
 
 	wxSizer* BeginGroup(wxSizer* sizer, wxString title = wxEmptyString, wxString checkTitle = wxEmptyString,
 			bool value = false, bool readonly = false, bool vertical = true, int proportion = 0);
