@@ -3,7 +3,7 @@
 // Purpose:     wxThumbnailFactory class
 // Author:      Alex Thuering
 // Created:		15.02.2003
-// RCS-ID:      $Id: ThumbnailFactory.cpp,v 1.22 2011/10/09 12:19:08 ntalex Exp $
+// RCS-ID:      $Id: ThumbnailFactory.cpp,v 1.31 2012/09/14 20:55:25 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -27,10 +27,9 @@
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-thumbnail.h>
 static GnomeThumbnailFactory* thumbnail_factory = NULL;
+#include <wxSVG/mediadec_ffmpeg.h>
 #elif defined(WX_SVG)
 #include <wxSVG/mediadec_ffmpeg.h>
-#elif defined(FFMPEG)
-#include "mediadec_ffmpeg.h"
 #endif
 #define THUMBNAILS_DIR wxGetHomeDir() + wxFILE_SEP_PATH + _T(".thumb")
 
@@ -226,7 +225,9 @@ wxImage wxThumbnailFactory::AddToQueue(ThumbInfo* info) {
 
 ThumbInfo* wxThumbnailFactory::GetThumbInfo(wxString filename, wxWindow* parent, int width, int height) {
 	ThumbInfo* info = new ThumbInfo(filename, parent, width, height);
-	if (wxImage::FindHandler(filename.AfterLast('.').Lower(), -1))
+	if (filename.StartsWith(wxT("concat:")))
+		info->mimeType = _T("concat");
+	else if (wxImage::FindHandler(filename.AfterLast('.').Lower(), BITMAP_TYPE_ANY))
 		info->mimeType = _T("image");
 	else if (wxThumbnails::IsVideo(filename))
 		info->mimeType = _T("video/mpeg");
@@ -243,12 +244,15 @@ ThumbInfo* wxThumbnailFactory::GetThumbInfo(wxString filename, wxWindow* parent,
 	info->mime_type = "video/mpeg";
 	gnome_vfs_file_info_unref(file_info);
 #else
-	info->uri = _T("file://");
-	char* fnameEscaped = escape_string(info->filename.mb_str());
+	info->uri = info->mimeType == _T("concat") ? _T("concat://") : _T("file://");
+	char* fnameEscaped = escape_string(filename.mb_str());
 	info->uri += wxString(fnameEscaped, *wxConvCurrent);
 	if (fnameEscaped)
 		free(fnameEscaped);
-	info->mtime = wxFileName(info->filename).GetModificationTime();
+	wxString fname = filename;
+	if (info->mimeType == _T("concat"))
+		fname = filename.Mid(7).BeforeFirst(wxT('|'));
+	info->mtime = wxFileName(fname).GetModificationTime();
 #endif
 	return info;
 }
@@ -320,7 +324,7 @@ bool wxThumbnailFactory::CanThumbnail(ThumbInfo& info) {
 	if (info.mimeType == _T("unknown"))
 		return false;
 #ifdef GNOME2
-	if (info.mimeType != _T("image"))
+	if (info.mimeType != _T("image") && !info.mimeType.StartsWith(wxT("video/")))
 		return info.uri
 			&& gnome_thumbnail_factory_can_thumbnail(thumbnail_factory, info.uri, info.mime_type, info.mtime);
 #endif
@@ -338,6 +342,50 @@ unsigned char GetBarColor(int x, int y) {
 	return 0xFF;
 }
 
+bool GenerateThumbnailFromVideo(ThumbInfo& info, wxImage& img) {
+	wxString fname = info.filename;
+	if (info.mimeType == _T("concat"))
+		fname = info.filename.Mid(7).BeforeFirst(wxT('|'));
+	wxFfmpegMediaDecoder decoder;
+	if (wxFileExists(fname) && decoder.Load(fname) && decoder.BeginDecode(96, 80)) {
+		if (decoder.GetDuration() > 0) {
+			double dpos = decoder.GetDuration() * 0.05;
+			decoder.SetPosition(dpos > 0.5 ? dpos - 0.5 : dpos);
+			for (int i = 0; i < 100; i++) {
+				img = decoder.GetNextFrame();
+				double dpos1 = decoder.GetPosition();
+				if (dpos1 >= dpos || dpos1 < 0)
+					break;
+			}
+		} else {
+			for (int i = 0; i < 15; i++)
+				img = decoder.GetNextFrame();
+		}
+		if (img.Ok() && decoder.GetFrameAspectRatio() > 0 
+				&& abs(decoder.GetFrameAspectRatio()*100 - img.GetWidth()*100/img.GetHeight()) >= 5) {
+			if (lround(img.GetWidth() / decoder.GetFrameAspectRatio()) > img.GetHeight())
+				img.Rescale(img.GetWidth(), lround(img.GetWidth() / decoder.GetFrameAspectRatio()));
+			else
+				img.Rescale(lround(img.GetHeight() * decoder.GetFrameAspectRatio()), img.GetHeight());
+		}
+	}
+	if (img.Ok()) {
+		unsigned char* data = img.GetData();
+		for (int y = 0; y<img.GetHeight(); y++) {
+			for (int x = 0; x<7; x++) {
+				data[0] = data[1] = data[2] = GetBarColor(x,y);
+				data += 3;
+			}
+			data += (img.GetWidth()-14)*3;
+			for (int x = 0; x<7; x++) {
+				data[0] = data[1] = data[2] = GetBarColor(x,y);
+				data += 3;
+			}
+		}
+	}
+	return img.Ok();
+}
+
 wxImage wxThumbnailFactory::GenerateThumbnail(ThumbInfo& info, bool save) {
 	wxImage img;
 	wxString thumbPath = GetThumbPath(info);
@@ -345,8 +393,14 @@ wxImage wxThumbnailFactory::GenerateThumbnail(ThumbInfo& info, bool save) {
 		img.Create(1, 1);
 		img.SetOption(wxIMAGE_OPTION_MAX_WIDTH, info.width);
 		img.SetOption(wxIMAGE_OPTION_MAX_HEIGHT, info.height);
-		if (wxFileExists(info.filename))
+		if (wxFileExists(info.filename)) {
+			wxLogNull l;
+#if wxCHECK_VERSION(2,9,0)
+			img.LoadFile(info.filename);
+#else
 			LoadImageFile(img, info.filename);
+#endif
+		}
 		if (img.Ok() && info.filename.length() > 4
 				&& info.filename.Mid(info.filename.length() - 4).Lower() == wxT(".jpg")) {
 			int orient = ExifHandler::getOrient(info.filename);
@@ -364,6 +418,8 @@ wxImage wxThumbnailFactory::GenerateThumbnail(ThumbInfo& info, bool save) {
 				break;
 			}
 		}
+	} else if (info.mimeType == _T("concat")) {
+		GenerateThumbnailFromVideo(info, img);
 	} else {
 #ifdef GNOME2
 		info.pixbuf = gnome_thumbnail_factory_generate_thumbnail(thumbnail_factory, info.uri, info.mime_type);
@@ -372,47 +428,15 @@ wxImage wxThumbnailFactory::GenerateThumbnail(ThumbInfo& info, bool save) {
 			wxLogNull log;
 			if (wxFileExists(thumbPath))
 				img.LoadFile(thumbPath);
-		} else
-			gnome_thumbnail_factory_create_failed_thumbnail(thumbnail_factory, info.uri, info.mtime);
-		return img;
+			return img;
+		} else {
+			if (!info.mimeType.StartsWith(wxT("video/")) || !GenerateThumbnailFromVideo(info, img)) {
+				gnome_thumbnail_factory_create_failed_thumbnail(thumbnail_factory, info.uri, info.mtime);
+				return img;
+			}
+		}
 #elif defined(WX_SVG)
-		wxFfmpegMediaDecoder decoder;
-		if (wxFileExists(info.filename) && decoder.Load(info.filename) && decoder.BeginDecode(96, 80)) {
-			if (decoder.GetDuration() > 0) {
-				double dpos = decoder.GetDuration() * 0.05;
-				decoder.SetPosition(dpos > 0.5 ? dpos - 0.5 : dpos);
-				for (int i = 0; i < 100; i++) {
-					img = decoder.GetNextFrame();
-					double dpos1 = decoder.GetPosition();
-					if (dpos1 >= dpos || dpos1 < 0)
-						break;
-				}
-			} else {
-				for (int i = 0; i < 15; i++)
-					img = decoder.GetNextFrame();
-			}
-			if (img.Ok() && decoder.GetFrameAspectRatio() > 0 
-					&& abs(decoder.GetFrameAspectRatio()*100 - img.GetWidth()*100/img.GetHeight()) >= 5) {
-				if (lround(img.GetWidth() / decoder.GetFrameAspectRatio()) > img.GetHeight())
-					img.Rescale(img.GetWidth(), lround(img.GetWidth() / decoder.GetFrameAspectRatio()));
-				else
-					img.Rescale(lround(img.GetHeight() * decoder.GetFrameAspectRatio()), img.GetHeight());
-			}
-		}
-		if (img.Ok()) {
-			unsigned char* data = img.GetData();
-			for (int y = 0; y<img.GetHeight(); y++) {
-				for (int x = 0; x<7; x++) {
-					data[0] = data[1] = data[2] = GetBarColor(x,y);
-					data += 3;
-				}
-				data += (img.GetWidth()-14)*3;
-				for (int x = 0; x<7; x++) {
-					data[0] = data[1] = data[2] = GetBarColor(x,y);
-					data += 3;
-				}
-			}
-		}
+		GenerateThumbnailFromVideo(info, img);
 #endif
 	}
 	if (!save)
@@ -421,7 +445,8 @@ wxImage wxThumbnailFactory::GenerateThumbnail(ThumbInfo& info, bool save) {
 	if (!img.Ok()) {
 		img.Create(1, 1);
 		img.SaveFile(GetThumbPath(info, THUMBNAIL_FAILED), wxBITMAP_TYPE_PNG);
-		wxRemoveFile(thumbPath);
+		if (wxFileExists(thumbPath))
+			wxRemoveFile(thumbPath);
 	} else {
 		if (img.GetWidth() > info.width || img.GetHeight() > info.height) {
 			float scale = (float) info.width / img.GetWidth();
@@ -450,6 +475,7 @@ wxImage wxThumbnailFactory::GenerateThumbnail(ThumbInfo& info, bool save) {
 			gnome_thumbnail_factory_save_thumbnail(thumbnail_factory, pixbuf, info.uri, info.mtime);
 		}
 #else
+		wxLogNull log;
 		img.SaveFile(thumbPath, wxBITMAP_TYPE_PNG);
 #endif
 	}
