@@ -3,7 +3,7 @@
 // Purpose:     wxThumbnails class
 // Author:      Alex Thuering
 // Created:		2.02.2003
-// RCS-ID:      $Id: Thumbnails.cpp,v 1.36 2011/11/22 20:28:11 ntalex Exp $
+// RCS-ID:      $Id: Thumbnails.cpp,v 1.38 2012/09/14 20:55:25 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -19,10 +19,6 @@
 #include "rc/noread.png.h"
 #include "rc/video.png.h"
 
-#if !wxCHECK_VERSION(2,9,0)
-#define wxBITMAP_TYPE_ANY -1
-#endif
-
 /////////////////////////////// wxThumb //////////////////////////////////////
 
 wxBitmap wxThumb::GetBitmap(wxThumbnails* parent, int width, int height) {
@@ -197,12 +193,12 @@ void wxThumbnails::ShowDir(wxString dir, int filter) {
 		wxString caption;
 		if (m_showFileNames)
 			caption = fname.AfterLast(wxFILE_SEP_PATH);
-		if (m_filter == -1 || (m_filter & wxTHUMB_FILTER_VIDEOS && IsVideo(fname))
-				|| (m_filter & wxTHUMB_FILTER_AUDIOS && IsAudio(fname))
-				|| (m_filter & wxTHUMB_FILTER_AUDIOS && IsSubtitle(fname))
-				|| (m_filter & wxTHUMB_FILTER_IMAGES
-						&& wxImage::FindHandler(fname.AfterLast('.').Lower(), wxBITMAP_TYPE_ANY))
-				|| (m_filter & wxTHUMB_FILTER_CUSTOM && IsCustomFile(fname)))
+		if (m_filter == -1 || ((m_filter & wxTHUMB_FILTER_VIDEOS) && IsVideo(fname))
+				|| ((m_filter & wxTHUMB_FILTER_AUDIOS) && IsAudio(fname))
+				|| ((m_filter & wxTHUMB_FILTER_AUDIOS) && IsSubtitle(fname))
+				|| ((m_filter & wxTHUMB_FILTER_IMAGES)
+						&& wxImage::FindHandler(fname.AfterLast('.').Lower(), BITMAP_TYPE_ANY))
+				|| ((m_filter & wxTHUMB_FILTER_CUSTOM) && IsCustomFile(fname)))
 			m_items.Add(new wxThumb(fname, caption));
 		fname = wxFindNextFile();
 	}
